@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2021 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -35,7 +35,7 @@ START_NAMESPACE_DISTRHO
  */
 
 /**
-   Audio port can be used as control voltage (LV2 only).
+   Audio port can be used as control voltage (LV2 and JACK standalone only).
  */
 static const uint32_t kAudioPortIsCV = 0x1;
 
@@ -44,6 +44,32 @@ static const uint32_t kAudioPortIsCV = 0x1;
  */
 static const uint32_t kAudioPortIsSidechain = 0x2;
 
+/**
+   CV port has bipolar range (-1 to +1, or -5 to +5 if scaled).
+   This is merely a hint to tell the host what value range to expect.
+ */
+static const uint32_t kCVPortHasBipolarRange = 0x10;
+
+/**
+   CV port has negative unipolar range (-1 to 0, or -10 to 0 if scaled).
+   This is merely a hint to tell the host what value range to expect.
+ */
+static const uint32_t kCVPortHasNegativeUnipolarRange = 0x20;
+
+/**
+   CV port has positive unipolar range (0 to +1, or 0 to +10 if scaled).
+   This is merely a hint to tell the host what value range to expect.
+ */
+static const uint32_t kCVPortHasPositiveUnipolarRange = 0x40;
+
+/**
+   CV port has scaled range to match real values (-5 to +5v bipolar, +/-10 to 0v unipolar).
+   One other range flag is required if this flag is set.
+
+   When enabled, this makes the port a mod:CVPort, compatible with the MOD Devices platform.
+ */
+static const uint32_t kCVPortHasScaledRange = 0x80;
+
 /** @} */
 
 /* ------------------------------------------------------------------------------------------------------------
@@ -83,12 +109,24 @@ static const uint32_t kParameterIsLogarithmic = 0x08;
    Parameter is of output type.@n
    When unset, parameter is assumed to be of input type.
 
-   Parameter inputs are changed by the host and must not be changed by the plugin.@n
-   The only exception being when changing programs, see Plugin::loadProgram().@n
+   Parameter inputs are changed by the host and typically should not be changed by the plugin.@n
+   One exception is when changing programs, see Plugin::loadProgram().@n
+   The other exception is with parameter change requests, see Plugin::requestParameterValueChange().@n
    Outputs are changed by the plugin and never modified by the host.
+
+   If you are targetting VST2, make sure to order your parameters so that all inputs are before any outputs.
  */
 static const uint32_t kParameterIsOutput = 0x10;
 
+/**
+   Parameter value is a trigger.@n
+   This means the value resets back to its default after each process/run call.@n
+   Cannot be used for output parameters.
+
+   @note Only officially supported under LV2. For other formats DPF simulates the behaviour.
+*/
+static const uint32_t kParameterIsTrigger = 0x20 | kParameterIsBoolean;
+
 /** @} */
 
 /* ------------------------------------------------------------------------------------------------------------
@@ -99,8 +137,61 @@ static const uint32_t kParameterIsOutput = 0x10;
    @{
  */
 
+/**
+   Parameter designation.@n
+   Allows a parameter to be specially designated for a task, like bypass.
+
+   Each designation is unique, there must be only one parameter that uses it.@n
+   The use of designated parameters is completely optional.
+
+   @note Designated parameters have strict ranges.
+   @see ParameterRanges::adjustForDesignation()
+ */
+enum ParameterDesignation {
+   /**
+     Null or unset designation.
+    */
+    kParameterDesignationNull = 0,
+
+   /**
+     Bypass designation.@n
+     When on (> 0.5f), it means the plugin must run in a bypassed state.
+    */
+    kParameterDesignationBypass = 1
+};
+
+/**
+   Predefined Port Groups Ids.
+
+   This enumeration provides a few commonly used groups for convenient use in plugins.
+   For preventing conflicts with user code, negative values are used here.
+   When rolling your own port groups, you MUST start their group ids from 0 and they MUST be sequential.
+
+   @see PortGroup
+ */
+enum PredefinedPortGroupsIds {
+   /**
+     Null or unset port group.
+    */
+    kPortGroupNone = (uint32_t)-1,
+
+   /**
+     A single channel audio group.
+    */
+    kPortGroupMono = (uint32_t)-2,
+
+   /**
+     A 2-channel discrete stereo audio group,
+     where the 1st audio port is the left channel and the 2nd port is the right channel.
+    */
+    kPortGroupStereo = (uint32_t)-3
+};
+
 /**
    Audio Port.
+
+   Can be used as CV port by specifying kAudioPortIsCV in hints,@n
+   but this is only supported in LV2 and JACK standalone formats.
  */
 struct AudioPort {
    /**
@@ -124,13 +215,24 @@ struct AudioPort {
     */
     String symbol;
 
+   /**
+      The group id that this audio/cv port belongs to.
+      No group is assigned by default.
+
+      You can use a group from PredefinedPortGroups or roll your own.@n
+      When rolling your own port groups, you MUST start their group ids from 0 and they MUST be sequential.
+      @see PortGroup, Plugin::initPortGroup
+    */
+    uint32_t groupId;
+
    /**
       Default constructor for a regular audio port.
     */
     AudioPort() noexcept
         : hints(0x0),
           name(),
-          symbol() {}
+          symbol(),
+          groupId(kPortGroupNone) {}
 };
 
 /**
@@ -157,7 +259,7 @@ struct ParameterRanges {
     float max;
 
    /**
-      Default constructor, using 0.0 as minimum, 1.0 as maximum and 0.0 as default.
+      Default constructor, using 0.0 as default, 0.0 as minimum, 1.0 as maximum.
     */
     ParameterRanges() noexcept
         : def(0.0f),
@@ -194,7 +296,7 @@ struct ParameterRanges {
    /**
       Get a fixed value within range.
     */
-    const float& getFixedValue(const float& value) const noexcept
+    float getFixedValue(const float& value) const noexcept
     {
         if (value <= min)
             return min;
@@ -251,6 +353,92 @@ struct ParameterRanges {
     }
 };
 
+/**
+   Parameter enumeration value.@n
+   A string representation of a plugin parameter value.@n
+   Used together can be used to give meaning to parameter values, working as an enumeration.
+ */
+struct ParameterEnumerationValue {
+   /**
+      Parameter value.
+    */
+    float value;
+
+   /**
+      String representation of this value.
+    */
+    String label;
+
+   /**
+      Default constructor, using 0.0 as value and empty label.
+    */
+    ParameterEnumerationValue() noexcept
+        : value(0.0f),
+          label() {}
+
+   /**
+      Constructor using custom values.
+    */
+    ParameterEnumerationValue(float v, const char* l) noexcept
+        : value(v),
+          label(l) {}
+};
+
+/**
+   Collection of parameter enumeration values.@n
+   Handy class to handle the lifetime and count of all enumeration values.
+ */
+struct ParameterEnumerationValues {
+   /**
+      Number of elements allocated in @values.
+    */
+    uint8_t count;
+
+   /**
+      Wherever the host is to be restricted to only use enumeration values.
+
+      @note This mode is only a hint! Not all hosts and plugin formats support this mode.
+    */
+    bool restrictedMode;
+
+   /**
+      Array of @ParameterEnumerationValue items.@n
+      This pointer must be null or have been allocated on the heap with `new ParameterEnumerationValue[count]`.
+    */
+    ParameterEnumerationValue* values;
+
+   /**
+      Default constructor, for zero enumeration values.
+    */
+    ParameterEnumerationValues() noexcept
+        : count(0),
+          restrictedMode(false),
+          values() {}
+
+   /**
+      Constructor using custom values.@n
+      The pointer to @values must have been allocated on the heap with `new`.
+    */
+    ParameterEnumerationValues(uint32_t c, bool r, ParameterEnumerationValue* v) noexcept
+        : count(c),
+          restrictedMode(r),
+          values(v) {}
+
+    ~ParameterEnumerationValues() noexcept
+    {
+        count = 0;
+        restrictedMode = false;
+
+        if (values != nullptr)
+        {
+            delete[] values;
+            values = nullptr;
+        }
+    }
+
+    DISTRHO_DECLARE_NON_COPYABLE(ParameterEnumerationValues)
+};
+
 /**
    Parameter.
  */
@@ -268,6 +456,13 @@ struct Parameter {
     */
     String name;
 
+   /**
+      The short name of this parameter.@n
+      Used when displaying the parameter name in a very limited space.
+      @note This value is optional, the full name is used when the short one is missing.
+    */
+    String shortName;
+
    /**
       The symbol of this parameter.@n
       A parameter symbol is a short restricted name used as a machine and human readable identifier.@n
@@ -283,12 +478,29 @@ struct Parameter {
     */
     String unit;
 
+   /**
+      An extensive description/comment about the parameter.
+      @note This value is optional and only used for LV2.
+    */
+    String description;
+
    /**
       Ranges of this parameter.@n
       The ranges describe the default, minimum and maximum values.
     */
     ParameterRanges ranges;
 
+   /**
+      Enumeration values.@n
+      Can be used to give meaning to parameter values, working as an enumeration.
+    */
+    ParameterEnumerationValues enumValues;
+
+   /**
+      Designation for this parameter.
+    */
+    ParameterDesignation designation;
+
    /**
       MIDI CC to use by default on this parameter.@n
       A value of 0 or 32 (bank change) is considered invalid.@n
@@ -297,16 +509,30 @@ struct Parameter {
     */
     uint8_t midiCC;
 
+   /**
+      The group id that this parameter belongs to.
+      No group is assigned by default.
+
+      You can use a group from PredefinedPortGroups or roll your own.@n
+      When rolling your own port groups, you MUST start their group ids from 0 and they MUST be sequential.
+      @see PortGroup, Plugin::initPortGroup
+    */
+    uint32_t groupId;
+
    /**
       Default constructor for a null parameter.
     */
     Parameter() noexcept
         : hints(0x0),
           name(),
+          shortName(),
           symbol(),
           unit(),
           ranges(),
-          midiCC(0) {}
+          enumValues(),
+          designation(kParameterDesignationNull),
+          midiCC(0),
+          groupId(kPortGroupNone) {}
 
    /**
       Constructor using custom values.
@@ -314,10 +540,74 @@ struct Parameter {
     Parameter(uint32_t h, const char* n, const char* s, const char* u, float def, float min, float max) noexcept
         : hints(h),
           name(n),
+          shortName(),
           symbol(s),
           unit(u),
           ranges(def, min, max),
-          midiCC(0) {}
+          enumValues(),
+          designation(kParameterDesignationNull),
+          midiCC(0),
+          groupId(kPortGroupNone) {}
+
+   /**
+      Initialize a parameter for a specific designation.
+    */
+    void initDesignation(ParameterDesignation d) noexcept
+    {
+        designation = d;
+
+        switch (d)
+        {
+        case kParameterDesignationNull:
+            break;
+        case kParameterDesignationBypass:
+            hints      = kParameterIsAutomable|kParameterIsBoolean|kParameterIsInteger;
+            name       = "Bypass";
+            shortName  = "Bypass";
+            symbol     = "dpf_bypass";
+            unit       = "";
+            midiCC     = 0;
+            groupId    = kPortGroupNone;
+            ranges.def = 0.0f;
+            ranges.min = 0.0f;
+            ranges.max = 1.0f;
+            break;
+        }
+    }
+};
+
+/**
+   Port Group.@n
+   Allows to group together audio/cv ports or parameters.
+
+   Each unique group MUST have an unique symbol and a name.
+   A group can be applied to both inputs and outputs (at the same time).
+   The same group cannot be used in audio ports and parameters.
+
+   An audio port group logically combines ports which should be considered part of the same stream.@n
+   For example, two audio ports in a group may form a stereo stream.
+
+   A parameter group provides meta-data to the host to indicate that some parameters belong together.
+
+   The use of port groups is completely optional.
+
+   @see Plugin::initPortGroup, AudioPort::group, Parameter::group
+ */
+struct PortGroup {
+   /**
+      The name of this port group.@n
+      A port group name can contain any character, but hosts might have a hard time with non-ascii ones.@n
+      The name doesn't have to be unique within a plugin instance, but it's recommended.
+    */
+    String name;
+
+   /**
+      The symbol of this port group.@n
+      A port group symbol is a short restricted name used as a machine and human readable identifier.@n
+      The first character must be one of _, a-z or A-Z and subsequent characters can be from _, a-z, A-Z and 0-9.
+      @note Port group symbols MUST be unique within a plugin instance.
+    */
+    String symbol;
 };
 
 /**
@@ -352,7 +642,7 @@ struct MidiEvent {
    The @a playing and @a frame values are always valid.@n
    BBT values are only valid when @a bbt.valid is true.
 
-   This struct is inspired by the JACK Transport API.
+   This struct is inspired by the [JACK Transport API](https://jackaudio.org/api/structjack__position__t.html).
  */
 struct TimePosition {
    /**
@@ -391,10 +681,11 @@ struct TimePosition {
 
        /**
           Current tick within beat.@n
-          Should always be > 0 and <= @a ticksPerBeat.@n
+          Should always be >= 0 and < @a ticksPerBeat.@n
           The first tick is tick '0'.
+          @note Fraction part of tick is only available on some plugin formats.
         */
-        int32_t tick;
+        double tick;
 
        /**
           Number of ticks that have elapsed between frame 0 and the first beat of the current measure.
@@ -412,7 +703,7 @@ struct TimePosition {
         float beatType;
 
        /**
-          Number of ticks within a bar.@n
+          Number of ticks within a beat.@n
           Usually a moderately large integer with many denominators, such as 1920.0.
         */
         double ticksPerBeat;
@@ -435,6 +726,22 @@ struct TimePosition {
               beatType(0.0f),
               ticksPerBeat(0.0),
               beatsPerMinute(0.0) {}
+
+       /**
+          Reinitialize this position using the default null initialization.
+        */
+        void clear() noexcept
+        {
+            valid = false;
+            bar = 0;
+            beat = 0;
+            tick = 0;
+            barStartTick = 0.0;
+            beatsPerBar = 0.0f;
+            beatType = 0.0f;
+            ticksPerBeat = 0.0;
+            beatsPerMinute = 0.0;
+        }
     } bbt;
 
    /**
@@ -444,6 +751,16 @@ struct TimePosition {
         : playing(false),
           frame(0),
           bbt() {}
+
+   /**
+      Reinitialize this position using the default null initialization.
+    */
+    void clear() noexcept
+    {
+        playing  = false;
+        frame = 0;
+        bbt.clear();
+    }
 };
 
 /** @} */
@@ -539,13 +856,28 @@ public:
       Write a MIDI output event.@n
       This function must only be called during run().@n
       Returns false when the host buffer is full, in which case do not call this again until the next run().
-      @note This function is not implemented yet!@n
-            It's here so that developers can prepare MIDI plugins in advance.@n
-            If you plan to use this, please report to DPF authos so it can be implemented.
     */
     bool writeMidiEvent(const MidiEvent& midiEvent) noexcept;
 #endif
 
+#if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+   /**
+      Check if parameter value change requests will work with the current plugin host.
+      @note This function is only available if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST is enabled.
+      @see requestParameterValueChange(uint32_t, float)
+    */
+    bool canRequestParameterValueChanges() const noexcept;
+
+   /**
+      Request a parameter value change from the host.
+      If successful, this function will automatically trigger a parameter update on the UI side as well.
+      This function can fail, for example if the host is busy with the parameter for read-only automation.
+      Some hosts simply do not have this functionality, which can be verified with canRequestParameterValueChanges().
+      @note This function is only available if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST is enabled.
+    */
+    bool requestParameterValueChange(uint32_t index, float value) noexcept;
+#endif
+
 protected:
    /* --------------------------------------------------------------------------------------------------------
     * Information */
@@ -613,6 +945,13 @@ protected:
     */
     virtual void initParameter(uint32_t index, Parameter& parameter) = 0;
 
+   /**
+      Initialize the port group @a groupId.@n
+      This function will be called once,
+      shortly after the plugin is created and all audio ports and parameters have been enumerated.
+    */
+    virtual void initPortGroup(uint32_t groupId, PortGroup& portGroup);
+
 #if DISTRHO_PLUGIN_WANT_PROGRAMS
    /**
       Set the name of the program @a index.@n
@@ -631,6 +970,13 @@ protected:
     virtual void initState(uint32_t index, String& stateKey, String& defaultStateValue) = 0;
 #endif
 
+#if DISTRHO_PLUGIN_WANT_STATEFILES
+   /**
+      TODO API under construction
+    */
+    virtual bool isStateFile(uint32_t index) = 0;
+#endif
+
    /* --------------------------------------------------------------------------------------------------------
     * Internal data */
 
@@ -661,7 +1007,7 @@ protected:
    /**
       Get the value of an internal state.@n
       The host may call this function from any non-realtime context.@n
-      Must be implemented by your plugin class if DISTRHO_PLUGIN_WANT_PROGRAMS or DISTRHO_PLUGIN_WANT_FULL_STATE is enabled.
+      Must be implemented by your plugin class if DISTRHO_PLUGIN_WANT_FULL_STATE is enabled.
       @note The use of this function breaks compatibility with the DSSI format.
     */
     virtual String getState(const char* key) const = 0;
