@@ -19,12 +19,16 @@
 #if defined(DISTRHO_PLUGIN_TARGET_CARLA)
 # include "src/DistrhoPluginCarla.cpp"
 #elif defined(DISTRHO_PLUGIN_TARGET_JACK)
-# include "src/DistrhoPluginJack.cpp"
+# include "src/DistrhoPluginJACK.cpp"
 #elif (defined(DISTRHO_PLUGIN_TARGET_LADSPA) || defined(DISTRHO_PLUGIN_TARGET_DSSI))
 # include "src/DistrhoPluginLADSPA+DSSI.cpp"
 #elif defined(DISTRHO_PLUGIN_TARGET_LV2)
 # include "src/DistrhoPluginLV2.cpp"
 # include "src/DistrhoPluginLV2export.cpp"
-#elif defined(DISTRHO_PLUGIN_TARGET_VST)
-# include "src/DistrhoPluginVST.cpp"
+#elif defined(DISTRHO_PLUGIN_TARGET_VST2)
+# include "src/DistrhoPluginVST2.cpp"
+#elif defined(DISTRHO_PLUGIN_TARGET_VST3)
+# include "src/DistrhoPluginVST3.cpp"
+#else
+# error unsupported format
 #endif
