@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2021 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -19,6 +19,8 @@
 
 #include "../DistrhoPlugin.hpp"
 
+#include <set>
+
 START_NAMESPACE_DISTRHO
 
 // -----------------------------------------------------------------------
@@ -31,6 +33,43 @@ static const uint32_t kMaxMidiEvents = 512;
 
 extern uint32_t d_lastBufferSize;
 extern double   d_lastSampleRate;
+extern bool     d_lastCanRequestParameterValueChanges;
+
+// -----------------------------------------------------------------------
+// DSP callbacks
+
+typedef bool (*writeMidiFunc) (void* ptr, const MidiEvent& midiEvent);
+typedef bool (*requestParameterValueChangeFunc) (void* ptr, uint32_t index, float value);
+
+// -----------------------------------------------------------------------
+// Helpers
+
+struct PortGroupWithId : PortGroup {
+    uint32_t groupId;
+
+    PortGroupWithId()
+        : PortGroup(),
+          groupId(kPortGroupNone) {}
+};
+
+static void fillInPredefinedPortGroupData(const uint32_t groupId, PortGroup& portGroup)
+{
+    switch (groupId)
+    {
+    case kPortGroupNone:
+        portGroup.name.clear();
+        portGroup.symbol.clear();
+        break;
+    case kPortGroupMono:
+        portGroup.name = "Mono";
+        portGroup.symbol = "dpf_mono";
+        break;
+    case kPortGroupStereo:
+        portGroup.name = "Stereo";
+        portGroup.symbol = "dpf_stereo";
+        break;
+    }
+}
 
 // -----------------------------------------------------------------------
 // Plugin private data
@@ -43,8 +82,12 @@ struct Plugin::PrivateData {
 #endif
 
     uint32_t   parameterCount;
+    uint32_t   parameterOffset;
     Parameter* parameters;
 
+    uint32_t         portGroupCount;
+    PortGroupWithId* portGroups;
+
 #if DISTRHO_PLUGIN_WANT_PROGRAMS
     uint32_t programCount;
     String*  programNames;
@@ -64,8 +107,14 @@ struct Plugin::PrivateData {
     TimePosition timePosition;
 #endif
 
+    // Callbacks
+    void*         callbacksPtr;
+    writeMidiFunc writeMidiCallbackFunc;
+    requestParameterValueChangeFunc requestParameterValueChangeCallbackFunc;
+
     uint32_t bufferSize;
     double   sampleRate;
+    bool     canRequestParameterValueChanges;
 
     PrivateData() noexcept
         : isProcessing(false),
@@ -73,7 +122,10 @@ struct Plugin::PrivateData {
           audioPorts(nullptr),
 #endif
           parameterCount(0),
+          parameterOffset(0),
           parameters(nullptr),
+          portGroupCount(0),
+          portGroups(nullptr),
 #if DISTRHO_PLUGIN_WANT_PROGRAMS
           programCount(0),
           programNames(nullptr),
@@ -86,11 +138,31 @@ struct Plugin::PrivateData {
 #if DISTRHO_PLUGIN_WANT_LATENCY
           latency(0),
 #endif
+          callbacksPtr(nullptr),
+          writeMidiCallbackFunc(nullptr),
+          requestParameterValueChangeCallbackFunc(nullptr),
           bufferSize(d_lastBufferSize),
-          sampleRate(d_lastSampleRate)
+          sampleRate(d_lastSampleRate),
+          canRequestParameterValueChanges(d_lastCanRequestParameterValueChanges)
     {
         DISTRHO_SAFE_ASSERT(bufferSize != 0);
         DISTRHO_SAFE_ASSERT(d_isNotZero(sampleRate));
+
+#if defined(DISTRHO_PLUGIN_TARGET_DSSI) || defined(DISTRHO_PLUGIN_TARGET_LV2)
+        parameterOffset += DISTRHO_PLUGIN_NUM_INPUTS + DISTRHO_PLUGIN_NUM_OUTPUTS;
+# if DISTRHO_PLUGIN_WANT_LATENCY
+        parameterOffset += 1;
+# endif
+#endif
+
+#ifdef DISTRHO_PLUGIN_TARGET_LV2
+# if (DISTRHO_PLUGIN_WANT_MIDI_INPUT || DISTRHO_PLUGIN_WANT_TIMEPOS || DISTRHO_PLUGIN_WANT_STATE)
+        parameterOffset += 1;
+# endif
+# if (DISTRHO_PLUGIN_WANT_MIDI_OUTPUT || DISTRHO_PLUGIN_WANT_STATE)
+        parameterOffset += 1;
+# endif
+#endif
     }
 
     ~PrivateData() noexcept
@@ -109,6 +181,12 @@ struct Plugin::PrivateData {
             parameters = nullptr;
         }
 
+        if (portGroups != nullptr)
+        {
+            delete[] portGroups;
+            portGroups = nullptr;
+        }
+
 #if DISTRHO_PLUGIN_WANT_PROGRAMS
         if (programNames != nullptr)
         {
@@ -131,6 +209,26 @@ struct Plugin::PrivateData {
         }
 #endif
     }
+
+#if DISTRHO_PLUGIN_WANT_MIDI_OUTPUT
+    bool writeMidiCallback(const MidiEvent& midiEvent)
+    {
+        if (writeMidiCallbackFunc != nullptr)
+            return writeMidiCallbackFunc(callbacksPtr, midiEvent);
+
+        return false;
+    }
+#endif
+
+#if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+    bool requestParameterValueChangeCallback(const uint32_t index, const float value)
+    {
+        if (requestParameterValueChangeCallbackFunc != nullptr)
+            return requestParameterValueChangeCallbackFunc(callbacksPtr, index, value);
+
+        return false;
+    }
+#endif
 };
 
 // -----------------------------------------------------------------------
@@ -139,7 +237,9 @@ struct Plugin::PrivateData {
 class PluginExporter
 {
 public:
-    PluginExporter()
+    PluginExporter(void* const callbacksPtr,
+                   const writeMidiFunc writeMidiCall,
+                   const requestParameterValueChangeFunc requestParameterValueChangeCall)
         : fPlugin(createPlugin()),
           fData((fPlugin != nullptr) ? fPlugin->pData : nullptr),
           fIsActive(false)
@@ -159,11 +259,42 @@ public:
                 fPlugin->initAudioPort(false, i, fData->audioPorts[j]);
 # endif
         }
-#endif
+#endif // DISTRHO_PLUGIN_NUM_INPUTS+DISTRHO_PLUGIN_NUM_OUTPUTS > 0
 
         for (uint32_t i=0, count=fData->parameterCount; i < count; ++i)
             fPlugin->initParameter(i, fData->parameters[i]);
 
+        {
+            std::set<uint32_t> portGroupIndices;
+
+#if DISTRHO_PLUGIN_NUM_INPUTS+DISTRHO_PLUGIN_NUM_OUTPUTS > 0
+            for (uint32_t i=0; i < DISTRHO_PLUGIN_NUM_INPUTS+DISTRHO_PLUGIN_NUM_OUTPUTS; ++i)
+                portGroupIndices.insert(fData->audioPorts[i].groupId);
+#endif
+            for (uint32_t i=0, count=fData->parameterCount; i < count; ++i)
+                portGroupIndices.insert(fData->parameters[i].groupId);
+
+            portGroupIndices.erase(kPortGroupNone);
+
+            if (const size_t portGroupSize = portGroupIndices.size())
+            {
+                fData->portGroups = new PortGroupWithId[portGroupSize];
+                fData->portGroupCount = portGroupSize;
+
+                uint32_t index = 0;
+                for (std::set<uint32_t>::iterator it = portGroupIndices.begin(); it != portGroupIndices.end(); ++it, ++index)
+                {
+                    PortGroupWithId& portGroup(fData->portGroups[index]);
+                    portGroup.groupId = *it;
+
+                    if (portGroup.groupId < portGroupSize)
+                        fPlugin->initPortGroup(portGroup.groupId, portGroup);
+                    else
+                        fillInPredefinedPortGroupData(portGroup.groupId, portGroup);
+                }
+            }
+        }
+
 #if DISTRHO_PLUGIN_WANT_PROGRAMS
         for (uint32_t i=0, count=fData->programCount; i < count; ++i)
             fPlugin->initProgramName(i, fData->programNames[i]);
@@ -173,6 +304,10 @@ public:
         for (uint32_t i=0, count=fData->stateCount; i < count; ++i)
             fPlugin->initState(i, fData->stateKeys[i], fData->stateDefValues[i]);
 #endif
+
+        fData->callbacksPtr = callbacksPtr;
+        fData->writeMidiCallbackFunc = writeMidiCall;
+        fData->requestParameterValueChangeCallbackFunc = requestParameterValueChangeCall;
     }
 
     ~PluginExporter()
@@ -283,6 +418,13 @@ public:
         return fData->parameterCount;
     }
 
+    uint32_t getParameterOffset() const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr, 0);
+
+        return fData->parameterOffset;
+    }
+
     uint32_t getParameterHints(const uint32_t index) const noexcept
     {
         DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, 0x0);
@@ -290,9 +432,33 @@ public:
         return fData->parameters[index].hints;
     }
 
+    ParameterDesignation getParameterDesignation(const uint32_t index) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, kParameterDesignationNull);
+
+        return fData->parameters[index].designation;
+    }
+
+    bool isParameterInput(const uint32_t index) const noexcept
+    {
+        return (getParameterHints(index) & kParameterIsOutput) == 0x0;
+    }
+
     bool isParameterOutput(const uint32_t index) const noexcept
     {
-        return (getParameterHints(index) & kParameterIsOutput);
+        return (getParameterHints(index) & kParameterIsOutput) != 0x0;
+    }
+
+    bool isParameterOutputOrTrigger(const uint32_t index) const noexcept
+    {
+        const uint32_t hints = getParameterHints(index);
+
+        if (hints & kParameterIsOutput)
+            return true;
+        if ((hints & kParameterIsTrigger) == kParameterIsTrigger)
+            return true;
+
+        return false;
     }
 
     const String& getParameterName(const uint32_t index) const noexcept
@@ -302,6 +468,13 @@ public:
         return fData->parameters[index].name;
     }
 
+    const String& getParameterShortName(const uint32_t index) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, sFallbackString);
+
+        return fData->parameters[index].shortName;
+    }
+
     const String& getParameterSymbol(const uint32_t index) const noexcept
     {
         DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, sFallbackString);
@@ -316,6 +489,20 @@ public:
         return fData->parameters[index].unit;
     }
 
+    const String& getParameterDescription(const uint32_t index) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, sFallbackString);
+
+        return fData->parameters[index].description;
+    }
+
+    const ParameterEnumerationValues& getParameterEnumValues(const uint32_t index) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, sFallbackEnumValues);
+
+        return fData->parameters[index].enumValues;
+    }
+
     const ParameterRanges& getParameterRanges(const uint32_t index) const noexcept
     {
         DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, sFallbackRanges);
@@ -330,6 +517,13 @@ public:
         return fData->parameters[index].midiCC;
     }
 
+    uint32_t getParameterGroupId(const uint32_t index) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->parameterCount, kPortGroupNone);
+
+        return fData->parameters[index].groupId;
+    }
+
     float getParameterValue(const uint32_t index) const
     {
         DISTRHO_SAFE_ASSERT_RETURN(fPlugin != nullptr, 0.0f);
@@ -346,6 +540,42 @@ public:
         fPlugin->setParameterValue(index, value);
     }
 
+    uint32_t getPortGroupCount() const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr, 0);
+
+        return fData->portGroupCount;
+    }
+
+    const PortGroupWithId& getPortGroupById(const uint32_t groupId) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && fData->portGroupCount != 0, sFallbackPortGroup);
+
+        for (uint32_t i=0; i < fData->portGroupCount; ++i)
+        {
+            const PortGroupWithId& portGroup(fData->portGroups[i]);
+
+            if (portGroup.groupId == groupId)
+                return portGroup;
+        }
+
+        return sFallbackPortGroup;
+    }
+
+    const PortGroupWithId& getPortGroupByIndex(const uint32_t index) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->portGroupCount, sFallbackPortGroup);
+
+        return fData->portGroups[index];
+    }
+
+    const String& getPortGroupSymbolForId(const uint32_t groupId) const noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr, sFallbackString);
+
+        return getPortGroupById(groupId).symbol;
+    }
+
 #if DISTRHO_PLUGIN_WANT_PROGRAMS
     uint32_t getProgramCount() const noexcept
     {
@@ -392,6 +622,15 @@ public:
         return fData->stateDefValues[index];
     }
 
+# if DISTRHO_PLUGIN_WANT_STATEFILES
+    bool isStateFile(const uint32_t index) const
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fData != nullptr && index < fData->stateCount, false);
+
+        return fPlugin->isStateFile(index);
+    }
+# endif
+
 # if DISTRHO_PLUGIN_WANT_FULL_STATE
     String getState(const char* key) const
     {
@@ -471,7 +710,7 @@ public:
         }
     }
 
-#if DISTRHO_PLUGIN_IS_SYNTH
+#if DISTRHO_PLUGIN_WANT_MIDI_INPUT
     void run(const float** const inputs, float** const outputs, const uint32_t frames,
              const MidiEvent* const midiEvents, const uint32_t midiEventCount)
     {
@@ -569,12 +808,16 @@ private:
     // -------------------------------------------------------------------
     // Static fallback data, see DistrhoPlugin.cpp
 
-    static const String          sFallbackString;
-    static const AudioPort       sFallbackAudioPort;
-    static const ParameterRanges sFallbackRanges;
+    static const String                     sFallbackString;
+    static const AudioPort                  sFallbackAudioPort;
+    static const ParameterRanges            sFallbackRanges;
+    static const ParameterEnumerationValues sFallbackEnumValues;
+    static const PortGroupWithId            sFallbackPortGroup;
 
     DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(PluginExporter)
+#ifndef DISTRHO_PLUGIN_TARGET_VST3 /* there is no way around this for VST3 */
     DISTRHO_PREVENT_HEAP_ALLOCATION
+#endif
 };
 
 // -----------------------------------------------------------------------
