@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2021 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -16,9 +16,19 @@
 
 #include "DistrhoPluginInternal.hpp"
 
+#if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+# error Cannot use parameter value change request with LADSPA or DSSI
+#endif
 #if DISTRHO_PLUGIN_WANT_MIDI_OUTPUT
 # error Cannot use MIDI Output with LADSPA or DSSI
 #endif
+#if DISTRHO_PLUGIN_WANT_FULL_STATE
+# error Cannot use full state with LADSPA or DSSI
+#endif
+
+#if DISTRHO_PLUGIN_WANT_TIMEPOS && !defined(DISTRHO_NO_WARNINGS)
+# warning LADSPA/DSSI does not support TimePos
+#endif
 
 #ifdef DISTRHO_PLUGIN_TARGET_DSSI
 # include "dssi/dssi.h"
@@ -27,15 +37,11 @@
 # if DISTRHO_PLUGIN_WANT_MIDI_INPUT
 #  error Cannot use MIDI with LADSPA
 # endif
-# if DISTRHO_PLUGIN_WANT_STATE
+# if DISTRHO_PLUGIN_WANT_STATE && !defined(DISTRHO_NO_WARNINGS)
 #  warning LADSPA cannot handle states
 # endif
 #endif
 
-#if DISTRHO_PLUGIN_WANT_TIMEPOS
-# warning LADSPA/DSSI does not support TimePos
-#endif
-
 START_NAMESPACE_DISTRHO
 
 // -----------------------------------------------------------------------
@@ -44,7 +50,8 @@ class PluginLadspaDssi
 {
 public:
     PluginLadspaDssi()
-        : fPortControls(nullptr),
+        : fPlugin(nullptr, nullptr, nullptr),
+          fPortControls(nullptr),
           fLastControlValues(nullptr)
     {
 #if DISTRHO_PLUGIN_NUM_INPUTS > 0
@@ -171,7 +178,7 @@ public:
     {
         // pre-roll
         if (sampleCount == 0)
-            return updateParameterOutputs();
+            return updateParameterOutputsAndTriggers();
 
         // Check for updated parameters
         float curValue;
@@ -183,7 +190,7 @@ public:
 
             curValue = *fPortControls[i];
 
-            if (fLastControlValues[i] != curValue && ! fPlugin.isParameterOutput(i))
+            if (fPlugin.isParameterInput(i) && d_isNotEqual(fLastControlValues[i], curValue))
             {
                 fLastControlValues[i] = curValue;
                 fPlugin.setParameterValue(i, curValue);
@@ -268,7 +275,7 @@ public:
         fPlugin.run(fPortAudioIns, fPortAudioOuts, sampleCount);
 #endif
 
-        updateParameterOutputs();
+        updateParameterOutputsAndTriggers();
 
 #if defined(DISTRHO_PLUGIN_TARGET_DSSI) && ! DISTRHO_PLUGIN_WANT_MIDI_INPUT
         return; // unused
@@ -282,9 +289,9 @@ public:
 # if DISTRHO_PLUGIN_WANT_STATE
     char* dssi_configure(const char* const key, const char* const value)
     {
-        if (std::strncmp(key, DSSI_RESERVED_CONFIGURE_PREFIX, std::strlen(DSSI_RESERVED_CONFIGURE_PREFIX) == 0))
+        if (std::strncmp(key, DSSI_RESERVED_CONFIGURE_PREFIX, std::strlen(DSSI_RESERVED_CONFIGURE_PREFIX)) == 0)
             return nullptr;
-        if (std::strncmp(key, DSSI_GLOBAL_CONFIGURE_PREFIX, std::strlen(DSSI_GLOBAL_CONFIGURE_PREFIX) == 0))
+        if (std::strncmp(key, DSSI_GLOBAL_CONFIGURE_PREFIX, std::strlen(DSSI_GLOBAL_CONFIGURE_PREFIX)) == 0)
             return nullptr;
 
         fPlugin.setState(key, value);
@@ -331,7 +338,12 @@ public:
 
     int dssi_get_midi_controller_for_port(const ulong port) noexcept
     {
-        const uint8_t midiCC = fPlugin.getParameterMidiCC(port);
+        const uint32_t parameterOffset = fPlugin.getParameterOffset();
+
+        if (port > parameterOffset)
+            return DSSI_NONE;
+
+        const uint8_t midiCC = fPlugin.getParameterMidiCC(port-parameterOffset);
 
         if (midiCC == 0 || midiCC == 32 || midiCC >= 0x78)
             return DSSI_NONE;
@@ -366,17 +378,33 @@ private:
 
     // -------------------------------------------------------------------
 
-    void updateParameterOutputs()
+    void updateParameterOutputsAndTriggers()
     {
+        float value;
+
         for (uint32_t i=0, count=fPlugin.getParameterCount(); i < count; ++i)
         {
-            if (! fPlugin.isParameterOutput(i))
-                continue;
+            if (fPlugin.isParameterOutput(i))
+            {
+                value = fLastControlValues[i] = fPlugin.getParameterValue(i);
 
-            fLastControlValues[i] = fPlugin.getParameterValue(i);
+                if (fPortControls[i] != nullptr)
+                    *fPortControls[i] = value;
+            }
+            else if ((fPlugin.getParameterHints(i) & kParameterIsTrigger) == kParameterIsTrigger)
+            {
+                // NOTE: no trigger support in LADSPA control ports, simulate it here
+                value = fPlugin.getParameterRanges(i).def;
 
-            if (fPortControls[i] != nullptr)
-                *fPortControls[i] = fLastControlValues[i];
+                if (d_isEqual(value, fPlugin.getParameterValue(i)))
+                    continue;
+
+                fLastControlValues[i] = value;
+                fPlugin.setParameterValue(i, value);
+
+                if (fPortControls[i] != nullptr)
+                    *fPortControls[i] = value;
+            }
         }
 
 #if DISTRHO_PLUGIN_WANT_LATENCY
@@ -518,15 +546,14 @@ static DSSI_Descriptor sDssiDescriptor = {
 
 // -----------------------------------------------------------------------
 
-class DescriptorInitializer
+static const struct DescriptorInitializer
 {
-public:
     DescriptorInitializer()
     {
         // Create dummy plugin to get data from
         d_lastBufferSize = 512;
         d_lastSampleRate = 44100.0;
-        PluginExporter plugin;
+        const PluginExporter plugin(nullptr, nullptr, nullptr);
         d_lastBufferSize = 0;
         d_lastSampleRate = 0.0;
 
@@ -591,23 +618,23 @@ public:
 
             {
                 const ParameterRanges& ranges(plugin.getParameterRanges(i));
-                const float defValue(ranges.def);
+                const float defValue = ranges.def;
 
                 portRangeHints[port].HintDescriptor = LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE;
                 portRangeHints[port].LowerBound     = ranges.min;
                 portRangeHints[port].UpperBound     = ranges.max;
 
-                if (defValue == 0.0f)
+                /**/ if (d_isZero(defValue))
                     portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_0;
-                else if (defValue == 1.0f)
+                else if (d_isEqual(defValue, 1.0f))
                     portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_1;
-                else if (defValue == 100.0f)
+                else if (d_isEqual(defValue, 100.0f))
                     portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_100;
-                else if (defValue == 440.0f)
+                else if (d_isEqual(defValue, 440.0f))
                     portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_440;
-                else if (ranges.min == defValue)
+                else if (d_isEqual(ranges.min, defValue))
                     portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_MINIMUM;
-                else if (ranges.max == defValue)
+                else if (d_isEqual(ranges.max, defValue))
                     portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_MAXIMUM;
                 else
                 {
@@ -615,7 +642,7 @@ public:
                     const float middleLow   = (ranges.min/2.0f + middleValue/2.0f)/2.0f + middleValue/2.0f;
                     const float middleHigh  = (ranges.max/2.0f + middleValue/2.0f)/2.0f + middleValue/2.0f;
 
-                    if (defValue < middleLow)
+                    /**/ if (defValue < middleLow)
                         portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_LOW;
                     else if (defValue > middleHigh)
                         portRangeHints[port].HintDescriptor |= LADSPA_HINT_DEFAULT_HIGH;
@@ -625,7 +652,7 @@ public:
             }
 
             {
-                const uint32_t hints(plugin.getParameterHints(i));
+                const uint32_t hints = plugin.getParameterHints(i);
 
                 if (hints & kParameterIsBoolean)
                 {
@@ -703,9 +730,7 @@ public:
             sLadspaDescriptor.PortNames = nullptr;
         }
     }
-};
-
-static DescriptorInitializer sDescInit;
+} sDescInit;
 
 // -----------------------------------------------------------------------
 
