@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2021 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -23,12 +23,19 @@
 #include "lv2/instance-access.h"
 #include "lv2/midi.h"
 #include "lv2/options.h"
+#include "lv2/parameters.h"
+#include "lv2/patch.h"
 #include "lv2/state.h"
 #include "lv2/time.h"
 #include "lv2/urid.h"
 #include "lv2/worker.h"
 #include "lv2/lv2_kxstudio_properties.h"
 #include "lv2/lv2_programs.h"
+#include "lv2/control-input-port-change-request.h"
+
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+# include "libmodla.h"
+#endif
 
 #ifdef noexcept
 # undef noexcept
@@ -44,24 +51,43 @@
 # define DISTRHO_PLUGIN_LV2_STATE_PREFIX "urn:distrho:"
 #endif
 
-#define DISTRHO_LV2_USE_EVENTS_IN  (DISTRHO_PLUGIN_WANT_MIDI_INPUT || DISTRHO_PLUGIN_WANT_TIMEPOS || (DISTRHO_PLUGIN_WANT_STATE && DISTRHO_PLUGIN_HAS_UI))
+#define DISTRHO_LV2_USE_EVENTS_IN  (DISTRHO_PLUGIN_WANT_MIDI_INPUT || DISTRHO_PLUGIN_WANT_TIMEPOS || (DISTRHO_PLUGIN_WANT_STATE && DISTRHO_PLUGIN_HAS_UI) || DISTRHO_PLUGIN_WANT_STATEFILES)
 #define DISTRHO_LV2_USE_EVENTS_OUT (DISTRHO_PLUGIN_WANT_MIDI_OUTPUT || (DISTRHO_PLUGIN_WANT_STATE && DISTRHO_PLUGIN_HAS_UI))
 
 START_NAMESPACE_DISTRHO
 
-typedef std::map<const String, String> StringMap;
+typedef std::map<const String, String> StringToStringMap;
+typedef std::map<const LV2_URID, String> UridToStringMap;
+
+#if ! DISTRHO_PLUGIN_WANT_MIDI_OUTPUT
+static const writeMidiFunc writeMidiCallback = nullptr;
+#endif
+#if ! DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+static const requestParameterValueChangeFunc requestParameterValueChangeCallback = nullptr;
+#endif
 
 // -----------------------------------------------------------------------
 
 class PluginLv2
 {
 public:
-    PluginLv2(const double sampleRate, const LV2_URID_Map* const uridMap, const LV2_Worker_Schedule* const worker, const bool usingNominal)
-        : fUsingNominal(usingNominal),
+    PluginLv2(const double sampleRate,
+              const LV2_URID_Map* const uridMap,
+              const LV2_Worker_Schedule* const worker,
+              const LV2_ControlInputPort_Change_Request* const ctrlInPortChangeReq,
+              const bool usingNominal)
+        : fPlugin(this, writeMidiCallback, requestParameterValueChangeCallback),
+          fUsingNominal(usingNominal),
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+          fRunCount(0),
+#endif
           fPortControls(nullptr),
           fLastControlValues(nullptr),
           fSampleRate(sampleRate),
           fURIDs(uridMap),
+#if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+          fCtrlInPortChangeReq(ctrlInPortChangeReq),
+#endif
           fUridMap(uridMap),
           fWorker(worker)
     {
@@ -99,9 +125,6 @@ public:
 #if DISTRHO_LV2_USE_EVENTS_IN
         fPortEventsIn = nullptr;
 #endif
-#if DISTRHO_LV2_USE_EVENTS_OUT
-        fPortEventsOut = nullptr;
-#endif
 #if DISTRHO_PLUGIN_WANT_LATENCY
         fPortLatency = nullptr;
 #endif
@@ -117,6 +140,15 @@ public:
 
                 const String& dkey(fPlugin.getStateKey(i));
                 fStateMap[dkey] = fPlugin.getStateDefaultValue(i);
+
+# if DISTRHO_PLUGIN_WANT_STATEFILES
+                if (fPlugin.isStateFile(i))
+                {
+                    const String dpf_lv2_key(DISTRHO_PLUGIN_URI "#" + dkey);
+                    const LV2_URID urid = uridMap->map(uridMap->handle, dpf_lv2_key.buffer());
+                    fUridStateFileMap[urid] = dkey;
+                }
+# endif
             }
         }
         else
@@ -127,6 +159,11 @@ public:
         // unused
         (void)fWorker;
 #endif
+
+#if ! DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+        // unused
+        (void)ctrlInPortChangeReq;
+#endif
     }
 
     ~PluginLv2()
@@ -156,19 +193,57 @@ public:
 
     // -------------------------------------------------------------------
 
+    bool getPortControlValue(uint32_t index, float& value) const
+    {
+        if (const float* control = fPortControls[index])
+        {
+            switch (fPlugin.getParameterDesignation(index))
+            {
+            default:
+                value = *control;
+                break;
+            case kParameterDesignationBypass:
+                value = 1.0f - *control;
+                break;
+            }
+
+            return true;
+        }
+
+        return false;
+    }
+
+    void setPortControlValue(uint32_t index, float value)
+    {
+        if (float* control = fPortControls[index])
+        {
+            switch (fPlugin.getParameterDesignation(index))
+            {
+            default:
+                *control = value;
+                break;
+            case kParameterDesignationBypass:
+                *control = 1.0f - value;
+                break;
+            }
+        }
+    }
+
+    // -------------------------------------------------------------------
+
     void lv2_activate()
     {
 #if DISTRHO_PLUGIN_WANT_TIMEPOS
-        std::memset(&fTimePosition, 0, sizeof(TimePosition));
+        fTimePosition.clear();
 
-        // hosts may not send all values, resulting on some invalid data
+        // hosts may not send all values, resulting on some invalid data, let's reset everything
         fTimePosition.bbt.bar   = 1;
         fTimePosition.bbt.beat  = 1;
-        fTimePosition.bbt.tick  = 0;
+        fTimePosition.bbt.tick  = 0.0;
         fTimePosition.bbt.barStartTick = 0;
         fTimePosition.bbt.beatsPerBar  = 4;
         fTimePosition.bbt.beatType     = 4;
-        fTimePosition.bbt.ticksPerBeat = 960.0;
+        fTimePosition.bbt.ticksPerBeat = 1920.0;
         fTimePosition.bbt.beatsPerMinute = 120.0;
 #endif
         fPlugin.activate();
@@ -218,7 +293,7 @@ public:
 #if DISTRHO_LV2_USE_EVENTS_OUT
         if (port == index++)
         {
-            fPortEventsOut = (LV2_Atom_Sequence*)dataLocation;
+            fEventsOutData.port = (LV2_Atom_Sequence*)dataLocation;
             return;
         }
 #endif
@@ -291,7 +366,7 @@ public:
                 if (obj->body.otype != fURIDs.timePosition)
                     continue;
 
-                LV2_Atom* bar     = nullptr;
+                LV2_Atom* bar = nullptr;
                 LV2_Atom* barBeat = nullptr;
                 LV2_Atom* beatUnit = nullptr;
                 LV2_Atom* beatsPerBar = nullptr;
@@ -379,8 +454,8 @@ public:
                     if (fLastPositionData.barBeat >= 0.0f)
                     {
                         const double rest = std::fmod(fLastPositionData.barBeat, 1.0f);
-                        fTimePosition.bbt.beat = fLastPositionData.barBeat-rest+1.0;
-                        fTimePosition.bbt.tick = rest*fTimePosition.bbt.ticksPerBeat+0.5;
+                        fTimePosition.bbt.beat = std::round(fLastPositionData.barBeat - rest + 1.0);
+                        fTimePosition.bbt.tick = rest * fTimePosition.bbt.ticksPerBeat;
                     }
                 }
 
@@ -473,16 +548,16 @@ public:
         }
 #endif
 
-        // check for messages from UI
-#if DISTRHO_PLUGIN_WANT_STATE && DISTRHO_PLUGIN_HAS_UI
+        // check for messages from UI or files
+#if DISTRHO_PLUGIN_WANT_STATE && (DISTRHO_PLUGIN_HAS_UI || DISTRHO_PLUGIN_WANT_STATEFILES)
         LV2_ATOM_SEQUENCE_FOREACH(fPortEventsIn, event)
         {
             if (event == nullptr)
                 break;
 
-            if (event->body.type == fURIDs.distrhoState && fWorker != nullptr)
+            if (event->body.type == fURIDs.dpfKeyValue)
             {
-                const void* const data((const void*)(event + 1));
+                const void* const data = (const void*)(event + 1);
 
                 // check if this is our special message
                 if (std::strcmp((const char*)data, "__dpf_ui_data__") == 0)
@@ -490,12 +565,28 @@ public:
                     for (uint32_t i=0, count=fPlugin.getStateCount(); i < count; ++i)
                         fNeededUiSends[i] = true;
                 }
-                else
                 // no, send to DSP as usual
+                else if (fWorker != nullptr)
                 {
-                    fWorker->schedule_work(fWorker->handle, event->body.size, data);
+                    fWorker->schedule_work(fWorker->handle, sizeof(LV2_Atom)+event->body.size, &event->body);
                 }
             }
+# if DISTRHO_PLUGIN_WANT_STATEFILES
+            else if (event->body.type == fURIDs.atomObject && fWorker != nullptr)
+            {
+                const LV2_Atom_Object* const object = (const LV2_Atom_Object*)&event->body;
+
+                const LV2_Atom* property = nullptr;
+                const LV2_Atom* value    = nullptr;
+                lv2_atom_object_get(object, fURIDs.patchProperty, &property, fURIDs.patchValue, &value, nullptr);
+
+                if (property != nullptr && property->type == fURIDs.atomURID &&
+                    value != nullptr && value->type == fURIDs.atomPath)
+                {
+                    fWorker->schedule_work(fWorker->handle, sizeof(LV2_Atom)+event->body.size, &event->body);
+                }
+            }
+# endif
         }
 #endif
 
@@ -504,14 +595,13 @@ public:
 
         for (uint32_t i=0, count=fPlugin.getParameterCount(); i < count; ++i)
         {
-            if (fPortControls[i] == nullptr)
+            if (!getPortControlValue(i, curValue))
                 continue;
 
-            curValue = *fPortControls[i];
-
-            if (fLastControlValues[i] != curValue && ! fPlugin.isParameterOutput(i))
+            if (fPlugin.isParameterInput(i) && d_isNotEqual(fLastControlValues[i], curValue))
             {
                 fLastControlValues[i] = curValue;
+
                 fPlugin.setParameterValue(i, curValue);
             }
         }
@@ -519,12 +609,21 @@ public:
         // Run plugin
         if (sampleCount != 0)
         {
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+            fRunCount = mod_license_run_begin(fRunCount, sampleCount);
+#endif
+
 #if DISTRHO_PLUGIN_WANT_MIDI_INPUT
             fPlugin.run(fPortAudioIns, fPortAudioOuts, sampleCount, fMidiEvents, midiEventCount);
 #else
             fPlugin.run(fPortAudioIns, fPortAudioOuts, sampleCount);
 #endif
 
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+            for (uint32_t i=0; i<DISTRHO_PLUGIN_NUM_OUTPUTS; ++i)
+                mod_license_run_silence(fRunCount, fPortAudioOuts[i], sampleCount, i);
+#endif
+
 #if DISTRHO_PLUGIN_WANT_TIMEPOS
             // update timePos for next callback
             if (d_isNotZero(fLastPositionData.speed))
@@ -557,8 +656,8 @@ public:
                                                               (double)fLastPositionData.beatsPerBar);
 
                         const double rest = std::fmod(fLastPositionData.barBeat, 1.0f);
-                        fTimePosition.bbt.beat = fLastPositionData.barBeat-rest+1.0;
-                        fTimePosition.bbt.tick = rest*fTimePosition.bbt.ticksPerBeat+0.5;
+                        fTimePosition.bbt.beat = std::round(fLastPositionData.barBeat - rest + 1.0);
+                        fTimePosition.bbt.tick = rest * fTimePosition.bbt.ticksPerBeat;
 
                         if (fLastPositionData.bar >= 0)
                         {
@@ -584,67 +683,63 @@ public:
 #endif
         }
 
-        updateParameterOutputs();
+        updateParameterOutputsAndTriggers();
 
 #if DISTRHO_PLUGIN_WANT_STATE && DISTRHO_PLUGIN_HAS_UI
-        const uint32_t capacity = fPortEventsOut->atom.size;
+        fEventsOutData.initIfNeeded(fURIDs.atomSequence);
 
-        uint32_t size, offset = 0;
         LV2_Atom_Event* aev;
-
-        fPortEventsOut->atom.size = sizeof(LV2_Atom_Sequence_Body);
-        fPortEventsOut->atom.type = fURIDs.atomSequence;
-        fPortEventsOut->body.unit = 0;
-        fPortEventsOut->body.pad  = 0;
-
-        // TODO - MIDI Output
+        const uint32_t capacity = fEventsOutData.capacity;
 
         for (uint32_t i=0, count=fPlugin.getStateCount(); i < count; ++i)
         {
             if (! fNeededUiSends[i])
                 continue;
 
-            const String& key = fPlugin.getStateKey(i);
+            const String& curKey(fPlugin.getStateKey(i));
 
-            for (StringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
+            for (StringToStringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
             {
-                const String& curKey = cit->first;
+                const String& key(cit->first);
 
                 if (curKey != key)
                     continue;
 
-                const String& value = cit->second;
+                const String& value(cit->second);
 
                 // set msg size (key + value + separator + 2x null terminator)
-                const size_t msgSize(key.length()+value.length()+3);
+                const size_t msgSize = key.length()+value.length()+3;
 
-                if (sizeof(LV2_Atom_Event) + msgSize > capacity - offset)
+                if (sizeof(LV2_Atom_Event) + msgSize > capacity - fEventsOutData.offset)
+                {
+                    d_stdout("Sending key '%s' to UI failed, out of space", key.buffer());
                     break;
-
-                // reserve msg space
-                char msgBuf[msgSize];
-                std::memset(msgBuf, 0, msgSize);
-
-                // write key and value in atom bufer
-                std::memcpy(msgBuf, key.buffer(), key.length());
-                std::memcpy(msgBuf+(key.length()+1), value.buffer(), value.length());
+                }
 
                 // put data
-                aev = (LV2_Atom_Event*)(LV2_ATOM_CONTENTS(LV2_Atom_Sequence, fPortEventsOut) + offset);
+                aev = (LV2_Atom_Event*)(LV2_ATOM_CONTENTS(LV2_Atom_Sequence, fEventsOutData.port) + fEventsOutData.offset);
                 aev->time.frames = 0;
-                aev->body.type   = fURIDs.distrhoState;
+                aev->body.type   = fURIDs.dpfKeyValue;
                 aev->body.size   = msgSize;
-                std::memcpy(LV2_ATOM_BODY(&aev->body), msgBuf, msgSize-1);
 
-                size    = lv2_atom_pad_size(sizeof(LV2_Atom_Event) + msgSize);
-                offset += size;
-                fPortEventsOut->atom.size += size;
+                uint8_t* const msgBuf = LV2_ATOM_BODY(&aev->body);
+                std::memset(msgBuf, 0, msgSize);
+
+                // write key and value in atom buffer
+                std::memcpy(msgBuf, key.buffer(), key.length()+1);
+                std::memcpy(msgBuf+(key.length()+1), value.buffer(), value.length()+1);
+
+                fEventsOutData.growBy(lv2_atom_pad_size(sizeof(LV2_Atom_Event) + msgSize));
 
                 fNeededUiSends[i] = false;
                 break;
             }
         }
 #endif
+
+#if DISTRHO_LV2_USE_EVENTS_OUT
+        fEventsOutData.endRun();
+#endif
     }
 
     // -------------------------------------------------------------------
@@ -663,7 +758,7 @@ public:
             {
                 if (options[i].type == fURIDs.atomInt)
                 {
-                    const int bufferSize(*(const int*)options[i].value);
+                    const int32_t bufferSize(*(const int32_t*)options[i].value);
                     fPlugin.setBufferSize(bufferSize);
                 }
                 else
@@ -675,7 +770,7 @@ public:
             {
                 if (options[i].type == fURIDs.atomInt)
                 {
-                    const int bufferSize(*(const int*)options[i].value);
+                    const int32_t bufferSize(*(const int32_t*)options[i].value);
                     fPlugin.setBufferSize(bufferSize);
                 }
                 else
@@ -683,11 +778,11 @@ public:
                     d_stderr("Host changed maxBlockLength but with wrong value type");
                 }
             }
-            else if (options[i].key == fUridMap->map(fUridMap->handle, LV2_CORE__sampleRate))
+            else if (options[i].key == fUridMap->map(fUridMap->handle, LV2_PARAMETERS__sampleRate))
             {
-                if (options[i].type == fURIDs.atomDouble)
+                if (options[i].type == fURIDs.atomFloat)
                 {
-                    const double sampleRate(*(const double*)options[i].value);
+                    const float sampleRate(*(const float*)options[i].value);
                     fSampleRate = sampleRate;
                     fPlugin.setSampleRate(sampleRate);
                 }
@@ -735,13 +830,12 @@ public:
 
             fLastControlValues[i] = fPlugin.getParameterValue(i);
 
-            if (fPortControls[i] != nullptr)
-                *fPortControls[i] = fLastControlValues[i];
+            setPortControlValue(i, fLastControlValues[i]);
         }
 
 # if DISTRHO_PLUGIN_WANT_FULL_STATE
         // Update state
-        for (StringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
+        for (StringToStringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
         {
             const String& key = cit->first;
             fStateMap[key] = fPlugin.getState(key);
@@ -757,22 +851,52 @@ public:
     {
 # if DISTRHO_PLUGIN_WANT_FULL_STATE
         // Update current state
-        for (StringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
+        for (StringToStringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
         {
             const String& key = cit->first;
             fStateMap[key] = fPlugin.getState(key);
         }
 # endif
 
-        for (StringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
+        String dpf_lv2_key;
+        LV2_URID urid;
+
+        for (uint32_t i=0, count=fPlugin.getStateCount(); i < count; ++i)
         {
-            const String& key   = cit->first;
-            const String& value = cit->second;
+            const String& curKey(fPlugin.getStateKey(i));
 
-            const String urnKey(DISTRHO_PLUGIN_LV2_STATE_PREFIX + key);
+            for (StringToStringMap::const_iterator cit=fStateMap.begin(), cite=fStateMap.end(); cit != cite; ++cit)
+            {
+                const String& key(cit->first);
 
-            // some hosts need +1 for the null terminator, even though the type is string
-            store(handle, fUridMap->map(fUridMap->handle, urnKey.buffer()), value.buffer(), value.length()+1, fURIDs.atomString, LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE);
+                if (curKey != key)
+                    continue;
+
+                const String& value(cit->second);
+
+# if DISTRHO_PLUGIN_WANT_STATEFILES
+                if (fPlugin.isStateFile(i))
+                {
+                    dpf_lv2_key = DISTRHO_PLUGIN_URI "#";
+                    urid = fURIDs.atomPath;
+                }
+                else
+# endif
+                {
+                    dpf_lv2_key = DISTRHO_PLUGIN_LV2_STATE_PREFIX;
+                    urid = fURIDs.atomString;
+                }
+
+                dpf_lv2_key += key;
+
+                // some hosts need +1 for the null terminator, even though the type is string
+                store(handle,
+                      fUridMap->map(fUridMap->handle, dpf_lv2_key.buffer()),
+                      value.buffer(),
+                      value.length()+1,
+                      urid,
+                      LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE);
+            }
         }
 
         return LV2_STATE_SUCCESS;
@@ -783,23 +907,42 @@ public:
         size_t   size;
         uint32_t type, flags;
 
+        String dpf_lv2_key;
+        LV2_URID urid;
+
         for (uint32_t i=0, count=fPlugin.getStateCount(); i < count; ++i)
         {
             const String& key(fPlugin.getStateKey(i));
-            const String urnKey(DISTRHO_PLUGIN_LV2_STATE_PREFIX + key);
+
+# if DISTRHO_PLUGIN_WANT_STATEFILES
+            if (fPlugin.isStateFile(i))
+            {
+                dpf_lv2_key = DISTRHO_PLUGIN_URI "#";
+                urid = fURIDs.atomPath;
+            }
+            else
+# endif
+            {
+                dpf_lv2_key = DISTRHO_PLUGIN_LV2_STATE_PREFIX;
+                urid = fURIDs.atomString;
+            }
+
+            dpf_lv2_key += key;
 
             size  = 0;
             type  = 0;
             flags = LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE;
-            const void* data = retrieve(handle, fUridMap->map(fUridMap->handle, urnKey.buffer()), &size, &type, &flags);
+            const void* data = retrieve(handle,
+                                        fUridMap->map(fUridMap->handle, dpf_lv2_key.buffer()),
+                                        &size, &type, &flags);
 
             if (data == nullptr || size == 0)
                 continue;
 
-            DISTRHO_SAFE_ASSERT_CONTINUE(type == fURIDs.atomString);
+            DISTRHO_SAFE_ASSERT_CONTINUE(type == urid);
 
-            const char* const value((const char*)data);
-            const std::size_t length(std::strlen(value));
+            const char* const value  = (const char*)data;
+            const std::size_t length = std::strlen(value);
             DISTRHO_SAFE_ASSERT_CONTINUE(length == size || length+1 == size);
 
             setState(key, value);
@@ -817,11 +960,59 @@ public:
 
     LV2_Worker_Status lv2_work(const void* const data)
     {
-        const char* const key((const char*)data);
-        const char* const value(key+std::strlen(key)+1);
+        const LV2_Atom* const eventBody = (const LV2_Atom*)data;
 
-        setState(key, value);
+        if (eventBody->type == fURIDs.dpfKeyValue)
+        {
+            const char* const key   = (const char*)(eventBody + 1);
+            const char* const value = key + (std::strlen(key) + 1U);
 
+            setState(key, value);
+            return LV2_WORKER_SUCCESS;
+        }
+
+# if DISTRHO_PLUGIN_WANT_STATEFILES
+        if (eventBody->type == fURIDs.atomObject)
+        {
+            const LV2_Atom_Object* const object = (const LV2_Atom_Object*)eventBody;
+
+            const LV2_Atom* property = nullptr;
+            const LV2_Atom* value    = nullptr;
+            lv2_atom_object_get(object, fURIDs.patchProperty, &property, fURIDs.patchValue, &value, nullptr);
+            DISTRHO_SAFE_ASSERT_RETURN(property != nullptr, LV2_WORKER_ERR_UNKNOWN);
+            DISTRHO_SAFE_ASSERT_RETURN(property->type == fURIDs.atomURID, LV2_WORKER_ERR_UNKNOWN);
+            DISTRHO_SAFE_ASSERT_RETURN(value != nullptr, LV2_WORKER_ERR_UNKNOWN);
+            DISTRHO_SAFE_ASSERT_RETURN(value->type == fURIDs.atomPath, LV2_WORKER_ERR_UNKNOWN);
+
+            const LV2_URID urid        = ((const LV2_Atom_URID*)property)->body;
+            const char* const filename = (const char*)(value + 1);
+
+            String key;
+
+            try {
+                key = fUridStateFileMap[urid];
+            } DISTRHO_SAFE_EXCEPTION_RETURN("lv2_work fUridStateFileMap[urid]", LV2_WORKER_ERR_UNKNOWN);
+
+            setState(key, filename);
+
+            for (uint32_t i=0, count=fPlugin.getStateCount(); i < count; ++i)
+            {
+                if (fPlugin.getStateKey(i) == key)
+                {
+                    fNeededUiSends[i] = true;
+                    break;
+                }
+            }
+
+            return LV2_WORKER_SUCCESS;
+        }
+# endif
+
+        return LV2_WORKER_ERR_UNKNOWN;
+    }
+
+    LV2_Worker_Status lv2_work_response(uint32_t, const void*)
+    {
         return LV2_WORKER_SUCCESS;
     }
 #endif
@@ -841,6 +1032,10 @@ private:
     PluginExporter fPlugin;
     const bool fUsingNominal; // if false use maxBlockLength
 
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+    uint32_t fRunCount;
+#endif
+
     // LV2 ports
 #if DISTRHO_PLUGIN_NUM_INPUTS > 0
     const float*  fPortAudioIns[DISTRHO_PLUGIN_NUM_INPUTS];
@@ -856,9 +1051,6 @@ private:
 #if DISTRHO_LV2_USE_EVENTS_IN
     LV2_Atom_Sequence* fPortEventsIn;
 #endif
-#if DISTRHO_LV2_USE_EVENTS_OUT
-    LV2_Atom_Sequence* fPortEventsOut;
-#endif
 #if DISTRHO_PLUGIN_WANT_LATENCY
     float* fPortLatency;
 #endif
@@ -895,18 +1087,61 @@ private:
     } fLastPositionData;
 #endif
 
+#if DISTRHO_LV2_USE_EVENTS_OUT
+    struct Lv2EventsOutData {
+        uint32_t capacity, offset;
+        LV2_Atom_Sequence* port;
+
+        Lv2EventsOutData()
+            : capacity(0),
+              offset(0),
+              port(nullptr) {}
+
+        void initIfNeeded(const LV2_URID uridAtomSequence)
+        {
+            if (capacity != 0)
+                return;
+
+            capacity = port->atom.size;
+
+            port->atom.size = sizeof(LV2_Atom_Sequence_Body);
+            port->atom.type = uridAtomSequence;
+            port->body.unit = 0;
+            port->body.pad  = 0;
+        }
+
+        void growBy(const uint32_t size)
+        {
+            offset += size;
+            port->atom.size += size;
+        }
+
+        void endRun()
+        {
+            capacity = 0;
+            offset = 0;
+        }
+
+    } fEventsOutData;
+#endif
+
     // LV2 URIDs
     struct URIDs {
+        const LV2_URID_Map* _uridMap;
         LV2_URID atomBlank;
         LV2_URID atomObject;
         LV2_URID atomDouble;
         LV2_URID atomFloat;
         LV2_URID atomInt;
         LV2_URID atomLong;
+        LV2_URID atomPath;
         LV2_URID atomSequence;
         LV2_URID atomString;
-        LV2_URID distrhoState;
+        LV2_URID atomURID;
+        LV2_URID dpfKeyValue;
         LV2_URID midiEvent;
+        LV2_URID patchProperty;
+        LV2_URID patchValue;
         LV2_URID timePosition;
         LV2_URID timeBar;
         LV2_URID timeBarBeat;
@@ -918,33 +1153,46 @@ private:
         LV2_URID timeSpeed;
 
         URIDs(const LV2_URID_Map* const uridMap)
-            : atomBlank(uridMap->map(uridMap->handle, LV2_ATOM__Blank)),
-              atomObject(uridMap->map(uridMap->handle, LV2_ATOM__Object)),
-              atomDouble(uridMap->map(uridMap->handle, LV2_ATOM__Double)),
-              atomFloat(uridMap->map(uridMap->handle, LV2_ATOM__Float)),
-              atomInt(uridMap->map(uridMap->handle, LV2_ATOM__Int)),
-              atomLong(uridMap->map(uridMap->handle, LV2_ATOM__Long)),
-              atomSequence(uridMap->map(uridMap->handle, LV2_ATOM__Sequence)),
-              atomString(uridMap->map(uridMap->handle, LV2_ATOM__String)),
-              distrhoState(uridMap->map(uridMap->handle, DISTRHO_PLUGIN_LV2_STATE_PREFIX "KeyValueState")),
-              midiEvent(uridMap->map(uridMap->handle, LV2_MIDI__MidiEvent)),
-              timePosition(uridMap->map(uridMap->handle, LV2_TIME__Position)),
-              timeBar(uridMap->map(uridMap->handle, LV2_TIME__bar)),
-              timeBarBeat(uridMap->map(uridMap->handle, LV2_TIME__barBeat)),
-              timeBeatUnit(uridMap->map(uridMap->handle, LV2_TIME__beatUnit)),
-              timeBeatsPerBar(uridMap->map(uridMap->handle, LV2_TIME__beatsPerBar)),
-              timeBeatsPerMinute(uridMap->map(uridMap->handle, LV2_TIME__beatsPerMinute)),
-              timeTicksPerBeat(uridMap->map(uridMap->handle, LV2_KXSTUDIO_PROPERTIES__TimePositionTicksPerBeat)),
-              timeFrame(uridMap->map(uridMap->handle, LV2_TIME__frame)),
-              timeSpeed(uridMap->map(uridMap->handle, LV2_TIME__speed)) {}
+            : _uridMap(uridMap),
+              atomBlank(map(LV2_ATOM__Blank)),
+              atomObject(map(LV2_ATOM__Object)),
+              atomDouble(map(LV2_ATOM__Double)),
+              atomFloat(map(LV2_ATOM__Float)),
+              atomInt(map(LV2_ATOM__Int)),
+              atomLong(map(LV2_ATOM__Long)),
+              atomPath(map(LV2_ATOM__Path)),
+              atomSequence(map(LV2_ATOM__Sequence)),
+              atomString(map(LV2_ATOM__String)),
+              atomURID(map(LV2_ATOM__URID)),
+              dpfKeyValue(map(DISTRHO_PLUGIN_LV2_STATE_PREFIX "KeyValueState")),
+              midiEvent(map(LV2_MIDI__MidiEvent)),
+              patchProperty(map(LV2_PATCH__property)),
+              patchValue(map(LV2_PATCH__value)),
+              timePosition(map(LV2_TIME__Position)),
+              timeBar(map(LV2_TIME__bar)),
+              timeBarBeat(map(LV2_TIME__barBeat)),
+              timeBeatUnit(map(LV2_TIME__beatUnit)),
+              timeBeatsPerBar(map(LV2_TIME__beatsPerBar)),
+              timeBeatsPerMinute(map(LV2_TIME__beatsPerMinute)),
+              timeTicksPerBeat(map(LV2_KXSTUDIO_PROPERTIES__TimePositionTicksPerBeat)),
+              timeFrame(map(LV2_TIME__frame)),
+              timeSpeed(map(LV2_TIME__speed)) {}
+
+        inline LV2_URID map(const char* const uri) const
+        {
+            return _uridMap->map(_uridMap->handle, uri);
+        }
     } fURIDs;
 
     // LV2 features
+#if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+    const LV2_ControlInputPort_Change_Request* const fCtrlInPortChangeReq;
+#endif
     const LV2_URID_Map* const fUridMap;
     const LV2_Worker_Schedule* const fWorker;
 
 #if DISTRHO_PLUGIN_WANT_STATE
-    StringMap fStateMap;
+    StringToStringMap fStateMap;
     bool* fNeededUiSends;
 
     void setState(const char* const key, const char* const newValue)
@@ -956,7 +1204,7 @@ private:
             return;
 
         // check if key already exists
-        for (StringMap::iterator it=fStateMap.begin(), ite=fStateMap.end(); it != ite; ++it)
+        for (StringToStringMap::iterator it=fStateMap.begin(), ite=fStateMap.end(); it != ite; ++it)
         {
             const String& dkey(it->first);
 
@@ -969,19 +1217,28 @@ private:
 
         d_stderr("Failed to find plugin state with key \"%s\"", key);
     }
+
+# if DISTRHO_PLUGIN_WANT_STATEFILES
+    UridToStringMap fUridStateFileMap;
+# endif
 #endif
 
-    void updateParameterOutputs()
+    void updateParameterOutputsAndTriggers()
     {
+        float curValue;
+
         for (uint32_t i=0, count=fPlugin.getParameterCount(); i < count; ++i)
         {
-            if (! fPlugin.isParameterOutput(i))
-                continue;
+            if (fPlugin.isParameterOutput(i))
+            {
+                curValue = fLastControlValues[i] = fPlugin.getParameterValue(i);
 
-            fLastControlValues[i] = fPlugin.getParameterValue(i);
-
-            if (fPortControls[i] != nullptr)
-                *fPortControls[i] = fLastControlValues[i];
+                setPortControlValue(i, curValue);
+            }
+            else if ((fPlugin.getParameterHints(i) & kParameterIsTrigger) == kParameterIsTrigger)
+            {
+                // NOTE: host is responsible for auto-updating control port buffers
+            }
         }
 
 #if DISTRHO_PLUGIN_WANT_LATENCY
@@ -989,6 +1246,52 @@ private:
             *fPortLatency = fPlugin.getLatency();
 #endif
     }
+
+#if DISTRHO_PLUGIN_WANT_PARAMETER_VALUE_CHANGE_REQUEST
+    bool requestParameterValueChange(const uint32_t index, const float value)
+    {
+        if (fCtrlInPortChangeReq == nullptr)
+            return false;
+        return fCtrlInPortChangeReq->request_change(fCtrlInPortChangeReq->handle, index, value);
+    }
+
+    static bool requestParameterValueChangeCallback(void* const ptr, const uint32_t index, const float value)
+    {
+        return (((PluginLv2*)ptr)->requestParameterValueChange(index, value) == 0);
+    }
+#endif
+
+#if DISTRHO_PLUGIN_WANT_MIDI_OUTPUT
+    bool writeMidi(const MidiEvent& midiEvent)
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(fEventsOutData.port != nullptr, false);
+
+        fEventsOutData.initIfNeeded(fURIDs.atomSequence);
+
+        const uint32_t capacity = fEventsOutData.capacity;
+        const uint32_t offset = fEventsOutData.offset;
+
+        if (sizeof(LV2_Atom_Event) + midiEvent.size > capacity - offset)
+            return false;
+
+        LV2_Atom_Event* const aev = (LV2_Atom_Event*)(LV2_ATOM_CONTENTS(LV2_Atom_Sequence, fEventsOutData.port) + offset);
+        aev->time.frames = midiEvent.frame;
+        aev->body.type   = fURIDs.midiEvent;
+        aev->body.size   = midiEvent.size;
+        std::memcpy(LV2_ATOM_BODY(&aev->body),
+                    midiEvent.size > MidiEvent::kDataSize ? midiEvent.dataExt : midiEvent.data,
+                    midiEvent.size);
+
+        fEventsOutData.growBy(lv2_atom_pad_size(sizeof(LV2_Atom_Event) + midiEvent.size));
+
+        return true;
+    }
+
+    static bool writeMidiCallback(void* ptr, const MidiEvent& midiEvent)
+    {
+        return ((PluginLv2*)ptr)->writeMidi(midiEvent);
+    }
+#endif
 };
 
 // -----------------------------------------------------------------------
@@ -998,6 +1301,7 @@ static LV2_Handle lv2_instantiate(const LV2_Descriptor*, double sampleRate, cons
     const LV2_Options_Option* options = nullptr;
     const LV2_URID_Map*       uridMap = nullptr;
     const LV2_Worker_Schedule* worker = nullptr;
+    const LV2_ControlInputPort_Change_Request* ctrlInPortChangeReq = nullptr;
 
     for (int i=0; features[i] != nullptr; ++i)
     {
@@ -1007,6 +1311,8 @@ static LV2_Handle lv2_instantiate(const LV2_Descriptor*, double sampleRate, cons
             uridMap = (const LV2_URID_Map*)features[i]->data;
         else if (std::strcmp(features[i]->URI, LV2_WORKER__schedule) == 0)
             worker = (const LV2_Worker_Schedule*)features[i]->data;
+        else if (std::strcmp(features[i]->URI, LV2_CONTROL_INPUT_PORT_CHANGE_REQUEST_URI) == 0)
+            ctrlInPortChangeReq = (const LV2_ControlInputPort_Change_Request*)features[i]->data;
     }
 
     if (options == nullptr)
@@ -1029,6 +1335,10 @@ static LV2_Handle lv2_instantiate(const LV2_Descriptor*, double sampleRate, cons
     }
 #endif
 
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+    mod_license_check(features, DISTRHO_PLUGIN_URI);
+#endif
+
     d_lastBufferSize = 0;
     bool usingNominal = false;
 
@@ -1066,8 +1376,9 @@ static LV2_Handle lv2_instantiate(const LV2_Descriptor*, double sampleRate, cons
     }
 
     d_lastSampleRate = sampleRate;
+    d_lastCanRequestParameterValueChanges = ctrlInPortChangeReq != nullptr;
 
-    return new PluginLv2(sampleRate, uridMap, worker, usingNominal);
+    return new PluginLv2(sampleRate, uridMap, worker, ctrlInPortChangeReq, usingNominal);
 }
 
 #define instancePtr ((PluginLv2*)instance)
@@ -1140,6 +1451,11 @@ LV2_Worker_Status lv2_work(LV2_Handle instance, LV2_Worker_Respond_Function, LV2
 {
     return instancePtr->lv2_work(data);
 }
+
+LV2_Worker_Status lv2_work_response(LV2_Handle instance, uint32_t size, const void* body)
+{
+    return instancePtr->lv2_work_response(size, body);
+}
 #endif
 
 // -----------------------------------------------------------------------
@@ -1169,7 +1485,7 @@ static const void* lv2_extension_data(const char* uri)
 
 #if DISTRHO_PLUGIN_WANT_STATE
     static const LV2_State_Interface state = { lv2_save, lv2_restore };
-    static const LV2_Worker_Interface worker = { lv2_work, nullptr, nullptr };
+    static const LV2_Worker_Interface worker = { lv2_work, lv2_work_response, nullptr };
 
     if (std::strcmp(uri, LV2_STATE__interface) == 0)
         return &state;
@@ -1188,7 +1504,11 @@ static const void* lv2_extension_data(const char* uri)
         return &directaccess;
 #endif
 
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+    return mod_license_interface(uri);
+#else
     return nullptr;
+#endif
 }
 
 #undef instancePtr
