@@ -24,6 +24,7 @@
 
 #include <assert.h>
 #include <type_traits>
+#include <cstring>
 
 #ifndef RTOSC_PORT_SUGAR
 #define RTOSC_PORT_SUGAR
@@ -48,8 +49,32 @@ struct rtosc_hack_decltype_t
 #define STRINGIFY(a) STRINGIFY2(a)
 
 //Helper for documenting varargs
-#define IMPL(_1,_2,_3,_4,_5,_6,_7,_8,_9,_10,_11,_12,_13,_14,_15,_16,N, ...) N
-#define LAST_IMP(...) IMPL(__VA_ARGS__,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,0,0,0)
+#define IMPL(_1,_2,_3,_4,_5,_6,_7,_8,_9,_10,_11,_12,_13,_14,_15,_16,_17,_18,_19,_20,_21,_22,_23,_24,N, ...) N
+#define LAST_IMP(...) IMPL(__VA_ARGS__,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0)
+#define DOC_IMP24(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x) \
+                a b c d e f g h i j k l m n o p q r s t u v w rDoc(x)
+#define DOC_IMP23(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w) \
+                a b c d e f g h i j k l m n o p q r s t u v rDoc(w)
+#define DOC_IMP22(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v) \
+                a b c d e f g h i j k l m n o p q r s t u rDoc(v)
+#define DOC_IMP21(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u) \
+                a b c d e f g h i j k l m n o p q r s t rDoc(u)
+#define DOC_IMP20(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t) \
+                a b c d e f g h i j k l m n o p q r s rDoc(t)
+#define DOC_IMP19(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s) \
+                a b c d e f g h i j k l m n o p q r rDoc(s)
+#define DOC_IMP18(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r) \
+                a b c d e f g h i j k l m n o p q rDoc(r)
+#define DOC_IMP17(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q) \
+                a b c d e f g h i j k l m n o p rDoc(q)
+#define DOC_IMP16(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p) \
+                a b c d e f g h i j k l m n o rDoc(p)
+#define DOC_IMP15(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o) \
+                a b c d e f g h i j k l m n rDoc(o)
+#define DOC_IMP14(a,b,c,d,e,f,g,h,i,j,k,l,m,n) \
+                a b c d e f g h i j k l m rDoc(n)
+#define DOC_IMP13(a,b,c,d,e,f,g,h,i,j,k,l,m) \
+                a b c d e f g h i j k l rDoc(m)
 #define DOC_IMP12(a,b,c,d,e,f,g,h,i,j,k,l) a b c d e f g h i j k rDoc(l)
 #define DOC_IMP11(a,b,c,d,e,f,g,h,i,j,k)   a b c d e f g h i j rDoc(k)
 #define DOC_IMP10(a,b,c,d,e,f,g,h,i,j)     a b c d e f g h i rDoc(j)
@@ -85,6 +110,7 @@ struct rtosc_hack_decltype_t
 #define rINC_13 14
 #define rINC_14 15
 #define rINC_15 16
+#define rINC_16 17
 
 //Helper for applying macro on varargs
 //arguments: counting offset, macro, macro args
@@ -114,7 +140,48 @@ struct rtosc_hack_decltype_t
     MAC_EACH_I(off, mac, data, LAST_IMP(__VA_ARGS__), __VA_ARGS__)
 #define MAC_EACH(mac, data, ...) MAC_EACH_OFF(0, mac, data, __VA_ARGS__)
 
-//                    1 2 3 4 5 6 7 8 910111213141516
+//                    1 2 3 4 5 6 7 8 9101112131415161718192021222324
+#define OPTIONS_IMP24(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)rOpt(17,r)rOpt(18,s)rOpt(19,t)rOpt(20,u)\
+    rOpt(21,v)rOpt(22,w)rOpt(23,x)
+//                    1 2 3 4 5 6 7 8 91011121314151617181920212223
+#define OPTIONS_IMP23(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)rOpt(17,r)rOpt(18,s)rOpt(19,t)rOpt(20,u)\
+    rOpt(21,v)rOpt(22,w)
+//                    1 2 3 4 5 6 7 8 910111213141516171819202122
+#define OPTIONS_IMP22(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)rOpt(17,r)rOpt(18,s)rOpt(19,t)rOpt(20,u)\
+    rOpt(21,v)
+//                    1 2 3 4 5 6 7 8 9101112131415161718192021
+#define OPTIONS_IMP21(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)rOpt(17,r)rOpt(18,s)rOpt(19,t)rOpt(20,u)
+//                    1 2 3 4 5 6 7 8 91011121314151617181920
+#define OPTIONS_IMP20(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)rOpt(17,r)rOpt(18,s)rOpt(19,t)
+//                    1 2 3 4 5 6 7 8 910111213141516171819
+#define OPTIONS_IMP19(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)rOpt(17,r)rOpt(18,s)
+//                    1 2 3 4 5 6 7 8 9101112131415161718
+#define OPTIONS_IMP18(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)rOpt(17,r)
+#define OPTIONS_IMP17(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q) \
+    rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
+    rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
+    rOpt(14,o)rOpt(15,p)rOpt(16,q)
 #define OPTIONS_IMP16(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p) \
     rOpt(0,a) rOpt(1,b) rOpt(2,c) rOpt(3,d) rOpt(4,e) rOpt(5,f) rOpt(6,g) \
     rOpt(7,h) rOpt(8,i) rOpt(9,j) rOpt(10,k)rOpt(11,l)rOpt(12,m)rOpt(13,n)\
@@ -167,6 +234,16 @@ struct rtosc_hack_decltype_t
 //This can be used to queue up interpolation or parameter regen
 #define rChangeCb
 
+#define rCrossBroadcast(loc, cross) \
+            char part_loc[128]; \
+            strncpy(part_loc, loc, sizeof(part_loc)); \
+            part_loc[sizeof(part_loc) - 1] = '\0'; \
+            char *end = strrchr(part_loc, '/'); \
+            if(end) { \
+                strcpy(&end[1], STRINGIFY(cross)); \
+                data.broadcast(part_loc, "f", obj->cross); \
+            }
+
 //Normal parameters
 #define rParam(name, ...) \
   {STRINGIFY(name) "::c",  rProp(parameter) rMap(min, 0) rMap(max, 127) DOC(__VA_ARGS__), NULL, rParamCb(name)}
@@ -182,15 +259,13 @@ struct rtosc_hack_decltype_t
 //Array operators
 #define rArrayF(name, length, ...) \
 {STRINGIFY(name) "#" STRINGIFY(length) "::f", rProp(parameter) DOC(__VA_ARGS__), NULL, rArrayFCb(name)}
-#define rArray(name, length, ...) \
-{STRINGIFY(name) "#" STRINGIFY(length) "::c:i", rProp(parameter) DOC(__VA_ARGS__), NULL, rArrayCb(name)}
 #define rArrayT(name, length, ...) \
 {STRINGIFY(name) "#" STRINGIFY(length) "::T:F", rProp(parameter) DOC(__VA_ARGS__), NULL, rArrayTCb(name)}
 #define rArrayI(name, length, ...) \
 {STRINGIFY(name) "#" STRINGIFY(length) "::i", rProp(parameter) DOC(__VA_ARGS__), NULL, rArrayICb(name)}
 #define rArrayOption(name, length, ...) \
 {STRINGIFY(name) "#" STRINGIFY(length) "::i:c:S", rProp(parameter) DOC(__VA_ARGS__), NULL, rArrayOptionCb(name)}
-
+#define rArray rArrayI
 
 //Method callback Actions
 #define rAction(name, ...) \
@@ -231,6 +306,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
 //{STRINGIFY(name) ":", rProp(internal), NULL, rRecurPtrCb(name)}
 
 //let this recurring parameter depend on another port
+//the path of the other port must be at the current level or one level above
 #define rEnabledBy(portname) rMap(enabled by, portname)
 #define rEnabledByCondition(cond_name) rEnabledBy(cond_name)
 #define rEnabledCondition(cond_name, condition) \
@@ -278,15 +354,14 @@ template<class T> constexpr T spice(T*t) {return *t;}
 #define rPresets(...) rPresetsAt(0, __VA_ARGS__)
 #define rDefaultDepends(dep_path_) rMap(default depends, dep_path_)
 #define rDefaultMissing "" // macro to denote yet missing default values
-//#define rNoDefaults() ":no defaults\0" //!< this port (and all children) have no defaults
-//! @brief Denote that this port and its children must always be skipped from
-//!        port-walking if a runtime is being given.
-#define rNoWalk rProp(no walk)
+//!< This port and all children have no default values on purpose
+#define rNoDefaults ":no defaults\0"
 
 //Misc properties
 #define rDoc(doc) ":documentation\0=" doc "\0"
 #define rOpt(numeric,symbolic) rMap(map numeric, symbolic)
 #define rOptions(...) OPTIONS(__VA_ARGS__)
+#define rOptionsBound(...) rOptions(__VA_ARGS__), rLinear(0,LAST_IMP(__VA_ARGS__))
 
 //Zest Metadata
 #define rShort(name) ":shortname\0=" name "\0"
@@ -304,14 +379,13 @@ template<class T> constexpr T spice(T*t) {return *t;}
 
 #define rLIMIT(var, convert) \
     if(prop["min"] && var < (decltype(var)) convert(prop["min"])) \
-        var = convert(prop["min"]);\
+        var = (decltype(var)) convert(prop["min"]);\
     if(prop["max"] && var > (decltype(var)) convert(prop["max"])) \
-        var = convert(prop["max"]);
+        var = (decltype(var)) convert(prop["max"]);
 
 #define rTYPE(n) decltype(obj->n)
 
-//#define rAPPLY(n,t) if(obj->n != var) data.reply("/undo_change", "s" #t #t, data.loc, obj->n, var); obj->n = var;
-#define rCAPPLY(getcode, t, setcode) if(getcode != var) data.reply("/undo_change", "s" #t #t, data.loc, getcode, var); setcode;
+#define rCAPPLY(getcode, t, setcode) if((decltype(var))(getcode) != var) data.reply("/undo_change", "s" #t #t, data.loc, static_cast<int>(getcode), var); setcode;
 #define rAPPLY(n,t) rCAPPLY(obj->n, t, obj->n = var)
 
 #define rParamCb(name) rBOIL_BEGIN \
@@ -322,7 +396,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
             rLIMIT(var, atoi) \
             rAPPLY(name, c) \
             data.broadcast(loc, "c", obj->name);\
-            rChangeCb \
+            rChangeCb; \
         } rBOIL_END
 
 #define rParamFCb(name) rBOIL_BEGIN \
@@ -333,7 +407,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
             rLIMIT(var, atof) \
             rAPPLY(name, f) \
             data.broadcast(loc, "f", obj->name);\
-            rChangeCb \
+            rChangeCb; \
         } rBOIL_END
 
 #define rParamICb(name) rBOIL_BEGIN \
@@ -344,14 +418,14 @@ template<class T> constexpr T spice(T*t) {return *t;}
             rLIMIT(var, atoi) \
             rAPPLY(name, i) \
             data.broadcast(loc, "i", obj->name);\
-            rChangeCb \
+            rChangeCb; \
         } rBOIL_END
 
 #define rCOptionCb_(getcode, setcode) { \
             if(!strcmp("", args)) {\
-                data.reply(loc, "i", getcode); \
+                data.reply(loc, "i", static_cast<int>(getcode)); \
             } else if(!strcmp("s", args) || !strcmp("S", args)) { \
-                auto var = \
+                int var = \
                     enum_key(prop, rtosc_argument(msg, 0).s); \
                 /* make sure we have no out-of-bound options */ \
                 assert(!prop["min"] || \
@@ -360,18 +434,18 @@ template<class T> constexpr T spice(T*t) {return *t;}
                        var <= atoi(prop["max"])); \
                 rCAPPLY(getcode, i, setcode) \
                 data.broadcast(loc, "i", getcode); \
-                rChangeCb \
+                rChangeCb; \
             } else {\
-                auto var = \
+                int var = \
                     rtosc_argument(msg, 0).i; \
                 rLIMIT(var, atoi) \
                 rCAPPLY(getcode, i, setcode) \
                 data.broadcast(loc, rtosc_argument_string(msg), getcode);\
-                rChangeCb \
+                rChangeCb; \
             } \
         }
 
-#define rOptionCb_(name) rCOptionCb_(obj->name, obj->name = var)
+#define rOptionCb_(name) rCOptionCb_(obj->name, obj->name = static_cast<std::remove_reference<decltype(obj->name)>::type>(var))
 
 #define rOptionCb(name) rBOIL_BEGIN \
     rOptionCb_(name) \
@@ -389,7 +463,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
             if(obj->name != rtosc_argument(msg, 0).T) { \
                 data.broadcast(loc, args);\
                 obj->name = rtosc_argument(msg, 0).T; \
-                rChangeCb \
+                rChangeCb; \
             } \
         } rBOIL_END
 
@@ -441,17 +515,6 @@ template<class T> constexpr T spice(T*t) {return *t;}
 #define rBOILS_END rBOIL_END
 
 
-#define rArrayCb(name) rBOILS_BEGIN \
-        if(!strcmp("", args)) {\
-            data.reply(loc, "c", obj->name[idx]); \
-        } else { \
-            char var = rtosc_argument(msg, 0).i; \
-            rLIMIT(var, atoi) \
-            rAPPLY(name[idx], c) \
-            data.broadcast(loc, "c", obj->name[idx]);\
-            rChangeCb \
-        } rBOILS_END
-
 #define rArrayFCb(name) rBOILS_BEGIN \
         if(!strcmp("", args)) {\
             data.reply(loc, "f", obj->name[idx]); \
@@ -468,7 +531,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
         } else { \
             if(obj->name[idx] != rtosc_argument(msg, 0).T) { \
                 data.broadcast(loc, args);\
-                rChangeCb \
+                rChangeCb; \
             } \
             obj->name[idx] = rtosc_argument(msg, 0).T; \
         } rBOILS_END
@@ -479,7 +542,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
         } else { \
             if(obj->name[idx].member != rtosc_argument(msg, 0).T) { \
                 data.broadcast(loc, args);\
-                rChangeCb \
+                rChangeCb; \
             } \
             obj->name[idx].member = rtosc_argument(msg, 0).T; \
         } rBOILS_END
@@ -493,7 +556,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
             rLIMIT(var, atoi) \
             rAPPLY(name[idx], i) \
             data.broadcast(loc, "i", obj->name[idx]);\
-            rChangeCb \
+            rChangeCb; \
         } rBOILS_END
 
 
@@ -511,7 +574,7 @@ template<class T> constexpr T spice(T*t) {return *t;}
             strncpy(obj->name, rtosc_argument(msg, 0).s, length-1); \
             obj->name[length-1] = '\0'; \
             data.broadcast(loc, "s", obj->name);\
-            rChangeCb \
+            rChangeCb; \
         } rBOIL_END
 
 
