@@ -51,6 +51,7 @@ struct Ports;
 struct RtData
 {
     RtData(void);
+    virtual ~RtData() = default;
 
     /**
      * Location of where the dispatch routine is currently being called.
@@ -313,7 +314,7 @@ typedef void(*port_walker_t)(const Port*,const char*,const char*,
  * @param walker Callback function
  * @param expand_bundles Whether walking over bundles without subports
  *   invokes walking over each of the bundle's port
- * @param runtime Runtime object corresponding to @param base. If given, checks
+ * @param runtime Runtime object corresponding to @p base . If given, checks
  *   the runtime object will be used stop recursion if the Ports are disabled
  *   (using the "enabled by" property) and it will be passed to the walker
  */
@@ -323,7 +324,34 @@ void walk_ports(const Ports *base,
                 void          *data,
                 port_walker_t  walker,
                 bool expand_bundles = true,
-                void *runtime = NULL);
+                void *runtime = NULL,
+                bool ranges = false);
+
+/**
+   Options for path_search. Examples see path-search tests.
+   @test path-search.cpp
+*/
+enum class path_search_opts
+{
+    /**
+       Return ports in the order they are found. Fastest option.
+       For "a/", "a/b", "a/", this will return "a/", "a/b", "a/".
+    */
+    unmodified,
+    /**
+       Return ports sorted, but not filtered.
+       For "a/", "a/b", "a/", this will return "a/", "a/", "a/b".
+    */
+    sorted,
+    /**
+       Sorted and prefix-filtered: If "a/" is found, don't add any "a/...".
+       Duplicate ports are still returned twice, because recursing into one of
+       them will not help to access the other (and because duplicate ports
+       shall be reported to port-checker).
+       For "a/", "a/b", "a/", this will return "a/", "a/".
+    */
+    sorted_and_unique_prefix
+};
 
 /**
  * Returns paths and metadata of all direct children of a port, or of the port
@@ -332,7 +360,7 @@ void walk_ports(const Ports *base,
  * @param root see @p m
  * @param str location under @p root to look up port, or empty-string to search
  *            directly at @p root
- * @param needle Only port names starting with this sting are returned (use
+ * @param needle Only port names starting with this string are returned (use
  *            empty-string or nullptr to match everything)
  * @param types A buffer where the OSC type string is being written
  * @param max_types should be @p max_args +1 for best performance,
@@ -340,10 +368,14 @@ void walk_ports(const Ports *base,
  * @param args An array where the argument values are being written
  * @param max_args maximum number of arguments in array at @p args. Should be
  *   greater or equal than 2 * (maximum no. of child ports of your app's ports).
+ * @param reply_with_query Whether the reply message shall contain the two
+ *   query strings in the beginning
  */
 void path_search(const rtosc::Ports& root, const char *str, const char *needle,
                  char *types, std::size_t max_types,
-                 rtosc_arg_t* args, std::size_t max_args);
+                 rtosc_arg_t* args, std::size_t max_args,
+                 path_search_opts opts = path_search_opts::sorted_and_unique_prefix,
+                 bool reply_with_query = false);
 
 /**
  * Returns a messages of all paths and metadata of all direct children of a
@@ -364,11 +396,15 @@ void path_search(const rtosc::Ports& root, const char *str, const char *needle,
  *     app's ports.
  * @param msgbuf Buffer for the reply message
  * @param bufsize Size of the message buffer @p msgbuf
+ * @param reply_with_query Whether the reply message shall contain the two
+ *   query strings in the beginning
  * @return The length of the reply message (0 means error)
  */
 std::size_t path_search(const rtosc::Ports& root, const char *m,
                         std::size_t max_ports,
-                        char *msgbuf, std::size_t bufsize);
+                        char *msgbuf, std::size_t bufsize,
+                        path_search_opts opts = path_search_opts::sorted_and_unique_prefix,
+                        bool reply_with_query = false);
 
 /**
  * Return the index with value @p value from the metadata's enumeration.
