@@ -24,11 +24,13 @@
 
 /**
  * @file pretty-format.h
- * @brief Header for pretty printer and scanner
+ * Header for pretty printer and scanner
+ *
+ * @test pretty-format.c
  */
 
-#ifndef PRETTYFORMAT_H
-#define PRETTYFORMAT_H
+#ifndef RTOSC_PRETTY_FORMAT
+#define RTOSC_PRETTY_FORMAT
 
 #include <rtosc/rtosc.h>
 
@@ -46,7 +48,8 @@ typedef struct
 } rtosc_print_options;
 
 /**
- * Pretty-print rtosct_arg_val_t structure into buffer
+ * Pretty-print rtosct_arg_val_t structure into buffer.
+ * Ranges are not being compressed (use rtosc_print_arg_vals instead).
  *
  * @param arg Pointer to the structure that shall be printed
  * @param buffer The buffer to write to
@@ -61,7 +64,8 @@ size_t rtosc_print_arg_val(const rtosc_arg_val_t* arg, char* buffer,
                            int* cols_used);
 
 /**
- * Pretty-print rtosct_arg_val_t array into buffer
+ * Pretty-print rtosct_arg_val_t array into buffer.
+ * Ranges are being compressed if opt->compress_ranges is true.
  *
  * @see rtosc_print_message
  * @warning in case of possible line breaks (almost always), buffer[-1] must
@@ -75,6 +79,7 @@ size_t rtosc_print_arg_vals(const rtosc_arg_val_t *args, size_t n,
 /**
  * Pretty-print OSC message into string buffer
  *
+ * Ranges are being compressed if opt->compress_ranges is true.
  * A newline will be appended.
  *
  * @param address OSC pattern to send message to
@@ -119,7 +124,7 @@ const char* rtosc_skip_next_printed_arg(const char* src, int* skipped,
  * no complete syntax check.
  *
  * @param src The string to scan from
- * @return The number of arguments that can be scanned (>=0), or if the nth arg
+ * @return The number of arguments that can be scanned (>0), or if the nth arg
  *   (range 1...) can not be scanned, -n. Array arguments and array start each
  *   count as one argument. This function never returns 0
  */
@@ -197,6 +202,6 @@ size_t rtosc_scan_message(const char* src,
                           char* buffer_for_strings, size_t bufsize);
 
 #ifdef __cplusplus
-};
+}
 #endif
-#endif // PRETTYFORMAT_H
+#endif // RTOSC_PRETTY_FORMAT
