@@ -43,22 +43,34 @@ typedef struct {
     uint8_t *data;
 } rtosc_blob_t;
 
+//! arg val element indicating an array, or a block of "nothing"
+//! blocks of "nothing" are just for internal use and indicate that the
+//! following array_start_t::len blocks must be ignored
+//! (they can contain rubbish)
 typedef struct {
     char type;               //!< common type of the elements
-    int32_t len;             //!< number of arg_val_t contained
+    int32_t len;             //!< number of arg_val_t (or "nothing") contained
 } array_start_t;
 
 //! a repeater is being followed by a delta argument (unless has_delta is 0)
 typedef struct
 {
-    int32_t       num;       //!< how often the element is being repeated
+    //! how often the first element is being repeated, including itself
+    int32_t       num;
     int32_t       has_delta; //!< if not 0, the next argument is the delta
 } repeater_t;
 
+//! indicates that the next nothing_t::len blocks count as if nonexistant
+//! they might contain rubbish
+typedef struct
+{
+    int32_t len;
+} nothing_t;
+
 typedef union {
 // types that can be used in messages
     int32_t       i;   //i,c,r
-    char          T;   //I,T,F,N
+    char          T;   //I,T,F,N (F=>0, T=>1, I=>n/a, N=>n/a)
     float         f;   //f
     double        d;   //d
     int64_t       h;   //h
@@ -67,10 +79,15 @@ typedef union {
     const char   *s;   //s,S
     rtosc_blob_t  b;   //b
 // types that can *not* be used in messages
-    array_start_t a;   //a
+    array_start_t a;   //a,' '
     repeater_t    r;   //-
 } rtosc_arg_t;
 
+typedef struct {
+    char type;
+    rtosc_arg_t val;
+} rtosc_arg_val_t;
+
 /**
  * Write OSC message to fixed length buffer
  *
@@ -111,6 +128,15 @@ size_t rtosc_vmessage(char   *buffer,
         const char *arguments,
         va_list va);
 
+/**
+ * @see rtosc_message()
+ */
+size_t rtosc_avmessage(char        *buffer,
+                       size_t       len,
+                       const char  *address,
+                       size_t       nargs,
+                       const rtosc_arg_val_t *args);
+
 /**
  * @see rtosc_message()
  */
@@ -140,78 +166,39 @@ typedef struct {
     const uint8_t *value_pos;
 } rtosc_arg_itr_t;
 
-typedef struct {
-    char type;
-    rtosc_arg_t val;
-} rtosc_arg_val_t;
-
 typedef struct
 {
     //!< tolerance to when two floats or doubles are equal
     double float_tolerance;
 } rtosc_cmp_options;
 
+/*
+ * arg val iterators
+ */
+/**
+ * Iterator over arg values
+ *
+ * Always use this iterator for iterating because it automatically skips
+ * to the next offset, even in case of arrays etc.
+ * Also, it walks through ranges, as if they were not existing.
+ */
 typedef struct
 {
     const rtosc_arg_val_t* av; //!< the arg val referenced
     size_t i;                  //!< position of this arg val
     int range_i;               //!< position of this arg val in its range
-} rtosc_arg_val_t_const_itr;
+} rtosc_arg_val_itr;
 
-/*
- * arg val iterators
- */
-void rtosc_arg_val_itr_init(rtosc_arg_val_t_const_itr* itr,
+void rtosc_arg_val_itr_init(rtosc_arg_val_itr* itr,
                             const rtosc_arg_val_t* av);
+//! this usually just returns the value from operand, except for range operands,
+//! where the value is being interpolated
+//! @param buffer Temporary. Don't access it afterwards.
 const rtosc_arg_val_t* rtosc_arg_val_itr_get(
-    const rtosc_arg_val_t_const_itr* itr,
+    const rtosc_arg_val_itr* itr,
     rtosc_arg_val_t* buffer);
-void rtosc_arg_val_itr_next(rtosc_arg_val_t_const_itr* itr);
-
-/*
- * arg val comparing helpers
- */
-int rtosc_arg_vals_cmp_has_next(const rtosc_arg_val_t_const_itr* litr,
-                                const rtosc_arg_val_t_const_itr* ritr,
-                                size_t lsize, size_t rsize);
-int rtosc_arg_vals_eq_after_abort(const rtosc_arg_val_t_const_itr* litr,
-                                  const rtosc_arg_val_t_const_itr* ritr,
-                                  size_t lsize, size_t rsize);
-int rtosc_arg_vals_eq_single(const rtosc_arg_val_t* _lhs,
-                             const rtosc_arg_val_t* _rhs,
-                             const rtosc_cmp_options* opt);
-int rtosc_arg_vals_cmp_single(const rtosc_arg_val_t* _lhs,
-                              const rtosc_arg_val_t* _rhs,
-                              const rtosc_cmp_options* opt);
-
-/**
- * Check if two arrays of rtosc_arg_val_t are equal
- *
- * @param lsize Array size of lhs, e.g. 3 if lhs is just one counting range
- * @param opt Comparison options or NULL for default options
- * @return One if they are equal, zero if not
- */
-int rtosc_arg_vals_eq(const rtosc_arg_val_t *lhs, const rtosc_arg_val_t *rhs,
-                      size_t lsize, size_t rsize,
-                      const rtosc_cmp_options* opt);
-
-/**
- * Compare two arrays of rtosc_arg_val_t.
- * Whether an argument value is less or greater than another is computed
- * - using memcmp for blobs
- * - using strcmp for strings and identifiers
- * - using numerical comparison for all other types.
- * As an exception, the timestamp "immediately" is defined to be smaller than
- * every other timestamp.
- *
- * @param opt Comparison options or NULL for default options
- * @param lsize Array size of lhs, e.g. 3 if lhs is just one counting range
- * @return An integer less than, equal to, or greater than zero if lhs is found,
- *         respectively, to be less than, to match, or be greater than rhs.
- */
-int rtosc_arg_vals_cmp(const rtosc_arg_val_t *lhs, const rtosc_arg_val_t *rhs,
-                       size_t lsize, size_t rsize,
-                       const rtosc_cmp_options* opt);
+//! @warning will loop forever on infinite ranges!
+void rtosc_arg_val_itr_next(rtosc_arg_val_itr* itr);
 
 //! va_list container, required for passing va_list as pointers to functions
 typedef struct { va_list a; } rtosc_va_list_t;
@@ -300,7 +287,7 @@ size_t rtosc_message_ring_length(ring_t *ring);
 bool rtosc_valid_message_p(const char *msg, size_t len);
 
 /**
- * @param OSC message
+ * @param msg OSC message
  * @returns the argument string of a given message
  */
 const char *rtosc_argument_string(const char *msg);
