@@ -7,6 +7,7 @@
 #include <cassert>
 #include <limits>
 #include <cstring>
+#include <set>
 #include <string>
 #include <algorithm>
 
@@ -252,8 +253,14 @@ typedef std::vector<tuple_t> tvec_t;
 namespace rtosc{
 class Port_Matcher
 {
+        bool *m_enump = nullptr; //!< @invariant nullptr or new[]'ed memory
     public:
-        bool *enump;
+        Port_Matcher(int ports_size) : m_enump(new bool[ports_size]) {}
+        ~Port_Matcher() { delete[] m_enump; }
+
+        const bool* enump() const { return m_enump; }
+        void set_enump(int i, bool val) const { m_enump[i] = val; }
+
         svec_t fixed;
         cvec_t arg_spec;
         ivec_t pos;
@@ -507,7 +514,6 @@ Ports::Ports(std::initializer_list<Port> l)
 
 Ports::~Ports()
 {
-    delete []impl->enump;
     delete impl;
 }
 
@@ -614,7 +620,7 @@ void Ports::dispatch(const char *m, rtosc::RtData &d, bool base_dispatch) const
                     d.matches++;
 
                 //Append the path
-                if(impl->enump[port_num]) {
+                if(impl->enump()[port_num]) {
                     const char *msg = m;
                     char       *pos = old_end;
                     while(*msg && *msg != '/')
@@ -745,21 +751,17 @@ const Port *Ports::operator[](const char *name) const
     return NULL;
 }
 
-static msg_t snip(msg_t m)
-{
-    while(*m && *m != '/') ++m;
-    return m+1;
-}
-
 const Port *Ports::apropos(const char *path) const
 {
     if(path && path[0] == '/')
         ++path;
 
+    const char* path_end;
     for(const Port &port: ports)
-        if(strchr(port.name,'/') && rtosc_match_path(port.name,path, NULL))
-            return (strchr(path,'/')[1]==0) ? &port :
-                port.ports->apropos(snip(path));
+        if(strchr(port.name,'/') && rtosc_match_path(port.name,path, &path_end))
+            return (port.ports && strchr(path,'/')[1])
+                ? port.ports->apropos(path_end)
+                : &port;
 
     //This is the lowest level, now find the best port
     for(const Port &port: ports)
@@ -838,11 +840,10 @@ char *Ports::collapsePath(char *p)
 void Ports::refreshMagic()
 {
     delete impl;
-    impl = new Port_Matcher;
+    impl = new Port_Matcher(ports.size());
     generate_minimal_hash(*this, *impl);
-    impl->enump = new bool[ports.size()];
     for(int i=0; i<(int)ports.size(); ++i)
-        impl->enump[i] = strchr(ports[i].name, '#');
+        impl->set_enump(i, !!strchr(ports[i].name, '#'));
 
     elms = ports.size();
 }
@@ -970,61 +971,151 @@ bool port_is_enabled(const Port* port, char* loc, size_t loc_size,
         return true;
 }
 
-// TODO: copy the changes into walk_ports_2
+/**
+    This recursing function is called if walk_ports hits one port @p p which has
+    sub-ports again. The @p runtime object still is the one belonging to the
+    base port which contains @p p as a subport.
+*/
+// this is doing nothing else than checking if a port is enabled (using the runtime),
+// and if yes, call walk_ports on its subports again
+// in case of no runtime, this only calls walk_ports
+static void walk_ports_recurse(const Port& p, char* name_buffer,
+                               size_t buffer_size, const Ports& base,
+                               void* data, port_walker_t walker,
+                               void* runtime, const char* old_end,
+                               bool expand_bundles, bool ranges)
+{
+    // TODO: all/most of these checks must also be done for the
+    // first, non-recursive call
+    bool enabled = true;
+    if(runtime)
+    {
+        // get child runtime and check if it's NULL
+
+        assert(old_end >= name_buffer);
+        assert(old_end - name_buffer <= 255);
+
+        const char* buf_ptr;
+        char buf[1024] = "";
+        fast_strcpy(buf, name_buffer, sizeof(buf));
+        // there is no "pointer" callback. thus, there will be nothing
+        // dispatched, but the rRecur*Cb already have set r.obj
+        // that way, we get our pointer
+        strncat(buf, "pointer", sizeof(buf) - strlen(buf) - 1);
+        assert(1024 - strlen(buf) >= 8);
+        fast_strcpy(buf + strlen(buf) + 1, ",", 2);
+
+        buf_ptr = buf + (old_end - name_buffer);
+
+        char locbuf[1024];
+        fast_strcpy(locbuf, name_buffer, sizeof(locbuf));
+        RtData r;
+        r.obj = runtime; // runtime object of the port that contains p
+        r.port = &p;
+        r.message = buf;
+        r.loc = locbuf;
+        r.loc_size = sizeof(locbuf);
+
+        p.cb(buf_ptr, r); // call "pointer" callback (see above)
+        // if there is runtime information (see above), but this pointer
+        // is NULL, the port is not enabled
+        enabled = (bool) r.obj; // r.obj = the next runtime object
+        if(enabled)
+        {
+            // check if the port is disabled by a switch
+            enabled = port_is_enabled(&p, name_buffer, buffer_size,
+                                      base, runtime);
+            runtime = r.obj; // callback has stored the pointer of p here
+        }
+    }
+    if(enabled)
+        rtosc::walk_ports(p.ports, name_buffer, buffer_size,
+                          data, walker, expand_bundles, runtime, ranges);
+};
+
+/**
+    This recursing function is called if walk_ports hits one port @p p which has
+    sub-ports again. The @p runtime object still is the one belonging to the
+    base port which contains @p p as a subport.
+*/
+/*
+char pointer example:
+
+          (Ports& base)         p.name       e.g. read_head
+                v               v            v
+               "/path/from/root/new#2/path#2/to#2/"
+
+               "/path/from/root/new1/path1/"
+                ^               ^          ^
+   name_buffer[buffer_size]     old_end    e.g. write_head
+
+*/
+static void walk_ports_recurse0(const Port& p, char* name_buffer,
+                                size_t buffer_size, const Ports* base,
+                                void* data, port_walker_t walker,
+                                void* runtime, char* const old_end, char* write_head,
+                                bool expand_bundles, const char* read_head,
+                                bool ranges)
+{
+    const char* hash_ptr = strchr(read_head + 1,'#');
+    std::size_t to_copy = hash_ptr ? hash_ptr - read_head : strlen(read_head);
+
+    //Append the path, until possible '#'
+    //TODO: buffer size checking
+    //yes, there are subports with ':', e.g. ".../::i"
+    while(to_copy-->0 && *read_head != ':')
+    {
+        *write_head++ = *read_head++;
+    }
+
+    if(hash_ptr)
+    {
+        //Overwrite the "#.../" by "0/", "1/" ...
+        assert(*read_head == '#');
+        const unsigned max = atoi(++read_head);
+        assert(isdigit(*read_head));
+        for(;isdigit(*read_head); ++read_head) {}
+
+        if(*read_head == '/') { ++read_head; }
+        if(ranges)
+        {
+            int written = sprintf(write_head,"[0,%d]/", max-1);
+            //Recurse
+            walk_ports_recurse0(p, name_buffer, buffer_size, base, data, walker,
+                                runtime, old_end, write_head + written,
+                                expand_bundles, read_head, ranges);
+        }
+        else for(unsigned i=0; i<max; ++i)
+        {
+            int written = sprintf(write_head,"%d/",i);
+            //Recurse
+            walk_ports_recurse0(p, name_buffer, buffer_size, base, data, walker,
+                                runtime, old_end, write_head + written,
+                                expand_bundles, read_head, ranges);
+        }
+    }
+    else
+    {
+        //Ensure last to trail with "/" before recursion
+        if(write_head[-1] != '/')
+            *write_head++ = '/';
+        *write_head = 0;
+        //Recurse
+        walk_ports_recurse(p, name_buffer, buffer_size,
+                           *base, data, walker, runtime, old_end,
+                           expand_bundles, ranges);
+    }
+};
+
 void rtosc::walk_ports(const Ports  *base,
                        char         *name_buffer,
                        size_t        buffer_size,
                        void         *data,
                        port_walker_t walker,
                        bool          expand_bundles,
-                       void*         runtime)
+                       void*         runtime,
+                       bool          ranges)
 {
-    auto walk_ports_recurse = [](const Port& p, char* name_buffer,
-                                 size_t buffer_size, const Ports& base,
-                                 void* data, port_walker_t walker,
-                                 void* runtime, const char* old_end,
-                                 bool expand_bundles)
-    {
-        // TODO: all/most of these checks must also be done for the
-        // first, non-recursive call
-        bool enabled = true;
-        if(runtime)
-        {
-            enabled = (p.meta().find("no walk") == p.meta().end());
-            if(enabled)
-            {
-                // get child runtime and check if it's NULL
-                RtData r;
-                r.obj = runtime;
-                r.port = &p;
-
-                char buf[1024] = "";
-                fast_strcpy(buf, old_end, sizeof(buf));
-                // there is no "pointer" callback. thus, there will be nothing
-                // dispatched, but the rRecur*Cb already have set r.obj
-                // that way, we get our pointer
-                strncat(buf, "pointer", sizeof(buf) - strlen(buf) - 1);
-                assert(1024 - strlen(buf) >= 8);
-                fast_strcpy(buf + strlen(buf) + 1, ",", 2);
-
-                p.cb(buf, r);
-                // if there is runtime information (see above), but this pointer
-                // is NULL, the port is not enabled
-                enabled = (bool) r.obj; // r.obj = the next runtime object
-                if(enabled)
-                {
-                    // check if the port is disabled by a switch
-                    enabled = port_is_enabled(&p, name_buffer, buffer_size,
-                                              base, runtime);
-                }
-                runtime = r.obj; // callback has stored the child pointer here
-            }
-        }
-        if(enabled)
-            rtosc::walk_ports(p.ports, name_buffer, buffer_size,
-                              data, walker, expand_bundles, runtime);
-    };
-
     //only walk valid ports
     if(!base)
         return;
@@ -1034,47 +1125,22 @@ void rtosc::walk_ports(const Ports  *base,
     if(name_buffer[0] == 0)
         name_buffer[0] = '/';
 
-    char *old_end         = name_buffer;
-    while(*old_end) ++old_end;
+    char * const old_end = name_buffer + strlen(name_buffer);
 
     if(port_is_enabled((*base)["self:"], name_buffer, buffer_size, *base,
                        runtime))
     for(const Port &p: *base) {
         //if(strchr(p.name, '/')) {//it is another tree
         if(p.ports) {//it is another tree
-            if(strchr(p.name,'#')) {
-                const char *name = p.name;
-                char       *pos  = old_end;
-                while(*name != '#') *pos++ = *name++;
-                const unsigned max = atoi(name+1);
 
-                for(unsigned i=0; i<max; ++i)
-                {
-                    sprintf(pos,"%d",i);
+            walk_ports_recurse0(p, name_buffer, buffer_size,
+                                base, data, walker, runtime, old_end, old_end,
+                                expand_bundles, p.name, ranges);
 
-                    //Ensure the result is a path
-                    if(strrchr(name_buffer, '/')[1] != '/')
-                        strcat(name_buffer, "/");
-
-                    //Recurse
-                    walk_ports_recurse(p, name_buffer, buffer_size,
-                                       *base, data, walker, runtime, old_end,
-                                       expand_bundles);
-                }
-            } else {
-                //Append the path
-                const char* old_end = name_buffer + strlen(name_buffer);
-                scat(name_buffer, p.name);
-
-                //Recurse
-                walk_ports_recurse(p, name_buffer, buffer_size,
-                                   *base, data, walker, runtime, old_end,
-                                   expand_bundles);
-            }
         } else {
             if(strchr(p.name,'#')) {
                 bundle_foreach(p, p.name, old_end, name_buffer, *base,
-                               data, runtime, walker, expand_bundles);
+                               data, runtime, walker, expand_bundles, true, ranges);
             } else {
                 //Append the path
                 scat(name_buffer, p.name);
@@ -1090,84 +1156,30 @@ void rtosc::walk_ports(const Ports  *base,
     }
 }
 
-void walk_ports2(const rtosc::Ports *base,
-                 char         *name_buffer,
-                 size_t        buffer_size,
-                 void         *data,
-                 rtosc::port_walker_t walker)
+// this is just an std::array replacement for path_search
+template <typename T, size_t N>
+struct my_array
 {
-    if(!base)
-        return;
-
-    assert(name_buffer);
-    //XXX buffer_size is not properly handled yet
-    if(name_buffer[0] == 0)
-        name_buffer[0] = '/';
-
-    char *old_end         = name_buffer;
-    while(*old_end) ++old_end;
-
-    for(const rtosc::Port &p: *base) {
-        if(strchr(p.name, '/')) {//it is another tree
-            if(strchr(p.name,'#')) {
-                const char *name = p.name;
-                char       *pos  = old_end;
-                while(*name != '#') *pos++ = *name++;
-                const unsigned max = atoi(name+1);
-
-                //for(unsigned i=0; i<max; ++i)
-                {
-                    sprintf(pos,"[0,%d]",max-1);
-
-                    //Ensure the result is a path
-                    if(strrchr(name_buffer, '/')[1] != '/')
-                        strcat(name_buffer, "/");
-
-                    //Recurse
-                    walk_ports2(p.ports, name_buffer, buffer_size,
-                                data, walker);
-                }
-            } else {
-                //Append the path
-                scat(name_buffer, p.name);
-
-                //Recurse
-                walk_ports2(p.ports, name_buffer, buffer_size,
-                        data, walker);
-            }
-        } else {
-            if(strchr(p.name,'#')) {
-                const char *name = p.name;
-                char       *pos  = old_end;
-                while(*name != '#') *pos++ = *name++;
-                const unsigned max = atoi(name+1);
-
-                //for(unsigned i=0; i<max; ++i)
-                {
-                    sprintf(pos,"[0,%d]",max-1);
-
-                    //Apply walker function
-                    walker(&p, name_buffer, old_end, *base, data, nullptr);
-                }
-            } else {
-                //Append the path
-                scat(name_buffer, p.name);
-
-                //Apply walker function
-                walker(&p, name_buffer, old_end, *base, data, nullptr);
-            }
-        }
-
-        //Remove the rest of the path
-        char *tmp = old_end;
-        while(*tmp) *tmp++=0;
+    T data[N];
+    inline void swap(my_array &other)
+    {
+        std::swap_ranges(&data[0], &data[0] + N, other.data);
     }
-}
+    inline T& operator[](const size_t idx)
+    {
+        return data[idx];
+    }
+    inline const T& operator[](const size_t idx) const
+    {
+        return data[idx];
+    }
+};
 
 void rtosc::path_search(const rtosc::Ports& root,
                         const char *str, const char* needle,
                         char *types, std::size_t max_types,
-                        rtosc_arg_t* args, std::size_t max_args)
+                        rtosc_arg_t* args, std::size_t max_args,
+                        path_search_opts opts, bool reply_with_query)
 {
     using rtosc::Ports;
     using rtosc::Port;
@@ -1185,10 +1197,19 @@ void rtosc::path_search(const rtosc::Ports& root,
     memset(types, 0, max + 1);
     memset(args,  0, max);
 
-    if(!*str) {
+    if(reply_with_query) {
+        assert(max >= 2);
+        types[pos]    = 's';
+        args[pos++].s = str;
+        types[pos]    = 's';
+        args[pos++].s = needle;
+    }
+
+    if(!*str || !strcmp(str, "/")) {
         ports = &root;
     } else {
         const Port *port = root.apropos(str);
+        //fprintf(stderr, "apropos %s -> %s\n",str,port?port->name:"<no result>");
         if(port) {
             if(port->ports) {
                 ports = port->ports;
@@ -1201,6 +1222,7 @@ void rtosc::path_search(const rtosc::Ports& root,
     const auto fn = [&pos,&needle,&types,&args,&max](const Port& p)
     {
         assert(pos < max);
+        //fprintf(stderr, "path search iterating port: %s (needle %s) (pos %d)\n", p.name, needle, (int)pos);
         if(p.name && strstr(p.name, needle) == p.name)
         {
             types[pos]    = 's';
@@ -1221,11 +1243,65 @@ void rtosc::path_search(const rtosc::Ports& root,
         for(const Port &p:*ports) fn(p);
     else if(single_port)
         fn(*single_port);
+
+    if (opts == path_search_opts::sorted ||
+        opts == path_search_opts::sorted_and_unique_prefix)
+    {
+        // we could use std::array, but its internal array does not necessarily
+        // have offset 0
+        using val_on_2 = my_array<rtosc_arg_t, 2>;
+        using ptr_on_2 = val_on_2*;
+        auto is_less = [](const val_on_2 &p1, const val_on_2 &p2) -> bool {
+            return strcmp(p1[0].s, p2[0].s) < 0;
+        };
+        std::size_t n_paths_found = pos >> 1;
+        std::sort((ptr_on_2)args, ((ptr_on_2)(args))+n_paths_found, is_less);
+
+        if (opts == path_search_opts::sorted_and_unique_prefix)
+        {
+            std::size_t prev_pos = 0;
+            std::size_t strlen_prev = n_paths_found > 1 ? strlen(args[prev_pos].s) : 0;
+            std::size_t unused_paths = 0;
+            for(pos = 2; pos < (n_paths_found<<1); ++++pos)
+            {
+                assert(args[prev_pos].s); // invariant
+
+                // is the prev path a (real) sub-path of this path?
+                // i.e. the current can be accessed by recursing into the prev?
+                if(strlen_prev < strlen(args[pos].s) &&
+                   0 == strncmp(args[pos].s, args[prev_pos].s, strlen_prev) &&
+                   args[prev_pos].s[strlen_prev-1] == '/')
+                {
+                    // then mark this as unused
+                    args[pos].s = nullptr;
+                    ++unused_paths;
+                }
+                else
+                {
+                    prev_pos = pos;
+                    strlen_prev = strlen(args[prev_pos].s);
+                }
+            }
+
+            // another sort, only to move unused paths to the end
+            auto is_less_2 = [](const val_on_2 &p1, const val_on_2 &p2) -> bool {
+                return (!(p1[0].s)) ? false // move p1 to the end
+                                    : (!(p2[0].s)) ? true // move p2 to the end
+                                                   // is actually already sorted:
+                                                   : (strcmp(p1[0].s, p2[0].s) < 0);
+            };
+            std::sort((ptr_on_2)args, ((ptr_on_2)(args))+n_paths_found, is_less_2);
+
+            // cut off unused paths
+            types[(n_paths_found - unused_paths)<<1] = 0;
+        }
+    }
 }
 
 std::size_t rtosc::path_search(const Ports &root, const char *m,
                                std::size_t max_ports,
-                               char *msgbuf, std::size_t bufsize)
+                               char *msgbuf, std::size_t bufsize,
+                               path_search_opts opts, bool reply_with_query)
 {
     const char *str    = rtosc_argument(m,0).s;
     const char *needle = rtosc_argument(m,1).s;
@@ -1234,7 +1310,7 @@ std::size_t rtosc::path_search(const Ports &root, const char *m,
     STACKALLOC(char, types, max_types);
     STACKALLOC(rtosc_arg_t, args, max_args);
 
-    path_search(root, str, needle, types, max_types, args, max_args);
+    path_search(root, str, needle, types, max_types, args, max_args, opts, reply_with_query);
     size_t length = rtosc_amessage(msgbuf, bufsize,
                                    "/paths", types, args);
     return length;
@@ -1379,8 +1455,7 @@ static ostream &dump_generic_port(ostream &o, string name, string doc, string ty
         return o;
 }
 
-void dump_ports_cb(const rtosc::Port *p, const char *name,const char*,
-                   const Ports&,void *v, void*)
+static bool do_dump_ports(const rtosc::Port *p, const char *name, void *v)
 {
     std::ostream &o  = *(std::ostream*)v;
     auto meta        = p->meta();
@@ -1413,12 +1488,12 @@ void dump_ports_cb(const rtosc::Port *p, const char *name,const char*,
         if(!type) {
             fprintf(stderr, "rtosc port dumper: Cannot handle '%s'\n", name);
             fprintf(stderr, "    args = <%s>\n", args);
-            return;
+            return false;
         }
 
         if(type == 't') {
             dump_t_f_port(o, name, doc);
-            return;
+            return true;
         }
 
         o << " <message_in pattern=\"" << name << "\" typetag=\"" << type << "\">\n";
@@ -1477,8 +1552,24 @@ void dump_ports_cb(const rtosc::Port *p, const char *name,const char*,
         if(args) {
             fprintf(stderr, "    type = %s\n", args);
         }
-    } else
+        return false;
+    } else {
         fprintf(stderr, "Skipping [UNDOCUMENTED] \"%s\"\n", name);
+        return false;
+    }
+    return true;
+}
+
+static void dump_ports_cb(const rtosc::Port *p, const char *name,const char*,
+                          const Ports&,void *v, void*)
+{
+    static std::set<std::pair<std::string, std::string>> already_dumped;
+    if(already_dumped.find(std::make_pair(name, p->name)) == already_dumped.end())
+    {
+        bool dumped = do_dump_ports(p, name, v);
+        if(dumped)
+            already_dumped.emplace(name, p->name);
+    }
 }
 
 std::ostream &rtosc::operator<<(std::ostream &o, rtosc::OscDocFormatter &formatter)
@@ -1494,7 +1585,7 @@ std::ostream &rtosc::operator<<(std::ostream &o, rtosc::OscDocFormatter &formatt
     o << " </meta>\n";
     char buffer[1024];
     memset(buffer, 0, sizeof(buffer));
-    walk_ports2(formatter.p, buffer, 1024, &o, dump_ports_cb);
+    walk_ports(formatter.p, buffer, 1024, &o, dump_ports_cb, false, nullptr, true);
     o << "</osc_unit>\n";
     return o;
 }
