@@ -2,6 +2,7 @@
 #include <cassert>
 #include <cstring>
 #include <algorithm>
+#include <set>
 
 #include "../util.h"
 #include <rtosc/arg-val-cmp.h>
@@ -19,18 +20,27 @@ namespace {
     constexpr size_t max_arg_vals = 2048;
 }
 
+// this basically does:
+// walk over all ports and for each port:
+// * get default value
+// * get current value
+// * compare: if values are different -> write to savefile
 std::string get_changed_values(const Ports& ports, void* runtime)
 {
-    std::string res;
     char port_buffer[buffersize];
     memset(port_buffer, 0, buffersize); // requirement for walk_ports
 
+    struct data_t
+    {
+        std::string res;
+        std::set<std::string> written;
+    } data;
+
     auto on_reach_port =
             [](const Port* p, const char* port_buffer,
                const char* port_from_base, const Ports& base,
                void* data, void* runtime)
     {
-
         assert(runtime);
         const Port::MetaContainer meta = p->meta();
 #if 0
@@ -60,6 +70,18 @@ std::string get_changed_values(const Ports& ports, void* runtime)
             }
         }
 
+        if(meta.find("alias") != meta.end())
+        {
+            // this param is already saved in another port
+            return;
+        }
+
+        {
+            std::set<std::string>& written = ((data_t*)data)->written;
+            if(written.find(port_buffer) != written.end()) { return; }
+            else { written.insert(port_buffer); }
+        }
+
         char loc[buffersize] = ""; // buffer to hold the dispatched path
         rtosc_arg_val_t arg_vals_default[max_arg_vals];
         rtosc_arg_val_t arg_vals_runtime[max_arg_vals];
@@ -67,7 +89,7 @@ std::string get_changed_values(const Ports& ports, void* runtime)
         char buffer_with_port[buffersize];
         char strbuf[buffersize]; // temporary string buffer for pretty-printing
 
-        std::string* res = (std::string*)data;
+        std::string* res = &((data_t*)data)->res;
         assert(strlen(port_buffer) + 1 < buffersize);
         // copy the path until before the message
         fast_strcpy(loc, port_buffer, std::min((ptrdiff_t)buffersize,
@@ -253,12 +275,12 @@ std::string get_changed_values(const Ports& ports, void* runtime)
         }
     };
 
-    walk_ports(&ports, port_buffer, buffersize, &res, on_reach_port, false,
+    walk_ports(&ports, port_buffer, buffersize, &data, on_reach_port, false,
                runtime);
 
-    if(res.length()) // remove trailing newline
-        res.resize(res.length()-1);
-    return res;
+    if(data.res.length()) // remove trailing newline
+        data.res.resize(data.res.length()-1);
+    return data.res;
 }
 
 bool savefile_dispatcher_t::do_dispatch(const char* msg)
@@ -454,22 +476,29 @@ int dispatch_printed_messages(const char* messages,
 }
 
 std::string save_to_file(const Ports &ports, void *runtime,
-                         const char *appname, rtosc_version appver)
+                         const char *appname, rtosc_version appver,
+                         std::string file_str)
 {
-    std::string res;
     char rtosc_vbuf[12], app_vbuf[12];
 
+    if(file_str.empty())
     {
-        rtosc_version rtoscver = rtosc_current_version();
-        rtosc_version_print_to_12byte_str(&rtoscver, rtosc_vbuf);
-        rtosc_version_print_to_12byte_str(&appver, app_vbuf);
+        {
+            rtosc_version rtoscver = rtosc_current_version();
+            rtosc_version_print_to_12byte_str(&rtoscver, rtosc_vbuf);
+            rtosc_version_print_to_12byte_str(&appver, app_vbuf);
+        }
+
+        file_str += "% RT OSC v"; file_str += rtosc_vbuf; file_str += " savefile\n"
+               "% "; file_str += appname; file_str += " v"; file_str += app_vbuf; file_str += "\n";
     }
+    else
+    {
+        // append mode - no header
+    }
+    file_str += get_changed_values(ports, runtime);
 
-    res += "% RT OSC v"; res += rtosc_vbuf; res += " savefile\n"
-           "% "; res += appname; res += " v"; res += app_vbuf; res += "\n";
-    res += get_changed_values(ports, runtime);
-
-    return res;
+    return file_str;
 }
 
 int load_from_file(const char* file_content,
