@@ -329,12 +329,6 @@ void rtosc_v2args(rtosc_arg_t* args, size_t nargs, const char* arg_str,
             case 'f':
                 args[arg_pos++].f = va_arg(ap->a, double);
                 break;
-            case 'T':
-                args[arg_pos++].T = 1;
-                break;
-            case 'F':
-                args[arg_pos++].T = 0;
-                break;
             default:
                 ;
         }
@@ -895,7 +889,7 @@ size_t rtosc_bundle_size(const char *buffer, unsigned elm)
     const uint32_t *lengths = (const uint32_t*) (buffer+16);
     size_t elm_pos = 0;
     size_t last_len = 0;
-    while(elm_pos!=elm && extract_uint32((const uint8_t*)lengths)) {
+    while(elm_pos!=elm+1 && extract_uint32((const uint8_t*)lengths)) {
         last_len = extract_uint32((const uint8_t*)lengths);
         ++elm_pos, lengths+=extract_uint32((const uint8_t*)lengths)/4+1;
     }
