@@ -16,10 +16,19 @@
 
 #include "port-checker.h"
 
+// enable this if you need to find bugs in port-checker
+// #define DEBUG_PORT_CHECKER
+
 namespace rtosc {
 
 static constexpr const issue_t _m_issue_types_arr[(int)issue::number] =
 {
+/*{
+    issue::trailing_slash_without_subports,
+    "ports with trailing slashes but no subports",
+    "ports have trailing slashes, but no subports",
+    severity::hint
+},*/
 {
     issue::duplicate_parameter,
     "multiple parameters with same name",
@@ -120,17 +129,27 @@ void port_checker::server::on_recv(const char *path, const char *types,
                                    lo_arg **argv, int argc, lo_message msg)
 {
     (void)argv;
-//  std::cout << "on_recv: " << path << ", " << waiting << std::endl;
-//  for(const char** exp_path = exp_paths; *exp_path;
-//      ++exp_path, ++_replied_path)
-//      std::cout << " - exp: " << *exp_path << std::endl;
-    if(waiting && exp_paths && exp_paths[0])
+#ifdef DEBUG_PORT_CHECKER
+    std::cout << "on_recv: " << path << ", " << waiting << std::endl;
+    for(std::vector<const char*>* exp_strs = exp_paths_n_args; exp_strs->size();
+        ++exp_strs, ++_replied_path)
+    {
+        std::cout << " - exp. path: " << (*exp_strs)[0] << std::endl;
+        for(std::size_t i = 1; i < exp_strs->size(); ++i)
+            std::cout << "   - exp. arg " << i-1 << ": " << (*exp_strs)[i] << std::endl;
+    }
+#endif
+    if(waiting && exp_paths_n_args[0].size())
     {
         _replied_path = 0;
-        for(const char** exp_path = exp_paths; *exp_path;
-            ++exp_path, ++_replied_path)
-        if(!strcmp(*exp_path, path))
+        for(std::vector<const char*>* exp_strs = exp_paths_n_args; exp_strs->size();
+            ++exp_strs, ++_replied_path)
+        if(!strcmp((*exp_strs)[0], path))
         {
+#ifdef DEBUG_PORT_CHECKER
+            std::cout << "on_recv: match:" << std::endl;
+            lo_message_pp(msg);
+#endif
             size_t len = lo_message_length(msg, path);
             *last_buffer = std::vector<char>(len);
             size_t written;
@@ -140,13 +159,18 @@ void port_checker::server::on_recv(const char *path, const char *types,
                                          "lo_message_length has been used");
 
             last_args->resize(argc);
-            for(int i = 0; i < argc; ++i)
+            bool still_ok = true;
+            for(std::size_t i = 0; i < (std::size_t)argc; ++i)
             {
                 (*last_args)[i].val = rtosc_argument(last_buffer->data(), i);
                 (*last_args)[i].type = types[i];
+                if(still_ok && (*last_args)[i].type == 's' && i+1 < exp_strs->size())
+                {
+                    still_ok = !strcmp((*last_args)[i].val.s, (*exp_strs)[i+1]);
+                }
             }
 
-            waiting = false;
+            if(still_ok) { waiting = false; }
             break;
         }
     }
@@ -165,13 +189,18 @@ void port_checker::server::init(const char* target_url)
         throw std::runtime_error("Could not create lo server");
     lo_server_add_method(srv, nullptr, nullptr, handle_st, this);
 
-    rtosc_arg_val_t hi[2];
+    rtosc_arg_val_t hi[3];
     hi[0].type = hi[1].type = 's';
     hi[0].val.s = hi[1].val.s = "";
-    send_msg("/path-search", 2, hi);
+    hi[2].type = 'T';
+    hi[2].val.T = 1;
+    send_msg("/path-search", 3, hi);
     std::vector<rtosc_arg_val_t> reply;
     std::vector<char> buf;
-    wait_for_reply(&buf, &reply, "/paths");
+    bool ok = wait_for_reply(&buf, &reply, "/paths", hi[0].val.s, hi[1].val.s);
+    if(!ok) {
+        throw std::runtime_error("OSC app does not reply at all");
+    }
 }
 
 bool port_checker::send_msg(const char* address,
@@ -191,9 +220,10 @@ bool port_checker::server::send_msg(const char* address,
         std::cout << "liblo error code: " << res << std::endl;
         throw std::runtime_error("could not deserialize message");
     }
-    // std::cout << "send message: " << address << ", args:" << std::endl;
-    // lo_message_pp(msg);
-
+#ifdef DEBUG_PORT_CHECKER
+    std::cout << "send message: " << address << ", args:" << std::endl;
+    lo_message_pp(msg);
+#endif
     res = lo_send_message_from(target, srv, buffer, msg);
     if(res == -1)
         throw std::runtime_error("Could not send message");
@@ -202,18 +232,21 @@ bool port_checker::server::send_msg(const char* address,
 
 bool port_checker::server::_wait_for_reply(std::vector<char>* buffer,
                                            std::vector<rtosc_arg_val_t> * args,
-                                           int unused)
+                                           int n0, int n1)
 {
-    (void)unused;
+    (void)n0;
+    exp_paths_n_args[n1].clear();
+
     last_args = args;
     last_buffer = buffer;
 
-    // allow up to 1000 ports = 1s. never wait more than 0.05 seconds
-    const int timeout_initial = 50;
+    // allow up to 1000 recv calls = 1s
+    // if there's no reply at all after 0.5 seconds, abort
+    const int timeout_initial = timeout_msecs;
     int tries_left = 1000, timeout = timeout_initial;
-    while(tries_left-->1 && timeout-->1 && waiting)
+    while(tries_left-->1 && timeout-->1 && waiting) // waiting is set in "on_recv"
     {
-        int n = lo_server_recv_noblock(srv, 1 /* 0,001 seconds */);
+        int n = lo_server_recv_noblock(srv, 1 /* 0,001 second = 1 milli second */);
         if(n)
             timeout = timeout_initial;
         // message will be dispatched to the server's callback
@@ -233,8 +266,7 @@ bool port_checker::port_is_enabled(const char* loc, const char* port,
     bool rval = true;
     if(enable_port_rel)
     {
-        std::string enable_port = "/";
-        enable_port += loc;
+        std::string enable_port = loc;
         enable_port += enable_port_rel;
 
         const char* collapsed_loc = Ports::collapsePath(&enable_port[0]);
@@ -249,11 +281,13 @@ bool port_checker::port_is_enabled(const char* loc, const char* port,
             }
             else
                 m_issues.emplace(issue::enabled_port_bad_reply,
-                               "/" + std::string(loc) + port);
+                                 std::string(loc) + port);
         } else
             m_issues.emplace(issue::enabled_port_not_replied,
-                           "/" + std::string(loc) + port);
+                             std::string(loc) + port);
     }
+    //std::cout << "port \"" << loc << "\" enabled? "
+    //          << (rval ? "yes" : "no") << std::endl;
     return rval;
 }
 
@@ -285,7 +319,7 @@ void port_checker::check_port(const char* loc, const char* portname,
     const char* port_args = strchr(portname, ':');
     if(!port_args)
         port_args = portname + strlen(portname);
-    std::string full_path = "/"; full_path += loc; full_path += portname;
+    std::string full_path = loc; full_path += portname;
     std::string::size_type arg_pos = full_path.find(':');
     if(arg_pos != std::string::npos)
     {
@@ -296,13 +330,19 @@ void port_checker::check_port(const char* loc, const char* portname,
         if(hash_pos != std::string::npos)
         {
             bundle_size = atoi(full_path.data() + hash_pos + 1);
+
             // .../port#16... -> .../port0
             full_path[hash_pos] = '0';
-            full_path.resize(hash_pos+1);
+            std::string::size_type slash = full_path.find('/', hash_pos + 1);
+            full_path.erase(hash_pos + 1,
+                (slash == std::string::npos)
+                ? std::string::npos // cut string after '#0'
+                : slash - (hash_pos + 1) // cut anything between '#0' and '/'
+            );
         }
 
         rtosc::Port::MetaContainer meta(metadata);
-        if(*metadata && meta_len)
+        if(meta_len && metadata && *metadata)
         {
             bool is_parameter = false, is_enumerated = false;
             int n_default_vals = 0;
@@ -337,7 +377,7 @@ void port_checker::check_port(const char* loc, const char* portname,
             }
 
             auto raise = [this, loc, portname](issue issue_type) {
-                m_issues.emplace(issue_type, "/" + std::string(loc) + portname);
+                m_issues.emplace(issue_type, std::string(loc) + portname);
             };
 
             for(auto pr : mapping_values)
@@ -445,8 +485,8 @@ void port_checker::check_port(const char* loc, const char* portname,
                     args1.clear();
                     strbuf1.clear();
                     res = sender.wait_for_reply(&strbuf1, &args1,
-                                                full_path.c_str(),
-                                                "/undo_change");
+                                                full_path.c_str(), nullptr,
+                                                "/undo_change", nullptr);
 
                     if(!res)
                         raise(issue::parameter_not_replied);
@@ -490,7 +530,7 @@ void port_checker::print_evaluation() const
     std::cout << "# Port evaluation" << std::endl;
 
     issue last_issue = issue::number;
-    for(const std::pair<issue, std::string>& p : m_issues)
+    for(const std::pair<const issue, std::string>& p : m_issues)
     {
         if(last_issue != p.first)
         {
@@ -598,7 +638,7 @@ const std::multimap<issue, std::string> &port_checker::issues() const {
 int port_checker::errors_found() const
 {
     int errors = 0;
-    for(const std::pair<issue, std::string>& p : m_issues)
+    for(const std::pair<const issue, std::string>& p : m_issues)
         errors += (m_issue_types.at(p.first).sev == severity::error);
     return errors;
 }
@@ -645,31 +685,37 @@ bool port_checker::print_sanity_checks() const
 void port_checker::do_checks(char* loc, int loc_size, bool check_defaults)
 {
     char* old_loc = loc + strlen(loc);
-//  std::cout << "Checking Ports: \"" << loc << "\"..." << std::endl;
+#ifdef DEBUG_PORT_CHECKER
+    std::cout << "Checking Ports: \"" << loc << "\"..." << std::endl;
+#endif
 
-
-    rtosc_arg_val_t query_args[2];
+    rtosc_arg_val_t query_args[3];
     query_args[0].type = query_args[1].type = 's';
     query_args[0].val.s = loc;
     query_args[1].val.s = "";
-    send_msg("/path-search", 2, query_args);
+    query_args[2].type = 'T';
+    query_args[2].val.T = 1;
+    send_msg("/path-search", 3, query_args);
     std::vector<rtosc_arg_val_t> args;
     std::vector<char> strbuf;
 
-    int res = sender.wait_for_reply(&strbuf, &args, "/paths");
+    int res = sender.wait_for_reply(&strbuf, &args, "/paths",
+                                    query_args[0].val.s, query_args[1].val.s);
     if(!res)
-        throw std::runtime_error("no reply from path-search");
+        throw port_error("no reply from path-search", loc);
     if(args.size() % 2)
-        throw std::runtime_error("bad reply from path-search");
+        throw port_error("bad reply from path-search", loc);
 
     bool self_disabled = false; // disabled by an rSelf() ?
     size_t port_max = args.size() >> 1;
-    // std::cout << "found " << port_max << " subports" << std::endl;
-    for(size_t port_no = 0; port_no < port_max; ++port_no)
+#ifdef DEBUG_PORT_CHECKER
+    std::cout << "found " << port_max-1 << " subports" << std::endl;
+#endif
+    for(size_t port_no = 1; port_no < port_max; ++port_no)
     {
         if(args[port_no << 1].type != 's' ||
                 args[(port_no << 1) + 1].type != 'b')
-            throw std::runtime_error("Invalid \"paths\" reply: bad types");
+            throw port_error("Invalid \"paths\" reply: bad types", loc);
 
         if(!strncmp(args[port_no << 1].val.s, "self:", 5)) {
             rtosc_blob_t& blob = args[(port_no << 1) + 1].val.b;
@@ -679,45 +725,79 @@ void port_checker::do_checks(char* loc, int loc_size, bool check_defaults)
         }
     }
     if(self_disabled)
-        m_skipped.insert("/" + std::string(loc));
+        m_skipped.insert(loc);
     else
         ++ports_checked;
 
     std::map<std::string, unsigned> port_count;
 
     if(!self_disabled)
-    for(size_t port_no = 0; port_no < port_max; ++port_no)
+    for(size_t port_no = 1; port_no < port_max; ++port_no)
     {
         const char* portname = args[port_no << 1].val.s;
-        int portlen = strlen(portname);
-        rtosc_blob_t& blob = args[(port_no << 1) + 1].val.b;
+        const int portlen = strlen(portname);
+        const rtosc_blob_t& blob = args[(port_no << 1) + 1].val.b;
         const char* metadata = (const char*)blob.data;
-        int32_t meta_len = blob.len;
-        if(!metadata)
-            metadata = "";
+        const int32_t meta_len = blob.len;
+        const bool has_meta = meta_len && metadata && *metadata;
         bool has_subports = portname[portlen-1] == '/';
-/*      std::cout << "port /" << loc << portname
-                  << " (" << port_no << "/" << port_max
+        if(!has_meta)
+           metadata = "";
+#ifdef DEBUG_PORT_CHECKER
+        std::cout << "port " << loc << ", replied: " << portname
+                  << " (" << port_no << "/" << (port_max-1)
                   << "), has subports: " << std::boolalpha << has_subports
-                  << std::endl;*/
-
-        if(port_is_enabled(loc, portname, metadata))
+                  << std::endl;
+#endif
+        if(!has_meta || port_is_enabled(loc, portname, metadata))
         {
             Port::MetaContainer meta(metadata);
 
-            if(meta.find("parameter") != meta.end())
+            if(has_meta && meta.find("parameter") != meta.end())
             {
                 std::string portname_noargs = portname;
                 portname_noargs.resize(portname_noargs.find(':'));
+                // count portname to detect duplicate parameter ports
                 ++port_count[portname_noargs];
             }
 
+            if(rtosc_match_path(portname, loc, nullptr))
+            {
+                // the returned path has equal length
+                // -> it's a single port without subports
+                if(has_subports)
+                {
+/*                  m_issues.emplace(issue::trailing_slash_without_subports,
+                                     loc);*/
+                    has_subports = false;
+                }
+            }
+            else
+            {
+                if(has_subports)
+                {
+                    // statistics: port may still be disabled, see above
+                    if(loc_size > portlen)
+                    {
+                        // TODO: it could still be a port without subports tht
+                        //       did not pass rtosc_match_path
+                        // it can not come from the root now,
+                        // so it's relative to the current port
+                        strcpy(old_loc, portname);
+                        // TODO: the portname could be relative (i.e. rtosc_match_path
+                        //       failed), but relative to a parent port
+                        //       => only copy last part ("basename") of portname?
+                    }
+                    else
+                        throw port_error("portname too long", loc);
+                }
+            }
+
             if(has_subports)
             {
                 // statistics: port may still be disabled, see above
                 if(loc_size > portlen)
                 {
-                    strcpy(old_loc, portname);
                     char* hashsign = strchr(old_loc, '#');
                     if(hashsign)
                     {
@@ -728,12 +808,9 @@ void port_checker::do_checks(char* loc, int loc_size, bool check_defaults)
                     }
 
                     bool next_check_defaults =
-                       (meta.find("no defaults") == meta.end());
-
+                       !has_meta || (meta.find("no defaults") == meta.end());
                     do_checks(loc, loc_size - portlen, next_check_defaults);
                 }
-                else
-                    throw std::runtime_error("portname too long");
             }
             else {
                 ++ports_checked;
@@ -741,7 +818,7 @@ void port_checker::do_checks(char* loc, int loc_size, bool check_defaults)
             }
         }
         else
-            m_skipped.insert("/" + std::string(loc) + portname);
+            m_skipped.insert(std::string(loc) + portname);
         *old_loc = 0;
     }
 
@@ -749,7 +826,7 @@ void port_checker::do_checks(char* loc, int loc_size, bool check_defaults)
     {
         if(pr.second > 1)
             m_issues.emplace(issue::duplicate_parameter,
-                             "/" + std::string(loc) + pr.first);
+                             std::string(loc) + pr.first);
     }
 }
 
@@ -777,12 +854,19 @@ bool port_checker::operator()(const char* url)
     sender.init(sendtourl.c_str());
     other.init(sendtourl.c_str());
 
-    char loc_buffer[4096] = { 0 };
+    char loc_buffer[4096] = { '/', 0 };
     do_checks(loc_buffer, sizeof(loc_buffer));
 
     finish_time = time(NULL);
     return true;
 }
 
+port_error::port_error(const char *errmsg, const char *port)
+    : runtime_error(errmsg)
+{
+    strncpy(m_port, port, sizeof(m_port) - 1);
+    m_port[sizeof(m_port) - 1] = 0;
+}
+
 }
 
