@@ -19,6 +19,8 @@
 #include "AnalogFilter.h"
 #include "FormantFilter.h"
 #include "SVFilter.h"
+#include "MoogFilter.h"
+#include "CombFilter.h"
 #include "../Params/FilterParams.h"
 #include "../Misc/Allocator.h"
 
@@ -52,6 +54,14 @@ Filter *Filter::generate(Allocator &memory, const FilterParams *pars,
             if(filter->outgain > 1.0f)
                 filter->outgain = sqrt(filter->outgain);
             break;
+        case 3:
+            filter = memory.alloc<MoogFilter>(Ftype, 1000.0f, pars->getq(), srate, bufsize);
+            filter->setgain(pars->getgain());
+            break;
+        case 4:
+            filter = memory.alloc<CombFilter>(&memory, Ftype, 1000.0f, pars->getq(), srate, bufsize);
+            filter->outgain = dB2rap(pars->getgain());
+            break;
         default:
             filter = memory.alloc<AnalogFilter>(Ftype, 1000.0f, pars->getq(), Fstages, srate, bufsize);
             if((Ftype >= 6) && (Ftype <= 8))
