@@ -18,12 +18,14 @@
 #include "Unison.h"
 #include "globals.h"
 
-#define errx(...)
-#define warnx(...)
+#define errx(...) {}
+#define warnx(...) {}
 #ifndef errx
 #include <err.h>
 #endif
 
+namespace zyn {
+
 Unison::Unison(Allocator *alloc_, int update_period_samples_, float max_delay_sec_, float srate_f)
     :unison_size(0),
       base_freq(1.0f),
@@ -196,3 +198,5 @@ void Unison::updateUnisonData()
     }
     first_time = false;
 }
+
+}
