@@ -1,7 +1,7 @@
 /*
   ZynAddSubFX - a software synthesizer
 
-  EffectMgr.cpp - Effect manager, an interface betwen the program and effects
+  EffectMgr.cpp - Effect manager, an interface between the program and effects
   Copyright (C) 2002-2005 Nasca Octavian Paul
   Author: Nasca Octavian Paul
 
@@ -28,6 +28,7 @@
 #include "Phaser.h"
 #include "../Misc/XMLwrapper.h"
 #include "../Misc/Util.h"
+#include "../Misc/Time.h"
 #include "../Params/FilterParams.h"
 #include "../Misc/Allocator.h"
 
@@ -117,6 +118,100 @@ static const rtosc::Ports local_ports = {
                 }
             }
         }},
+    {"numerator::i", rShort("num") rDefault(0) rLinear(0,99)
+        rProp(parameter) rDoc("Numerator of ratio to bpm"), NULL,
+        [](const char *msg, rtosc::RtData &d)
+        {
+            EffectMgr *eff = (EffectMgr*)d.obj;
+            if(rtosc_narguments(msg)) {
+                int val = rtosc_argument(msg, 0).i;
+                if (val>=0) {
+                    eff->numerator = val;
+                    int Pdelay, Pfreq;
+                    float freq;
+                    if(eff->denominator) {
+                        switch(eff->nefx) {
+                        case 2: // Echo
+                            // invert:
+                            // delay = (Pdelay / 127.0f * 1.5f); //0 .. 1.5 sec
+                            Pdelay = (int)roundf((20320.0f / (float)eff->time->tempo) * 
+                                                 ((float)eff->numerator / (float)eff->denominator));
+                            if (eff->numerator&&eff->denominator)
+                                eff->seteffectparrt(2, Pdelay);
+                            break;
+                        case 3: // Chorus
+                        case 4: // Phaser
+                        case 5: // Alienwah
+                        case 8: // DynamicFilter
+                            freq =  ((float)eff->time->tempo * 
+                                     (float)eff->denominator / 
+                                     (240.0f * (float)eff->numerator));
+                            // invert:
+                            // (powf(2.0f, Pfreq / 127.0f * 10.0f) - 1.0f) * 0.03f
+                            Pfreq = (int)roundf(logf((freq/0.03f)+1.0f)/LOG_2 * 12.7f);
+                            if (eff->numerator&&eff->denominator)
+                                eff->seteffectparrt(2, Pfreq);
+                            break;
+                        case 1: // Reverb
+                        case 6: // Distorsion
+                        case 7: // EQ
+                        default:
+                            break;
+                        }
+                    }
+                }
+                d.broadcast(d.loc, "i", val);
+            } else {
+                d.reply(d.loc, "i", eff->numerator); 
+            }
+        }},
+    {"denominator::i", rShort("dem") rDefault(4) rLinear(1,99)
+        rProp(parameter) rDoc("Denominator of ratio to bpm"), NULL,
+        [](const char *msg, rtosc::RtData &d)
+        {
+            EffectMgr *eff = (EffectMgr*)d.obj;
+            if(rtosc_narguments(msg)) {
+                int val = rtosc_argument(msg, 0).i;
+                if (val > 0) {
+                    eff->denominator = val;
+                    int Pdelay, Pfreq;
+                    float freq;
+                    if(eff->numerator) {
+                        switch(eff->nefx) {
+                        case 2: // Echo
+                            // invert:
+                            // delay = (Pdelay / 127.0f * 1.5f); //0 .. 1.5 sec
+                            Pdelay = (int)roundf((20320.0f / (float)eff->time->tempo) * 
+                                                 ((float)eff->numerator / (float)eff->denominator));
+                            if (eff->numerator&&eff->denominator)
+                                eff->seteffectparrt(2, Pdelay);
+                            break;
+                        case 3: // Chorus
+                        case 4: // Phaser
+                        case 5: // Alienwah
+                        case 8: // DynamicFilter
+                            freq =  ((float)eff->time->tempo * 
+                                     (float)eff->denominator / 
+                                     (240.0f * (float)eff->numerator));
+                            // invert:
+                            // (powf(2.0f, Pfreq / 127.0f * 10.0f) - 1.0f) * 0.03f
+                            Pfreq = (int)roundf(logf((freq/0.03f)+1.0f)/LOG_2 * 12.7f);
+                            if (eff->numerator&&eff->denominator)
+                                eff->seteffectparrt(2, Pfreq);
+                            break;
+                        case 1: // Reverb
+                        case 6: // Distorsion
+                        case 7: // EQ
+                        default:
+                            break;
+                        }
+                    }
+                }
+                d.broadcast(d.loc, "i", val);
+            } else {
+                d.reply(d.loc, "i", eff->denominator); 
+            }
+        }},
     {"eq-coeffs:", rProp(internal) rDoc("Get equalizer Coefficients"), NULL,
         [](const char *, rtosc::RtData &d)
         {
@@ -149,7 +244,7 @@ static const rtosc::Ports local_ports = {
             std::swap(eff->efxoutl, eff_->efxoutl);
             std::swap(eff->efxoutr, eff_->efxoutr);
 
-            //Return the old data for distruction
+            //Return the old data for destruction
             d.reply("/free", "sb", "EffectMgr", sizeof(EffectMgr*), &eff_);
         }},
     rSubtype(Alienwah),
@@ -173,6 +268,8 @@ EffectMgr::EffectMgr(Allocator &alloc, const SYNTH_T &synth_,
       nefx(0),
       efx(NULL),
       time(time_),
+      numerator(0),
+      denominator(4),
       dryonly(false),
       memory(alloc),
       synth(synth_)
@@ -180,7 +277,7 @@ EffectMgr::EffectMgr(Allocator &alloc, const SYNTH_T &synth_,
     setpresettype("Peffect");
     memset(efxoutl, 0, synth.bufferbytes);
     memset(efxoutr, 0, synth.bufferbytes);
-    memset(settings, 0, sizeof(settings));
+    memset(settings, 255, sizeof(settings));
     defaults();
 }
 
@@ -236,20 +333,55 @@ void EffectMgr::changeeffectrt(int _nefx, bool avoidSmash)
                 efx = memory.alloc<EQ>(pars);
                 break;
             case 8:
-                efx = memory.alloc<DynamicFilter>(pars, time);
+                efx = memory.alloc<DynamicFilter>(pars);
                 break;
             //put more effect here
             default:
                 efx = NULL;
                 break; //no effect (thru)
         }
+        
+        // set freq / delay params according to bpm ratio 
+        int Pdelay, Pfreq;
+        float freq;
+        if (numerator>0) {
+            switch(nefx) {
+                case 2: // Echo
+                    // invert:
+                    // delay = (Pdelay / 127.0f * 1.5f); //0 .. 1.5 sec
+                    Pdelay = (int)roundf((20320.0f / (float)time->tempo) * 
+                                         ((float)numerator / (float)denominator));
+                    if (numerator&&denominator)
+                        seteffectparrt(2, Pdelay);
+                    break;
+                case 3: // Chorus
+                case 4: // Phaser
+                case 5: // Alienwah
+                case 8: // DynamicFilter
+                    freq =  ((float)time->tempo * 
+                             (float)denominator / 
+                             (240.0f * (float)numerator));
+                    // invert:
+                    // (powf(2.0f, Pfreq / 127.0f * 10.0f) - 1.0f) * 0.03f
+                    Pfreq = (int)roundf(logf((freq/0.03f)+1.0f)/LOG_2 * 12.7f);
+                    if (numerator&&denominator)
+                        seteffectparrt(2, Pfreq);
+                    break;
+                case 1: // Reverb
+                case 6: // Distorsion
+                case 7: // EQ
+                default:
+                    break;
+            }
+        }
+        
     } catch (std::bad_alloc &ba) {
         std::cerr << "failed to change effect " << _nefx << ": " << ba.what() << std::endl;
         return;
     }
 
     if(!avoidSmash)
-        for(int i=0; i<128; ++i)
+        for(int i = 0; i != 128; i++)
             settings[i] = geteffectparrt(i);
 }
 
@@ -257,7 +389,6 @@ void EffectMgr::changeeffect(int _nefx)
 {
     nefx = _nefx;
     //preset    = 0;
-    //memset(settings, 0, sizeof(settings));
 }
 
 //Obtain the effect number
@@ -266,14 +397,29 @@ int EffectMgr::geteffect(void)
     return nefx;
 }
 
+void EffectMgr::changesettingsrt(const short int *p_value)
+{
+    for(int i = 0; i != 128; i++) {
+        short int value = p_value[i];
+        /* check if setting is missing */
+        if(value == -1) {
+            if(efx)
+                value = efx->getpresetpar(preset, i);
+            else
+                value = 0;
+        }
+        /* update settings */
+        seteffectparrt(i, value);
+    }
+}
+
 // Initialize An Effect in RT context
 void EffectMgr::init(void)
 {
     kill();
     changeeffectrt(nefx, true);
     changepresetrt(preset, true);
-    for(int i=0; i<128; ++i)
-        seteffectparrt(i, settings[i]);
+    changesettingsrt(settings);
 }
 
 //Strip effect manager of it's realtime memory
@@ -317,15 +463,17 @@ void EffectMgr::changepresetrt(unsigned char npreset, bool avoidSmash)
     if(efx)
         efx->setpreset(npreset);
     if(!avoidSmash)
-        for(int i=0; i<128; ++i)
+        for(int i = 0; i != 128; i++)
             settings[i] = geteffectparrt(i);
 }
 
 //Change a parameter of the current effect
 void EffectMgr::seteffectparrt(int npar, unsigned char value)
 {
-    if(npar<128)
-        settings[npar] = value;
+    if(npar < 0 || npar >= 128)
+        return;
+    settings[npar] = value;
+
     if(!efx)
         return;
     try {
@@ -335,23 +483,6 @@ void EffectMgr::seteffectparrt(int npar, unsigned char value)
     }
 }
 
-//Change a parameter of the current effect
-void EffectMgr::seteffectpar(int npar, unsigned char value)
-{
-    settings[npar] = value;
-}
-
-//Get a parameter of the current effect
-unsigned char EffectMgr::geteffectpar(int npar)
-{
-    if(npar<128)
-        return settings[npar];
-
-    if(!efx)
-        return 0;
-    return efx->getpar(npar);
-}
-
 unsigned char EffectMgr::geteffectparrt(int npar)
 {
     if(!efx)
@@ -448,8 +579,7 @@ void EffectMgr::paste(EffectMgr &e)
 {
     changeeffectrt(e.nefx, true);
     changepresetrt(e.preset, true);
-    for(int i=0;i<128;++i)
-        seteffectparrt(i, e.settings[i]);
+    changesettingsrt(e.settings);
     if(dynamic_cast<DynamicFilter*>(efx)) {
         std::swap(filterpars, e.filterpars);
         efx->filterpars = filterpars;
@@ -466,14 +596,18 @@ void EffectMgr::add2XML(XMLwrapper& xml)
     xml.addpar("preset", preset);
 
     xml.beginbranch("EFFECT_PARAMETERS");
-    for(int n = 0; n < 128; ++n) {
-        int par = 0;
-        if(efx)
+    for(int n = 0; n != 128; n++) {
+        int par;
+        int def;
+        if(efx) {
             par = efx->getpar(n);
-        else if(n<128)
+            def = efx->getpresetpar(preset, n);
+        } else {
             par = settings[n];
-
-        if(par == 0)
+            def = -1;
+        }
+        /* don't store default values */
+        if(par == def)
             continue;
         xml.beginbranch("par_no", n);
         xml.addpar("par", par);
@@ -486,6 +620,8 @@ void EffectMgr::add2XML(XMLwrapper& xml)
         xml.endbranch();
     }
     xml.endbranch();
+    xml.addpar("numerator", numerator);
+    xml.addpar("denominator", denominator);
 }
 
 void EffectMgr::getfromXML(XMLwrapper& xml)
@@ -498,13 +634,31 @@ void EffectMgr::getfromXML(XMLwrapper& xml)
     preset = xml.getpar127("preset", preset);
 
     if(xml.enterbranch("EFFECT_PARAMETERS")) {
-        for(int n = 0; n < 128; ++n) {
-            seteffectpar(n, 0); //erase effect parameter
-            if(xml.enterbranch("par_no", n) == 0)
-                continue;
-            int par = geteffectpar(n);
-            seteffectpar(n, xml.getpar127("par", par));
-            xml.exitbranch();
+        for(int n = 0; n != 128; n++) {
+            if(xml.enterbranch("par_no", n) == 0) {
+                /*
+                 * XXX workaround for old presets:
+                 *
+                 * All effect parameters have a default value.
+                 * Default values are skipped when storing parameters,
+                 * and must appear as the default value when loading.
+                 *
+                 * Up until recently it was assumed that the default
+                 * value of all parameters is zero. This is no longer
+                 * true, but when loading old presets we need to
+                 * preserve this behaviour! Else sounds may change.
+                 */
+                if (xml.fileversion() < version_type(3,0,6) &&
+                    /* XXX old presets don't have DC offset */
+                    (geteffect() != 6 || n < 11)) {
+                        settings[n] = 0;
+                } else {
+                        settings[n] = -1; /* use parameter default */
+                }
+            } else {
+                settings[n] = xml.getpar127("par", 0);
+                xml.exitbranch();
+            }
         }
         assert(filterpars);
         if(xml.enterbranch("FILTER")) {
@@ -513,6 +667,8 @@ void EffectMgr::getfromXML(XMLwrapper& xml)
         }
         xml.exitbranch();
     }
+    numerator = xml.getpar("numerator", numerator, 0, 99);
+    denominator = xml.getpar("denominator", denominator, 1, 99);
     cleanup();
 }
 
