@@ -22,6 +22,8 @@
 //entries in a bank
 #define BANK_SIZE 160
 
+namespace zyn {
+
 /**The instrument Bank*/
 class Bank
 {
@@ -110,4 +112,6 @@ class Bank
         uint8_t bank_lsb;
 };
 
+}
+
 #endif
