@@ -23,6 +23,7 @@
 #include <rtosc/undo-history.h>
 #include <rtosc/thread-link.h>
 #include <rtosc/ports.h>
+#include <rtosc/port-sugar.h>
 #include <lo/lo.h>
 
 #include <unistd.h>
@@ -31,14 +32,21 @@
 #include "../UI/Fl_Osc_Interface.h"
 
 #include <map>
+#include <queue>
 
 #include "Util.h"
 #include "CallbackRepeater.h"
 #include "Master.h"
+#include "MsgParsing.h"
 #include "Part.h"
 #include "PresetExtractor.h"
 #include "../Containers/MultiPseudoStack.h"
 #include "../Params/PresetsStore.h"
+#include "../Params/EnvelopeParams.h"
+#include "../Params/LFOParams.h"
+#include "../Params/FilterParams.h"
+#include "../Effects/EffectMgr.h"
+#include "../Synth/Resonance.h"
 #include "../Params/ADnoteParameters.h"
 #include "../Params/SUBnoteParameters.h"
 #include "../Params/PADnoteParameters.h"
@@ -94,6 +102,10 @@ static void liblo_error_cb(int i, const char *m, const char *loc)
     fprintf(stderr, "liblo :-( %d-%s@%s\n",i,m,loc);
 }
 
+// we need to access this before the definitions
+// bad style?
+static const rtosc::Ports& getNonRtParamPorts();
+
 static int handler_function(const char *path, const char *types, lo_arg **argv,
         int argc, lo_message msg, void *user_data)
 {
@@ -117,21 +129,39 @@ static int handler_function(const char *path, const char *types, lo_arg **argv,
     lo_message_serialise(msg, path, buffer, &size);
 
     if(!strcmp(buffer, "/path-search") &&
-       !strcmp("ss", rtosc_argument_string(buffer)))
+       (!strcmp("ss",  rtosc_argument_string(buffer)) ||
+        !strcmp("ssT", rtosc_argument_string(buffer)) ) )
     {
+        constexpr bool debug_path_search = false;
+        if(debug_path_search) {
+            fprintf(stderr, "MW: path-search: %s, %s\n",
+                   rtosc_argument(buffer, 0).s, rtosc_argument(buffer, 1).s);
+        }
+        bool reply_with_query = rtosc_narguments(buffer) == 3;
+
         char reply_buffer[1024*20];
         std::size_t length =
-            rtosc::path_search(Master::ports, buffer, 128,
-                               reply_buffer, sizeof(reply_buffer));
+            rtosc::path_search(MiddleWare::getAllPorts(), buffer, 128,
+                               reply_buffer, sizeof(reply_buffer),
+                               rtosc::path_search_opts::sorted_and_unique_prefix,
+                               reply_with_query);
         if(length) {
             lo_message msg  = lo_message_deserialise((void*)reply_buffer,
                                                      length, NULL);
+            if(debug_path_search) {
+                fprintf(stderr, "  reply:\n");
+                lo_message_pp(msg);
+            }
             lo_address addr = lo_address_new_from_url(mw->activeUrl().c_str());
             if(addr)
                 lo_send_message(addr, reply_buffer, msg);
             lo_address_free(addr);
             lo_message_free(msg);
         }
+        else {
+            if(debug_path_search)
+                fprintf(stderr, "  -> no reply!\n");
+        }
     }
     else if(buffer[0]=='/' && strrchr(buffer, '/')[1])
     {
@@ -172,6 +202,28 @@ void deallocate(const char *str, void *v)
         delete (SclInfo*)v;
     else if(!strcmp(str, "Microtonal"))
         delete (Microtonal*)v;
+    else if(!strcmp(str, "ADnoteParameters"))
+        delete (ADnoteParameters*)v;
+    else if(!strcmp(str, "SUBnoteParameters"))
+        delete (SUBnoteParameters*)v;
+    else if(!strcmp(str, "PADnoteParameters"))
+        delete (PADnoteParameters*)v;
+    else if(!strcmp(str, "EffectMgr"))
+        delete (EffectMgr*)v;
+    else if(!strcmp(str, "EnvelopeParams"))
+        delete (EnvelopeParams*)v;
+    else if(!strcmp(str, "FilterParams"))
+        delete (FilterParams*)v;
+    else if(!strcmp(str, "LFOParams"))
+        delete (LFOParams*)v;
+    else if(!strcmp(str, "OscilGen"))
+        delete (OscilGen*)v;
+    else if(!strcmp(str, "Resonance"))
+        delete (Resonance*)v;
+    else if(!strcmp(str, "rtosc::AutomationMgr"))
+        delete (rtosc::AutomationMgr*)v;
+    else if(!strcmp(str, "PADsample"))
+        delete[] (float*)v;
     else
         fprintf(stderr, "Unknown type '%s', leaking pointer %p!!\n", str, v);
 }
@@ -181,6 +233,7 @@ void deallocate(const char *str, void *v)
  *                    PadSynth Setup                                         *
  *****************************************************************************/
 
+// This lets MiddleWare compute non-realtime PAD synth data and send it to the backend
 void preparePadSynth(string path, PADnoteParameters *p, rtosc::RtData &d)
 {
     //printf("preparing padsynth parameters\n");
@@ -189,7 +242,7 @@ void preparePadSynth(string path, PADnoteParameters *p, rtosc::RtData &d)
 
 #ifdef WIN32
     unsigned num = p->sampleGenerator([&path,&d]
-                       (unsigned N, PADnoteParameters::Sample &s)
+                       (unsigned N, PADnoteParameters::Sample &&s)
                        {
                            //printf("sending info to '%s'\n",
                            //       (path+to_s(N)).c_str());
@@ -199,11 +252,12 @@ void preparePadSynth(string path, PADnoteParameters *p, rtosc::RtData &d)
 #else
     std::mutex rtdata_mutex;
     unsigned num = p->sampleGenerator([&rtdata_mutex, &path,&d]
-                       (unsigned N, PADnoteParameters::Sample &s)
+                       (unsigned N, PADnoteParameters::Sample&& s)
                        {
                            //printf("sending info to '%s'\n",
                            //       (path+to_s(N)).c_str());
                            rtdata_mutex.lock();
+                           // send non-realtime computed data to PADnoteParameters
                            d.chain((path+to_s(N)).c_str(), "ifb",
                                    s.size, s.basefreq, sizeof(float*), &s.smp);
                            rtdata_mutex.unlock();
@@ -217,6 +271,73 @@ void preparePadSynth(string path, PADnoteParameters *p, rtosc::RtData &d)
     }
 }
 
+/******************************************************************************
+ *                      MIDI Serialization                                    *
+ *                                                                            *
+ ******************************************************************************/
+void saveMidiLearn(XMLwrapper &xml, const rtosc::MidiMappernRT &midi)
+{
+    xml.beginbranch("midi-learn");
+    for(auto value:midi.inv_map) {
+        XmlNode binding("midi-binding");
+        auto biject = std::get<3>(value.second);
+        binding["osc-path"]  = value.first;
+        binding["coarse-CC"] = to_s(std::get<1>(value.second));
+        binding["fine-CC"]   = to_s(std::get<2>(value.second));
+        binding["type"]      = "i";
+        binding["minimum"]   = to_s(biject.min);
+        binding["maximum"]   = to_s(biject.max);
+        xml.add(binding);
+    }
+    xml.endbranch();
+}
+
+void loadMidiLearn(XMLwrapper &xml, rtosc::MidiMappernRT &midi)
+{
+    using rtosc::Port;
+    if(xml.enterbranch("midi-learn")) {
+        auto nodes = xml.getBranch();
+
+        //TODO clear mapper
+
+        for(auto node:nodes) {
+            if(node.name != "midi-binding" ||
+                    !node.has("osc-path") ||
+                    !node.has("coarse-CC"))
+                continue;
+            const string path = node["osc-path"];
+            const int    CC   = atoi(node["coarse-CC"].c_str());
+            const Port  *p    = Master::ports.apropos(path.c_str());
+            if(p) {
+                printf("loading midi port...\n");
+                midi.addNewMapper(CC, *p, path);
+            } else {
+                printf("unknown midi bindable <%s>\n", path.c_str());
+            }
+        }
+        xml.exitbranch();
+    } else
+        printf("cannot find 'midi-learn' branch...\n");
+}
+
+void connectMidiLearn(int par, int chan, bool isNrpn, string path, rtosc::MidiMappernRT &midi)
+{
+    const rtosc::Port *p = Master::ports.apropos(path.c_str());
+    if(p) {
+        if(isNrpn)
+            printf("mapping midi NRPN: %d, CH: %d to Port: %s\n", par, chan, path.c_str());
+        else
+            printf("mapping midi CC: %d, CH: %d to Port: %s\n", par, chan, path.c_str());
+            
+        if(chan<1) chan=1;
+        int ID = (isNrpn<<18) + (((chan-1)&0x0f)<<14) + par;
+        //~ printf("ID = %d\n", ID);
+
+        midi.addNewMapper(ID, *p, path);
+    } else {
+        printf("unknown port to midi bind <%s>\n", path.c_str());
+    }
+}
 /******************************************************************************
  *                      Non-RealTime Object Store                             *
  *                                                                            *
@@ -258,8 +379,8 @@ struct NonRtObjStore
             std::string nbase = base+"adpars/VoicePar"+to_s(k)+"/";
             if(adpars) {
                 auto &nobj = adpars->VoicePar[k];
-                objmap[nbase+"OscilSmp/"]     = nobj.OscilSmp;
-                objmap[nbase+"FMSmp/"] = nobj.FMSmp;
+                objmap[nbase+"OscilSmp/"] = nobj.OscilGn;
+                objmap[nbase+"FMSmp/"]    = nobj.FmGn;
             } else {
                 objmap[nbase+"OscilSmp/"] = nullptr;
                 objmap[nbase+"FMSmp/"]    = nullptr;
@@ -298,6 +419,10 @@ struct NonRtObjStore
 
     void handleOscil(const char *msg, rtosc::RtData &d) {
         string obj_rl(d.message, msg);
+        assert(d.message);
+        assert(msg);
+        assert(msg >= d.message);
+        assert(msg - d.message < 256);
         void *osc = get(obj_rl);
         if(osc)
         {
@@ -305,9 +430,15 @@ struct NonRtObjStore
             d.obj = osc;
             OscilGen::non_realtime_ports.dispatch(msg, d);
         }
-        else
-            fprintf(stderr, "Warning: trying to access oscil object \"%s\","
-                            "which does not exist\n", obj_rl.c_str());
+        else {
+            // print warning, except in rtosc::walk_ports
+            if(!strstr(d.message, "/pointer"))
+            {
+                fprintf(stderr, "Warning: trying to access oscil object \"%s\","
+                                "which does not exist\n", obj_rl.c_str());
+            }
+            d.obj = nullptr; // tell walk_ports that there's nothing to recurse here...
+        }
     }
     void handlePad(const char *msg, rtosc::RtData &d) {
         string obj_rl(d.message, msg);
@@ -322,7 +453,7 @@ struct NonRtObjStore
                 strcpy(d.loc, obj_rl.c_str());
                 d.obj = pad;
                 PADnoteParameters::non_realtime_ports.dispatch(msg, d);
-                if(rtosc_narguments(msg)) {
+                if(d.matches && rtosc_narguments(msg)) {
                     if(!strcmp(msg, "oscilgen/prepare"))
                         ; //ignore
                     else {
@@ -330,10 +461,16 @@ struct NonRtObjStore
                     }
                 }
             }
-            else
-                fprintf(stderr, "Warning: trying to access pad synth object "
-                                "\"%s\", which does not exist\n",
-                        obj_rl.c_str());
+            else {
+                // print warning, except in rtosc::walk_ports
+                if(!strstr(d.message, "/pointer"))
+                {
+                    fprintf(stderr, "Warning: trying to access pad synth object "
+                                    "\"%s\", which does not exist\n",
+                            obj_rl.c_str());
+                }
+                d.obj = nullptr; // tell walk_ports that there's nothing to recurse here...
+            }
         }
     }
 };
@@ -403,6 +540,11 @@ namespace Nio
                     d.reply(d.loc, "s", Nio::getSink().c_str());
                 else
                     Nio::setSink(rtosc_argument(msg,0).s);}},
+        {"audio-compressor::T:F", 0, 0, [](const char *msg, rtosc::RtData &d) {
+                if(rtosc_narguments(msg) == 0)
+                    d.reply(d.loc, Nio::getAudioCompressor() ? "T" : "F");
+                else
+                    Nio::setAudioCompressor(rtosc_argument(msg,0).T);}},
     };
 }
 
@@ -426,15 +568,16 @@ public:
 
 class MiddleWareImpl
 {
-    public:
-    MiddleWare *parent;
-    private:
-
+    // messages chained with MwDataObj::chain
+    // must yet be handled after a previous handleMsg
+    std::queue<std::vector<char>> msgsToHandle;
 public:
+    MiddleWare *parent;
     Config* const config;
     MiddleWareImpl(MiddleWare *mw, SYNTH_T synth, Config* config,
                    int preferred_port);
     ~MiddleWareImpl(void);
+    void recreateMinimalMaster();
 
     //Check offline vs online mode in plugins
     void heartBeat(Master *m);
@@ -473,6 +616,7 @@ public:
         if(actual_load[npart] != pending_load[npart])
             return;
         assert(actual_load[npart] <= pending_load[npart]);
+        assert(filename);
 
         //load part in async fashion when possible
 #ifndef WIN32
@@ -578,6 +722,7 @@ public:
         //Update resource locator table
         updateResources(m);
 
+        previous_master = master;
         master = m;
 
         //Give it to the backend and wait for the old part to return for
@@ -586,7 +731,11 @@ public:
         return 0;
     }
 
-    int saveMaster(const char *filename, bool osc_format = false)
+    // Save all possible parameters
+    // In user language, this is called "saving a master", but we
+    // are saving parameters owned by Master and by MiddleWare
+    // Return 0 if OK, <0 if not
+    int saveParams(const char *filename, bool osc_format = false)
     {
         int res;
         if(osc_format)
@@ -597,17 +746,109 @@ public:
             // after the savefile will have been saved, it will be loaded into this
             // dummy master, and then the two masters will be compared
             zyn::Config config;
+            config.cfg.SaveFullXml = master->SaveFullXml;
+
             zyn::SYNTH_T* synth = new zyn::SYNTH_T;
             synth->buffersize = master->synth.buffersize;
             synth->samplerate = master->synth.samplerate;
             synth->alias();
+
             zyn::Master master2(*synth, &config);
             master->copyMasterCbTo(&master2);
             master2.frozenState = true;
 
-            doReadOnlyOp([this,filename,&dispatcher,&master2,&res](){
-                             res = master->saveOSC(filename, &dispatcher,
-                                                   &master2);});
+            std::string savefile;
+            rtosc_version m_version =
+            {
+                (unsigned char) version.get_major(),
+                (unsigned char) version.get_minor(),
+                (unsigned char) version.get_revision()
+            };
+            savefile = rtosc::save_to_file(getNonRtParamPorts(), this, "ZynAddSubFX", m_version);
+            savefile += '\n';
+
+            doReadOnlyOp([this,filename,&dispatcher,&master2,&savefile,&res]()
+            {
+                savefile = master->saveOSC(savefile);
+#if 1
+                // load the savefile string into another master to compare the results
+                // between the original and the savefile-loaded master
+                // this requires a temporary master switch
+                Master* old_master = master;
+                dispatcher.updateMaster(&master2);
+
+                res = master2.loadOSCFromStr(savefile.c_str(), &dispatcher);
+                // TODO: compare MiddleWare, too?
+
+                // The above call is done by this thread (i.e. the MiddleWare thread), but
+                // it sends messages to master2 in order to load the values
+                // We need to wait until savefile has been loaded into master2
+                int i;
+                for(i = 0; i < 20 && master2.uToB->hasNext(); ++i)
+                    os_usleep(50000);
+                if(i >= 20) // >= 1 second?
+                {
+                    // Master failed to fetch its messages
+                    res = -1;
+                }
+                printf("Saved in less than %d ms.\n", 50*i);
+
+                dispatcher.updateMaster(old_master);
+#endif
+                if(res < 0)
+                {
+                    std::cerr << "invalid savefile (or a backend error)!" << std::endl;
+                    std::cerr << "complete savefile:" << std::endl;
+                    std::cerr << savefile << std::endl;
+                    std::cerr << "first entry that could not be parsed:" << std::endl;
+
+                    for(int i = -res + 1; savefile[i]; ++i)
+                    if(savefile[i] == '\n')
+                    {
+                        savefile.resize(i);
+                        break;
+                    }
+                    std::cerr << (savefile.c_str() - res) << std::endl;
+
+                    res = -1;
+                }
+                else
+                {
+                    char* xml = master->getXMLData(),
+                        * xml2 = master2.getXMLData();
+                    // TODO: below here can be moved out of read only op
+
+                    res = strcmp(xml, xml2) ? -1 : 0;
+
+                    if(res == 0)
+                    {
+                        if(filename && *filename)
+                        {
+                            std::ofstream ofs(filename);
+                            ofs << savefile;
+                        }
+                        else {
+                            std::cout << "The savefile content follows" << std::endl;
+                            std::cout << "---->8----" << std::endl;
+                            std::cout << savefile << std::endl;
+                            std::cout << "---->8----" << std::endl;
+                        }
+                    }
+                    else
+                    {
+                        std::cout << savefile << std::endl;
+                        std::cerr << "Can not write OSC savefile!! (see tmp1.txt and tmp2.txt)"
+                                  << std::endl;
+                        std::ofstream tmp1("tmp1.txt"), tmp2("tmp2.txt");
+                        tmp1 << xml;
+                        tmp2 << xml2;
+                        res = -1;
+                    }
+
+                    free(xml);
+                    free(xml2);
+                }
+            });
         }
         else // xml format
         {
@@ -696,10 +937,13 @@ public:
 
         heartBeat(master);
 
-        //XXX This might have problems with a master swap operation
         if(offline)
-            master->runOSC(0,0,true);
-
+        {
+            //pass previous master in case it will have to be freed
+            //similar to previous_master->runOSC(0,0,true)
+            //but note that previous_master could have been freed already
+            master->runOSC(0,0,true, previous_master);
+        }
     }
 
 
@@ -707,7 +951,13 @@ public:
     void kitEnable(int part, int kit, int type);
 
     // Handle an event with special cases
-    void handleMsg(const char *msg);
+    void handleMsg(const char *msg, bool msg_comes_from_realtime = false);
+
+    // Add a message for handleMsg to a queue
+    void queueMsg(const char* msg)
+    {
+        msgsToHandle.emplace(msg, msg+rtosc_message_length(msg, -1));
+    }
 
     void write(const char *path, const char *args, ...);
     void write(const char *path, const char *args, va_list va);
@@ -741,6 +991,10 @@ public:
     //this assumption is broken
     Master *master;
 
+    //The master before the last load operation, if any
+    //Only valid until freed
+    Master *previous_master = nullptr;
+
     //The ONLY means that any chunk of UI code should have for interacting with the
     //backend
     Fl_Osc_Interface *osc;
@@ -763,7 +1017,7 @@ public:
     rtosc::UndoHistory undo;
 
     //MIDI Learn
-    //rtosc::MidiMappernRT midi_mapper;
+    rtosc::MidiMappernRT midi_mapper;
 
     //Link To the Realtime
     rtosc::ThreadLink *bToU;
@@ -778,7 +1032,7 @@ public:
     std::set<string> known_remotes;
 
     //Synthesis Rate Parameters
-    const SYNTH_T synth;
+    SYNTH_T synth;
 
     PresetsStore presetsstore;
 
@@ -859,7 +1113,7 @@ class MwDataObj:public rtosc::RtData
         {
             assert(msg);
             // printf("chain call on <%s>\n", msg);
-            mwi->handleMsg(msg);
+            mwi->queueMsg(msg);
         }
 
         virtual void chain(const char *path, const char *args, ...) override
@@ -945,7 +1199,7 @@ static int extractInt(const char *msg)
 
 static const char *chomp(const char *msg)
 {
-    while(*msg && *msg!='/') ++msg; \
+    while(*msg && *msg!='/') ++msg;
     msg = *msg ? msg+1 : msg;
     return msg;
 };
@@ -991,6 +1245,7 @@ const rtosc::Ports bankPorts = {
                 d.reply("/bankview", "iss", i, "", "");
             }
         }
+        d.broadcast("/damage", "s", "/bank/");
         rEnd},
     {"bank_list:", 0, 0,
         rBegin;
@@ -1215,11 +1470,32 @@ void save_cb(const char *msg, RtData &d)
     if(rtosc_narguments(msg) > 1)
         request_time = rtosc_argument(msg, 1).t;
 
-    int res = impl.saveMaster(file.c_str(), osc_format);
+    int res = impl.saveParams(file.c_str(), osc_format);
     d.broadcast(d.loc, (res == 0) ? "stT" : "stF",
                 file.c_str(), request_time);
 }
 
+#if defined(__GNUC__) && !defined(__clang__) && !defined(__INTEL_COMPILER)
+#pragma GCC push_options
+#pragma GCC optimize("O0")
+#endif
+
+void gcc_10_1_0_is_dumb(const std::vector<std::string> &files,
+        const int N,
+        char *types,
+        rtosc_arg_t *args)
+{
+        types[N] = 0;
+        for(int i=0; i<N; ++i) {
+            args[i].s = files[i].c_str();
+            types[i]  = 's';
+        }
+}
+
+#if defined(__GNUC__) && !defined(__clang__) && !defined(__INTEL_COMPILER)
+#pragma GCC pop_options
+#endif
+
 /*
  * BASE/part#/kititem#
  * BASE/part#/kit#/adpars/voice#/oscil/\*
@@ -1227,7 +1503,7 @@ void save_cb(const char *msg, RtData &d)
  * BASE/part#/kit#/padpars/prepare
  * BASE/part#/kit#/padpars/oscil/\*
  */
-static rtosc::Ports middwareSnoopPorts = {
+static rtosc::Ports nonRtParamPorts = {
     {"part#" STRINGIFY(NUM_MIDI_PARTS)
         "/kit#" STRINGIFY(NUM_KIT_ITEMS) "/adpars/VoicePar#"
             STRINGIFY(NUM_VOICES) "/OscilSmp/", 0, &OscilGen::non_realtime_ports,
@@ -1245,6 +1521,9 @@ static rtosc::Ports middwareSnoopPorts = {
         rBegin
         impl.obj_store.handlePad(chomp(chomp(chomp(msg))), d);
         rEnd},
+};
+
+static rtosc::Ports middwareSnoopPortsWithoutNonRtParams = {
     {"bank/", 0, &bankPorts,
         rBegin;
         d.obj = &impl.master->bank;
@@ -1264,6 +1543,7 @@ static rtosc::Ports middwareSnoopPorts = {
                     "Failed To Save To Bank Slot, please check file permissions");
             GUI::raiseUi(impl.ui, buffer);
         }
+        else d.broadcast("/damage", "s", "/bank/search_results/");
         rEnd},
     {"config/", 0, &Config::ports,
         rBegin;
@@ -1275,7 +1555,7 @@ static rtosc::Ports middwareSnoopPorts = {
         d.obj = (void*)obj->parent;
         real_preset_ports.dispatch(chomp(msg), d);
         if(strstr(msg, "paste") && rtosc_argument_string(msg)[0] == 's')
-            d.reply("/damage", "s", rtosc_argument(msg, 0).s);
+            d.broadcast("/damage", "s", rtosc_argument(msg, 0).s);
         }},
     {"io/", 0, &Nio::ports,               [](const char *msg, RtData &d) {
         Nio::ports.dispatch(chomp(msg), d);}},
@@ -1284,6 +1564,44 @@ static rtosc::Ports middwareSnoopPorts = {
         impl.kitEnable(msg);
         d.forward();
         rEnd},
+    {"save_xcz:s", 0, 0,
+        rBegin;
+        const char *file = rtosc_argument(msg, 0).s;
+        XMLwrapper xml;
+        saveMidiLearn(xml, impl.midi_mapper);
+        xml.saveXMLfile(file, impl.master->gzip_compression);
+        rEnd},
+    {"load_xcz:s", 0, 0,
+        rBegin;
+        const char *file = rtosc_argument(msg, 0).s;
+        XMLwrapper xml;
+        xml.loadXMLfile(file);
+        loadMidiLearn(xml, impl.midi_mapper);
+        rEnd},
+    {"clear_xcz:", 0, 0,
+        rBegin;
+        impl.midi_mapper.clear();
+        rEnd},
+    {"midi-map-cc:is", rDoc("bind a midi CC on CH to an OSC path"), 0,
+        rBegin;
+        const int par = rtosc_argument(msg, 0).i;
+        const string path = rtosc_argument(msg, 1).s;
+        connectMidiLearn(par, 1, false, path, impl.midi_mapper);
+        rEnd},
+    {"midi-map-cc:iis", rDoc("bind a midi CC on CH to an OSC path"), 0,
+        rBegin;
+        const int par = rtosc_argument(msg, 0).i;
+        const int ch = rtosc_argument(msg, 1).i;
+        const string path = rtosc_argument(msg, 2).s;
+        connectMidiLearn(par, ch, false, path, impl.midi_mapper);
+        rEnd},
+    {"midi-map-nrpn:iis", rDoc("bind nrpn on channel to an OSC path"), 0,
+        rBegin;
+        const int par = rtosc_argument(msg, 0).i;
+        const int ch = rtosc_argument(msg, 1).i;
+        const string path = rtosc_argument(msg, 2).s;
+        connectMidiLearn(par, ch, true, path, impl.midi_mapper);
+        rEnd},
     {"save_xlz:s", 0, 0,
         rBegin;
         impl.doReadOnlyOp([&]() {
@@ -1318,7 +1636,7 @@ static rtosc::Ports middwareSnoopPorts = {
         const char *file = rtosc_argument(msg, 0).s;
         impl.saveXsz(file, d);
         rEnd},
-    {"load_scl:s", 0, 0,
+    {"load_scl:s", rDoc("Load a scale from a file"), 0,
         rBegin;
         const char *file = rtosc_argument(msg, 0).s;
         impl.loadScl(file, d);
@@ -1364,11 +1682,7 @@ static rtosc::Ports middwareSnoopPorts = {
         const int N = files.size();
         rtosc_arg_t *args  = new rtosc_arg_t[N];
         char        *types = new char[N+1];
-        types[N] = 0;
-        for(int i=0; i<N; ++i) {
-            args[i].s = files[i].c_str();
-            types[i]  = 's';
-        }
+        gcc_10_1_0_is_dumb(files, N, types, args);
 
         d.replyArray(d.loc, types, args);
         delete [] types;
@@ -1383,11 +1697,7 @@ static rtosc::Ports middwareSnoopPorts = {
         const int N = files.size();
         rtosc_arg_t *args  = new rtosc_arg_t[N];
         char        *types = new char[N+1];
-        types[N] = 0;
-        for(int i=0; i<N; ++i) {
-            args[i].s = files[i].c_str();
-            types[i]  = 's';
-        }
+        gcc_10_1_0_is_dumb(files, N, types, args);
 
         d.replyArray(d.loc, types, args);
         delete [] types;
@@ -1417,7 +1727,7 @@ static rtosc::Ports middwareSnoopPorts = {
     {"reset_master:", 0, 0,
         rBegin;
         impl.loadMaster(NULL);
-        d.reply("/damage", "s", "/");
+        d.broadcast("/damage", "s", "/");
         rEnd},
     {"load_xiz:is", 0, 0,
         rBegin;
@@ -1457,7 +1767,7 @@ static rtosc::Ports middwareSnoopPorts = {
         rBegin;
         int id = extractInt(msg);
         impl.loadClearPart(id);
-        d.reply("/damage", "s", ("/part"+to_s(id)).c_str());
+        d.broadcast("/damage", "s", ("/part"+to_s(id)).c_str());
         rEnd},
     {"undo:", 0, 0,
         rBegin;
@@ -1468,56 +1778,99 @@ static rtosc::Ports middwareSnoopPorts = {
         impl.undo.seekHistory(+1);
         rEnd},
     //port to observe the midi mappings
-    //{"midi-learn-values:", 0, 0,
-    //    rBegin;
-    //    auto &midi  = impl.midi_mapper;
-    //    auto  key   = keys(midi.inv_map);
-    //    //cc-id, path, min, max
-//#define MAX_MIDI 32
-    //    rtosc_arg_t args[MAX_MIDI*4];
-    //    char        argt[MAX_MIDI*4+1] = {};
-    //    int j=0;
-    //    for(unsigned i=0; i<key.size() && i<MAX_MIDI; ++i) {
-    //        auto val = midi.inv_map[key[i]];
-    //        if(std::get<1>(val) == -1)
-    //            continue;
-    //        argt[4*j+0]   = 'i';
-    //        args[4*j+0].i = std::get<1>(val);
-    //        argt[4*j+1]   = 's';
-    //        args[4*j+1].s = key[i].c_str();
-    //        argt[4*j+2]   = 'i';
-    //        args[4*j+2].i = 0;
-    //        argt[4*j+3]   = 'i';
-    //        args[4*j+3].i = 127;
-    //        j++;
+    {"mlearn-values:", 0, 0,
+        rBegin;
+        auto &midi  = impl.midi_mapper;
+        auto  key   = keys(midi.inv_map);
+        //cc-id, path, min, max
+#define MAX_MIDI 32
+        rtosc_arg_t args[MAX_MIDI*4];
+        char        argt[MAX_MIDI*4+1] = {};
+        int j=0;
+        for(unsigned i=0; i<key.size() && i<MAX_MIDI; ++i) {
+            auto par = midi.inv_map[key[i]];
+            if(std::get<1>(par) == -1)
+                continue;
+            auto bounds = midi.getBounds(key[i].c_str());
+            argt[4*j+0]   = 'i';
+            args[4*j+0].i = std::get<1>(par);
+            argt[4*j+1]   = 's';
+            args[4*j+1].s = key[i].c_str();
+            argt[4*j+2]   = 'f';
+            args[4*j+2].f = std::get<0>(bounds);
+            argt[4*j+3]   = 'f';
+            args[4*j+3].f = std::get<1>(bounds);
+            j++;
 
-    //    }
-    //    d.replyArray(d.loc, argt, args);
-//#undef  MAX_MIDI
-    //    rEnd},
-    //{"learn:s", 0, 0,
-    //    rBegin;
-    //    string addr = rtosc_argument(msg, 0).s;
-    //    auto &midi  = impl.midi_mapper;
-    //    auto map    = midi.getMidiMappingStrings();
-    //    if(map.find(addr) != map.end())
-    //        midi.map(addr.c_str(), false);
-    //    else
-    //        midi.map(addr.c_str(), true);
-    //    rEnd},
-    //{"unlearn:s", 0, 0,
-    //    rBegin;
-    //    string addr = rtosc_argument(msg, 0).s;
-    //    auto &midi  = impl.midi_mapper;
-    //    auto map    = midi.getMidiMappingStrings();
-    //    midi.unMap(addr.c_str(), false);
-    //    midi.unMap(addr.c_str(), true);
-    //    rEnd},
+        }
+        d.replyArray(d.loc, argt, args);
+#undef  MAX_MIDI
+        rEnd},
+    {"mlearn:s", 0, 0,
+        rBegin;
+        string addr = rtosc_argument(msg, 0).s;
+        auto &midi  = impl.midi_mapper;
+        auto map    = midi.getMidiMappingStrings();
+        if(map.find(addr) != map.end())
+            midi.map(addr.c_str(), false);
+        else
+            midi.map(addr.c_str(), true);
+        rEnd},
+    {"munlearn:s", 0, 0,
+        rBegin;
+        string addr = rtosc_argument(msg, 0).s;
+        auto &midi  = impl.midi_mapper;
+        auto map    = midi.getMidiMappingStrings();
+        midi.unMap(addr.c_str(), false);
+        midi.unMap(addr.c_str(), true);
+        rEnd},
     //drop this message into the abyss
-    {"ui/title:", 0, 0, [](const char *msg, RtData &d) {}},
-    {"quit:", 0, 0, [](const char *, RtData&) {Pexitprogram = 1;}},
+    {"ui/title:", 0, 0, [](const char *, RtData &) {}},
+    {"quit:", rDoc("Stops the Zynaddsubfx process"),
+     0, [](const char *, RtData&) {Pexitprogram = 1;}},
+    // may only be called when Master is not being run
+    {"change-synth:iiit", 0, 0,
+        rBegin
+        // save all data, overwrite all params defining SYNTH,
+        // restart the master and load all data back into it
+
+        char* data = nullptr;
+        impl.master->getalldata(&data);
+        delete impl.master;
+
+        impl.synth.samplerate = (unsigned)rtosc_argument(msg, 0).i;
+        impl.synth.buffersize = rtosc_argument(msg, 1).i;
+        impl.synth.oscilsize = rtosc_argument(msg, 2).i;
+        impl.synth.alias();
+
+        impl.recreateMinimalMaster();
+        impl.master->defaults();
+        impl.master->putalldata(data);
+        impl.master->applyparameters();
+        impl.master->initialize_rt();
+        impl.updateResources(impl.master);
+
+        d.broadcast("/change-synth", "t", rtosc_argument(msg, 3).t);
+        rEnd
+    }
 };
 
+static rtosc::MergePorts middwareSnoopPorts =
+{
+    &nonRtParamPorts,
+    &middwareSnoopPortsWithoutNonRtParams
+};
+
+const rtosc::MergePorts allPorts =
+{
+    // order is important: params should be queried on Master first
+    // (because MiddleWare often just redirects, hiding the metadata)
+    &Master::ports,
+    &middwareSnoopPorts
+};
+const rtosc::Ports& getNonRtParamPorts() { return nonRtParamPorts; }
+const rtosc::MergePorts& MiddleWare::getAllPorts() { return allPorts; }
+
 static rtosc::Ports middlewareReplyPorts = {
     {"echo:ss", 0, 0,
         rBegin;
@@ -1545,8 +1898,15 @@ static rtosc::Ports middlewareReplyPorts = {
         Bank &bank        = impl.master->bank;
         const int part    = rtosc_argument(msg, 0).i;
         const int program = rtosc_argument(msg, 1).i + 128*bank.bank_lsb;
-        impl.loadPart(part, impl.master->bank.ins[program].filename.c_str(), impl.master, d);
-        impl.uToB->write(("/part"+to_s(part)+"/Pname").c_str(), "s", impl.master->bank.ins[program].name.c_str());
+        if (program >= BANK_SIZE) {
+            fprintf(stderr, "bank:program number %d:%d is out of range.",
+                    program >> 7, program & 0x7f);
+            return;
+        }
+        const char *fn  = impl.master->bank.ins[program].filename.c_str();
+        impl.loadPart(part, fn, impl.master, d);
+        impl.uToB->write(("/part"+to_s(part)+"/Pname").c_str(), "s",
+                         fn ? impl.master->bank.ins[program].name.c_str() : "");
         rEnd},
     {"setbank:c", 0, 0,
         rBegin;
@@ -1559,6 +1919,10 @@ static rtosc::Ports middlewareReplyPorts = {
         if(impl.recording_undo)
             impl.undo.recordEvent(msg);
         rEnd},
+    {"midi-use-CC:i", 0, 0,
+        rBegin;
+        impl.midi_mapper.useFreeID(rtosc_argument(msg, 0).i);
+        rEnd},
     {"broadcast:", 0, 0, rBegin; impl.broadcast = true; rEnd},
     {"forward:", 0, 0, rBegin; impl.forward = true; rEnd},
 };
@@ -1583,8 +1947,8 @@ MiddleWareImpl::MiddleWareImpl(MiddleWare *mw, SYNTH_T synth_,
 {
     bToU = new rtosc::ThreadLink(4096*2*16,1024/16);
     uToB = new rtosc::ThreadLink(4096*2*16,1024/16);
-    //midi_mapper.base_ports = &Master::ports;
-    //midi_mapper.rt_cb      = [this](const char *msg){handleMsg(msg);};
+    midi_mapper.base_ports = &Master::ports;
+    midi_mapper.rt_cb      = [this](const char *msg){handleMsg(msg);};
     if(preferrred_port != -1)
         server = lo_server_new_with_proto(to_s(preferrred_port).c_str(),
                                           LO_UDP, liblo_error_cb);
@@ -1603,9 +1967,7 @@ MiddleWareImpl::MiddleWareImpl(MiddleWare *mw, SYNTH_T synth_,
     idle = 0;
     idle_ptr = 0;
 
-    master = new Master(synth, config);
-    master->bToU = bToU;
-    master->uToB = uToB;
+    recreateMinimalMaster();
     osc    = GUI::genOscInterface(mw);
 
     //Grab objects of interest from master
@@ -1650,6 +2012,13 @@ MiddleWareImpl::~MiddleWareImpl(void)
 
 }
 
+void zyn::MiddleWareImpl::recreateMinimalMaster()
+{
+    master = new Master(synth, config);
+    master->bToU = bToU;
+    master->uToB = uToB;
+}
+
 /** Threading When Saving
  *  ----------------------
  *
@@ -1657,8 +2026,8 @@ MiddleWareImpl::~MiddleWareImpl(void)
  *   1) Middleware sends /freeze_state to backend
  *   2) Middleware waits on /state_frozen from backend
  *      All intervening commands are held for out of order execution
- *   3) Aquire memory
- *      At this time by the memory barrier we are guarenteed that all old
+ *   3) Acquire memory
+ *      At this time by the memory barrier we are guaranteed that all old
  *      writes are done and assuming the freezing logic is sound, then it is
  *      impossible for any other parameter to change at this time
  *   3) Middleware performs saving operation
@@ -1886,7 +2255,7 @@ void MiddleWareImpl::bToUhandle(const char *rtmsg)
     assert(strcmp(rtmsg, "/part0/kit0/Ppadenableda"));
     assert(strcmp(rtmsg, "/ze_state"));
 
-    //Dump Incomming Events For Debugging
+    //Dump Incoming Events For Debugging
     if(strcmp(rtmsg, "/vu-meter") && false) {
         fprintf(stdout, "%c[%d;%d;%dm", 0x1B, 0, 1 + 30, 0 + 40);
         fprintf(stdout, "frontend[%c]: '%s'<%s>\n", forward?'f':broadcast?'b':'N',
@@ -1910,7 +2279,7 @@ void MiddleWareImpl::bToUhandle(const char *rtmsg)
     if(d.matches == 0) {
         if(forward) {
             forward = false;
-            handleMsg(rtmsg);
+            handleMsg(rtmsg, true);
         } if(broadcast)
             broadcastToRemote(rtmsg);
         else
@@ -1938,20 +2307,9 @@ void MiddleWareImpl::kitEnable(const char *msg)
     else
         return;
 
-    const char *tmp = strstr(msg, "part");
-
-    if(tmp == NULL)
-        return;
-
-    const int part = atoi(tmp+4);
-
-    tmp = strstr(msg, "kit");
-
-    if(tmp == NULL)
-        return;
-
-    const int kit = atoi(tmp+3);
-
+    int part, kit;
+    bool res = idsFromMsg(msg, &part, &kit, nullptr);
+    assert(res);
     kitEnable(part, kit, type);
 }
 
@@ -1984,7 +2342,7 @@ void MiddleWareImpl::kitEnable(int part, int kit, int type)
 /*
  * Handle all messages traveling to the realtime side.
  */
-void MiddleWareImpl::handleMsg(const char *msg)
+void MiddleWareImpl::handleMsg(const char *msg, bool msg_comes_from_realtime)
 {
     //Check for known bugs
     assert(msg && *msg && strrchr(msg, '/')[1]);
@@ -2014,13 +2372,27 @@ void MiddleWareImpl::handleMsg(const char *msg)
 
     //A message unmodified by snooping
     if(d.matches == 0 || d.forwarded) {
-        //if(strcmp("/get-vu", msg)) {
-        //    printf("Message Continuing on<%s:%s>...\n", msg, rtosc_argument_string(msg));
-        //}
-        uToB->raw_write(msg);
+        if(msg_comes_from_realtime) {
+            // don't reply the same msg to realtime - avoid cycles
+            //printf("Message from RT will not be replied to RT: <%s:%s>...\n",
+            //       msg, rtosc_argument_string(msg));
+        } else {
+            //if(strcmp("/get-vu", msg)) {
+            //    printf("Message Continuing on<%s:%s>...\n", msg, rtosc_argument_string(msg));
+            //}
+            uToB->raw_write(msg);
+        }
     } else {
         //printf("Message Handled<%s:%s>...\n", msg, rtosc_argument_string(msg));
     }
+
+    // now handle all chained messages
+    while(!msgsToHandle.empty())
+    {
+        std::vector<char> front = msgsToHandle.front();
+        msgsToHandle.pop();
+        handleMsg(front.data());
+    }
 }
 
 void MiddleWareImpl::write(const char *path, const char *args, ...)
@@ -2076,7 +2448,7 @@ void MiddleWare::enableAutoSave(int interval_sec)
     impl->autoSave.dt = interval_sec;
 }
 
-int MiddleWare::checkAutoSave(void)
+int MiddleWare::checkAutoSave(void) const
 {
     //save spec zynaddsubfx-PID-autosave.xmz
     const std::string home     = getenv("HOME");
@@ -2208,7 +2580,12 @@ void MiddleWare::pendingSetProgram(int part, int program)
     impl->bToU->write("/setprogram", "cc", part, program);
 }
 
-std::string MiddleWare::activeUrl(void)
+std::string zyn::MiddleWare::getProgramName(int program) const
+{
+    return impl->master->bank.ins[program].name;
+}
+
+std::string MiddleWare::activeUrl(void) const
 {
     return impl->last_url;
 }
@@ -2223,12 +2600,26 @@ const SYNTH_T &MiddleWare::getSynth(void) const
     return impl->synth;
 }
 
-const char* MiddleWare::getServerAddress(void) const
+char* MiddleWare::getServerAddress(void) const
 {
     if(impl->server)
         return lo_server_get_url(impl->server);
     else
-        return "NULL";
+        return nullptr;
+}
+
+char* MiddleWare::getServerPort(void) const
+{
+    char* addr = getServerAddress();
+    char* result;
+    if(addr)
+    {
+        result = lo_url_get_port(addr);
+        free(addr);
+    }
+    else
+        result = nullptr;
+    return result;
 }
 
 const PresetsStore& MiddleWare::getPresetsStore() const
