@@ -20,6 +20,7 @@
 #include "../Params/ADnoteParameters.h"
 #include "../Params/SUBnoteParameters.h"
 #include "../Params/PADnoteParameters.h"
+#include "../Synth/Portamento.h"
 #include "../Synth/Resonance.h"
 #include "../Synth/SynthNote.h"
 #include "../Synth/ADnote.h"
@@ -31,6 +32,7 @@
 #include <cstdio>
 #include <cstring>
 #include <cassert>
+#include <ctime>
 
 #include <rtosc/ports.h>
 #include <rtosc/port-sugar.h>
@@ -43,6 +45,7 @@ using rtosc::RtData;
 
 #define rObject Part
 static const Ports partPorts = {
+    rSelf(Part, rEnabledBy(Penabled)),
     rRecurs(kit, 16, "Kit"),//NUM_KIT_ITEMS
     rRecursp(partefx, 3, "Part Effect"),
     rRecur(ctl,       "Controller"),
@@ -55,18 +58,20 @@ static const Ports partPorts = {
 #undef rChangeCb
 #define rChangeCb
 #undef rChangeCb
-#define rChangeCb obj->setVolume(obj->Volume);
-    rParamF(Volume, rShort("Vol"), rDefault(0.0), rUnit(dB), 
+#define rChangeCb obj->setVolumedB(obj->Volume);
+    rParamF(Volume, rShort("Vol"), rDefault(0.0), rUnit(dB),
             rLinear(-40.0, 13.3333), "Part Volume"),
 #undef rChangeCb
     {"Pvolume::i", rShort("Vol") rProp(parameter) rLinear(0,127)
         rDefault(96) rDoc("Part Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
+            Part *obj = (Part*)d.obj;
         if(rtosc_narguments(m)==0) {
-            d.reply(d.loc, "i", (int) roundf(96.0f * ((Part*)d.obj)->Volume / 40.0f + 96.0f));
+            d.reply(d.loc, "i", (int) roundf(96.0f * obj->Volume / 40.0f + 96.0f));
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
-            ((Part *)d.obj)->Volume  = ((Part *)d.obj)->volume127ToFloat(limit<unsigned char>(rtosc_argument(m, 0).i, 0, 127));
-             d.broadcast(d.loc, "i", limit<char>(rtosc_argument(m, 0).i, 0, 127));
+            obj->Volume  = obj->volume127TodB(limit<unsigned char>(rtosc_argument(m, 0).i, 0, 127));
+            obj->setVolumedB(obj->Volume);
+            d.broadcast(d.loc, "i", limit<char>(rtosc_argument(m, 0).i, 0, 127));
         }}},
 #define rChangeCb obj->setPpanning(obj->Ppanning);
     rParamZyn(Ppanning, rShort("pan"), rDefault(64), "Set Panning"),
@@ -75,6 +80,10 @@ static const Ports partPorts = {
     rParamI(Pkeylimit, rShort("limit"), rProp(parameter),
     rMap(min,0), rMap(max, POLYPHONY), rDefault(15), "Key limit per part"),
 #undef rChangeCb
+#define rChangeCb obj->setvoicelimit(obj->Pvoicelimit);
+    rParamI(Pvoicelimit, rShort("vlimit"), rProp(parameter),
+    rMap(min,0), rMap(max, POLYPHONY), rDefault(0), "Voice limit per part"),
+#undef rChangeCb
 #define rChangeCb
     rParamZyn(Pminkey, rShort("min"), rDefault(0), "Min Used Key"),
     rParamZyn(Pmaxkey, rShort("max"), rDefault(127), "Max Used Key"),
@@ -112,22 +121,22 @@ static const Ports partPorts = {
     {"captureMax:", rDoc("Capture maximum valid note"), NULL,
         [](const char *, RtData &r)
         {Part *p = (Part*)r.obj; p->Pmaxkey = p->lastnote;}},
-    {"polyType::c:i", rProp(parameter) rOptions(Polyphonic, Monophonic, Legato)
-        rDoc("Synthesis polyphony type\n"
-                "Polyphonic - Each note is played independently\n"
-                "Monophonic - A single note is played at a time with"
-                " envelopes resetting between notes\n"
-                "Legato     - A single note is played at a time without"
-                " envelopes resetting between notes\n"
-            ), NULL,
+    {"polyType::c:i", rProp(parameter) rOptions(Poly, Mono, Legato, Latch)
+        rDoc("Synthesis polyphony type\n"), NULL,
         [](const char *msg, RtData &d)
         {
             Part *p = (Part*)d.obj;
-            if(!rtosc_narguments(msg)) {
+            auto get_polytype = [&p](){
                 int res = 0;
                 if(!p->Ppolymode)
                     res = p->Plegatomode ? 2 : 1;
-                d.reply(d.loc, "i", res);
+                if(p->Platchmode)
+                    res = 3;
+                return res;
+            };
+
+            if(!rtosc_narguments(msg)) {
+                d.reply(d.loc, "i", get_polytype());
                 return;
             }
 
@@ -135,15 +144,25 @@ static const Ports partPorts = {
             if(i == 0) {
                 p->Ppolymode = 1;
                 p->Plegatomode = 0;
+                p->Platchmode = 0;
             } else if(i==1) {
                 p->Ppolymode = 0;
                 p->Plegatomode = 0;
-            } else {
+                p->Platchmode = 0;
+            } else if(i==2) {
                 p->Ppolymode = 0;
                 p->Plegatomode = 1;
-            }}},
+                p->Platchmode = 0;
+            } else {
+                p->Ppolymode = 1;
+                p->Plegatomode = 0;
+                p->Platchmode = 1;
+            }
+            d.broadcast(d.loc, "i", get_polytype());
+        }
+    },
     {"clear:", rProp(internal) rDoc("Reset Part To Defaults"), 0,
-        [](const char *, RtData &d)
+        [](const char *, RtData &)
         {
             //XXX todo forward this event for middleware to handle
             //Part *p = (Part*)d.obj;
@@ -156,8 +175,25 @@ static const Ports partPorts = {
 
             //d.broadcast("/damage", "s", part_loc);
         }},
-
-
+        {"savexml:", rProp(internal) rDoc("Save Part to the file it has been loaded from"), 0,
+        [](const char *, RtData &d)
+        {
+            Part *p = (Part*)d.obj;
+            if (p->loaded_file[0] == '\0') {  // if part was never loaded or saved
+                time_t rawtime;     // make a new name from date and time
+                char filename[32];
+                time (&rawtime);
+                const struct tm* timeinfo = localtime (&rawtime);
+                strftime (filename,23,"%F_%R.xiz",timeinfo);
+                p->saveXML(filename);
+                fprintf(stderr, "Part %d saved to %s\n", (p->partno + 1), filename);
+            }
+            else
+            {
+                p->saveXML(p->loaded_file);
+                fprintf(stderr, "Part %d saved to %s\n", (p->partno + 1), p->loaded_file);
+            }
+        }},
     //{"kit#16::T:F", "::Enables or disables kit item", 0,
     //    [](const char *m, RtData &d) {
     //        auto loc = d.loc;
@@ -236,6 +272,7 @@ Part::Part(Allocator &alloc, const SYNTH_T &synth_, const AbsTime &time_,
     :Pdrummode(false),
     Ppolymode(true),
     Plegatomode(false),
+    Platchmode(false),
     partoutl(new float[synth_.buffersize]),
     partoutr(new float[synth_.buffersize]),
     ctl(synth_, &time_),
@@ -248,6 +285,8 @@ Part::Part(Allocator &alloc, const SYNTH_T &synth_, const AbsTime &time_,
     gzip_compression(gzip_compression),
     interpolation(interpolation)
 {
+    loaded_file[0] = '\0';
+
     if(prefix_)
         fast_strcpy(prefix, prefix_, sizeof(prefix));
     else
@@ -278,13 +317,15 @@ Part::Part(Allocator &alloc, const SYNTH_T &synth_, const AbsTime &time_,
     }
 
     killallnotes = false;
-    oldfreq      = -1.0f;
+    silent = false;
+    oldfreq_log2 = -1.0f;
+    oldportamento = NULL;
+    legatoportamento = NULL;
 
     cleanup();
 
     Pname = new char[PART_MAX_NAME_LEN];
 
-    oldvolumel = oldvolumer = 0.5f;
     lastnote = -1;
 
     defaults();
@@ -307,7 +348,7 @@ void Part::cloneTraits(Part &p) const
 #define CLONE(x) p.x = this->x
     CLONE(Penabled);
 
-    p.setVolume(this->Volume);
+    p.setVolumedB(this->Volume);
     p.setPpanning(this->Ppanning);
 
     CLONE(Pminkey);
@@ -322,8 +363,11 @@ void Part::cloneTraits(Part &p) const
     CLONE(Ppolymode);
     CLONE(Plegatomode);
     CLONE(Pkeylimit);
+    CLONE(Pvoicelimit);
 
-    CLONE(ctl);
+    // Controller has a reference, so it can not be re-assigned
+    // So, destroy and reconstruct it.
+    p.ctl.~Controller(); new (&p.ctl) Controller(this->ctl);
 }
 
 void Part::defaults()
@@ -334,13 +378,14 @@ void Part::defaults()
     Pnoteon     = 1;
     Ppolymode   = 1;
     Plegatomode = 0;
-    setVolume(0.0);
+    setVolumedB(0.0);
     Pkeyshift = 64;
     Prcvchn   = 0;
     setPpanning(64);
     Pvelsns   = 64;
     Pveloffs  = 64;
     Pkeylimit = 15;
+    Pvoicelimit = 0;
     defaultsinstrument();
     ctl.defaults();
 }
@@ -460,9 +505,8 @@ static int kit_usage(const Part::Kit *kits, int note, int mode)
 /*
  * Note On Messages
  */
-bool Part::NoteOn(note_t note,
+bool Part::NoteOnInternal(note_t note,
                   unsigned char velocity,
-                  int masterkeyshift,
                   float note_log2_freq)
 {
     //Verify Basic Mode and sanity
@@ -481,7 +525,6 @@ bool Part::NoteOn(note_t note,
     if(isMonoMode() || isLegatoMode()) {
         monomemPush(note);
         monomem[note].velocity  = velocity;
-        monomem[note].mkeyshift = masterkeyshift;
         monomem[note].note_log2_freq = note_log2_freq;
 
     } else if(!monomemEmpty())
@@ -495,37 +538,98 @@ bool Part::NoteOn(note_t note,
 
     //Compute Note Parameters
     const float vel          = getVelocity(velocity, Pvelsns, Pveloffs);
-    const int   partkeyshift = (int)Pkeyshift - 64;
-    const int   keyshift     = masterkeyshift + partkeyshift;
-    const float notebasefreq = getBaseFreq(note_log2_freq, keyshift);
-
-    if(notebasefreq < 0.0f)
-        return false;
 
     //Portamento
     lastnote = note;
-    if(oldfreq < 1.0f)
-        oldfreq = notebasefreq;//this is only the first note is played
+
+    /* check if first note is played */
+    if(oldfreq_log2 < 0.0f)
+        oldfreq_log2 = note_log2_freq;
 
     // For Mono/Legato: Force Portamento Off on first
     // notes. That means it is required that the previous note is
     // still held down or sustained for the Portamento to activate
     // (that's like Legato).
-    bool portamento = false;
-    if(Ppolymode || isRunningNote)
-        portamento = ctl.initportamento(oldfreq, notebasefreq, doingLegato);
+    PortamentoRealtime *portamento_realtime = NULL;
+    if(Ppolymode || isRunningNote) {
+        // If there is a currently ongoing glide, shift the starting point
+        // for any new portamento to where the current glide is right now
+        if (oldportamento && oldportamento->portamento.active)
+            oldportamentofreq_log2 += oldportamento->portamento.freqdelta_log2;
+        // Non-portamento settings and conditions say the note may have
+        // portamento, but it remains for Portamento.init to make the
+        // final decision depending on the portamento enable, threshold and
+        // other parameters.
+        Portamento portamento(ctl, synth, oldfreq_log2, oldportamentofreq_log2, note_log2_freq);
+        if(portamento.active) {
+            // We're good to go! Just need to figure out how.
+            // If we're doing legato and we already have a portamento structure,
+            // reuse it.
+            if (doingLegato && legatoportamento) {
+                portamento_realtime = legatoportamento;
+                portamento_realtime->portamento = portamento;
+            } else {
+                // Create new one if we don't already have one, or for each
+                // note in poly/mono mode
+                portamento_realtime = memory.alloc<PortamentoRealtime>
+                    (this,
+                     memory,
+                     // Cleanup function: Destroy any references we might
+                     // have to the current realtime pointer so that it
+                     // can not be (re)used, with disastrous results.
+                     [](PortamentoRealtime *realtime)
+                        {
+                            assert(realtime);
+                            Part *part = static_cast<Part *>(realtime->handle);
+                            assert(part);
+                            if (realtime == part->oldportamento) {
+                                // Since the last note is going away, capture
+                                // the portamento:ed pitch offset to our saved
+                                // previous note. This will be our starting
+                                // point for the next portamento glide.
+                                if (realtime->portamento.active)
+                                    part->oldportamentofreq_log2 +=
+                                        realtime->portamento.freqdelta_log2;
+                                part->oldportamento = NULL;
+                            }
+                            if (realtime == part->legatoportamento)
+                                part->legatoportamento = NULL;
+                        },
+                     portamento
+                    );
+                if (doingLegato)
+                    legatoportamento = portamento_realtime;
+            }
+        }
+    }
 
-    oldfreq = notebasefreq;
+    // Create the portamento pointer that we distribute to the synth notes
+    Portamento *portamento = NULL;
+    if(portamento_realtime)
+        portamento = &portamento_realtime->portamento;
+
+    // Save note freq and pointer to portamento state for next note
+    oldfreq_log2 = note_log2_freq;
+    oldportamentofreq_log2 = oldfreq_log2;
+    oldportamento = portamento_realtime;
 
     //Adjust Existing Notes
     if(doingLegato) {
-        LegatoParams pars = {notebasefreq, vel, portamento, note_log2_freq, true, prng()};
-        notePool.applyLegato(note, pars);
+        LegatoParams pars = {vel, portamento, note_log2_freq, true, prng()};
+        notePool.applyLegato(note, pars, portamento_realtime);
         return true;
     }
 
+    // We know now that we are not doing legato, so we destroy the reference
+    // to the previous legato portamento info so we don't try to reuse it
+    legatoportamento = NULL;
+
     if(Ppolymode)
         notePool.makeUnsustainable(note);
+    
+    // in latch mode release latched notes before creating the new one
+    if(Platchmode)
+        notePool.releaseLatched();
 
     //Create New Notes
     for(uint8_t i = 0; i < NUM_KIT_ITEMS; ++i) {
@@ -534,22 +638,28 @@ bool Part::NoteOn(note_t note,
         if(Pkitmode != 0 && !item.validNote(note))
             continue;
 
-        SynthParams pars{memory, ctl, synth, time, notebasefreq, vel,
+        SynthParams pars{memory, ctl, synth, time, vel,
             portamento, note_log2_freq, false, prng()};
         const int sendto = Pkitmode ? item.sendto() : 0;
 
+        // Enforce voice limit, before we trigger new note
+        limit_voices(note);
+
         try {
             if(item.Padenabled)
                 notePool.insertNote(note, sendto,
                         {memory.alloc<ADnote>(kit[i].adpars, pars,
-                            wm, (pre+"kit"+i+"/adpars/").c_str), 0, i});
+                            wm, (pre+"kit"+i+"/adpars/").c_str), 0, i},
+                                    portamento_realtime);
             if(item.Psubenabled)
                 notePool.insertNote(note, sendto,
-                        {memory.alloc<SUBnote>(kit[i].subpars, pars, wm, (pre+"kit"+i+"/subpars/").c_str), 1, i});
+                        {memory.alloc<SUBnote>(kit[i].subpars, pars, wm, (pre+"kit"+i+"/subpars/").c_str), 1, i},
+                                    portamento_realtime);
             if(item.Ppadenabled)
                 notePool.insertNote(note, sendto,
                         {memory.alloc<PADnote>(kit[i].padpars, pars, interpolation, wm,
-                            (pre+"kit"+i+"/padpars/").c_str), 2, i});
+                            (pre+"kit"+i+"/padpars/").c_str), 2, i},
+                                    portamento_realtime);
         } catch (std::bad_alloc & ba) {
             std::cerr << "dropped new note: " << ba.what() << std::endl;
         }
@@ -579,7 +689,10 @@ void Part::NoteOff(note_t note) //release the key
     for(auto &desc:notePool.activeDesc()) {
         if(desc.note != note || !desc.playing())
             continue;
-        if(!ctl.sustain.sustain) { //the sustain pedal is not pushed
+        // if latch is on we ignore noteoff, but set the state to lateched
+        if(Platchmode) {
+            notePool.latch(desc);
+        } else if(!ctl.sustain.sustain) { //the sustain pedal is not pushed
             if((isMonoMode() || isLegatoMode()) && !monomemEmpty())
                 MonoMemRenote();//Play most recent still active note
             else
@@ -588,21 +701,26 @@ void Part::NoteOff(note_t note) //release the key
         else {   //the sustain pedal is pushed
             if(desc.canSustain())
                 desc.doSustain();
-            else
+            else {
                 notePool.release(desc);
+            }
         }
     }
 }
 
 void Part::PolyphonicAftertouch(note_t note,
-				unsigned char velocity,
-				int masterkeyshift)
+                unsigned char velocity)
 {
-    (void) masterkeyshift;
-
     if(!Pnoteon || !inRange(note, Pminkey, Pmaxkey) || Pdrummode)
         return;
 
+    /*
+     * Don't allow the velocity to reach zero.
+     * Keep it alive until note off.
+     */
+    if(velocity == 0)
+        velocity = 1;
+
     // MonoMem stuff:
     if(!Ppolymode)   // if Poly is off
         monomem[note].velocity = velocity;       // Store this note's velocity.
@@ -626,7 +744,7 @@ void Part::SetController(unsigned int type, int par)
             break;
         case C_expression:
             ctl.setexpression(par);
-            setVolume(Volume); //update the volume
+            setVolumedB(Volume); //update the volume
             break;
         case C_portamento:
             ctl.setportamento(par);
@@ -653,9 +771,10 @@ void Part::SetController(unsigned int type, int par)
         case C_volume:
             ctl.setvolume(par);
             if(ctl.volume.receive != 0)
-                volume = ctl.volume.volume;
+                setVolumedB(volume127TodB( ctl.volume.volume * 127.0f ) );
             else
-                setVolume(Volume);
+                /* FIXME: why do this? */
+                setVolumedB(Volume);
             break;
         case C_sustain:
             ctl.setsustain(par);
@@ -669,10 +788,9 @@ void Part::SetController(unsigned int type, int par)
             ctl.resetall();
             ReleaseSustainedKeys();
             if(ctl.volume.receive != 0)
-                volume = ctl.volume.volume;
+                setVolumedB(volume127TodB( ctl.volume.volume * 127.0f ) );
             else
-                setVolume(Volume);
-            setVolume(Volume); //update the volume
+                setVolumedB(Volume);
             setPpanning(Ppanning); //update the panning
 
             for(int item = 0; item < NUM_KIT_ITEMS; ++item) {
@@ -706,6 +824,47 @@ void Part::SetController(unsigned int type, int par)
             break;
     }
 }
+
+/*
+ * Per note controllers.
+ */
+void Part::SetController(unsigned int type, note_t note, float value,
+                         int masterkeyshift)
+{
+    if(!Pnoteon || !inRange(note, Pminkey, Pmaxkey) || Pdrummode)
+        return;
+
+    switch (type) {
+    case C_aftertouch:
+        PolyphonicAftertouch(note, floorf(value));
+        break;
+    case C_pitch: {
+        if (getNoteLog2Freq(masterkeyshift, value) == false)
+            break;
+
+        /* Make sure MonoMem's frequency information is kept up to date */
+        if(!Ppolymode)
+            monomem[note].note_log2_freq = value;
+
+        for(auto &d:notePool.activeDesc()) {
+            if(d.note == note && d.playing())
+                for(auto &s:notePool.activeNotes(d))
+                    s.note->setPitch(value);
+        }
+        break;
+    }
+    case C_filtercutoff:
+        for(auto &d:notePool.activeDesc()) {
+            if(d.note == note && d.playing())
+                for(auto &s:notePool.activeNotes(d))
+                    s.note->setFilterCutoff(value);
+        }
+        break;
+    default:
+        break;
+    }
+}
+
 /*
  * Release the sustained keys
  */
@@ -741,18 +900,19 @@ void Part::MonoMemRenote()
 {
     note_t mmrtempnote = monomemBack(); // Last list element.
     monomemPop(mmrtempnote); // We remove it, will be added again in NoteOn(...).
-    NoteOn(mmrtempnote,
+    NoteOnInternal(mmrtempnote,
            monomem[mmrtempnote].velocity,
-           monomem[mmrtempnote].mkeyshift,
            monomem[mmrtempnote].note_log2_freq);
 }
 
-float Part::getBaseFreq(float note_log2_freq, int keyshift) const
+bool Part::getNoteLog2Freq(int masterkeyshift, float &note_log2_freq)
 {
-    if(Pdrummode)
-        return 440.0f * powf(2.0f, note_log2_freq - (69.0f / 12.0f));
-    else
-        return microtonal->getnotefreq(note_log2_freq, keyshift);
+    if(Pdrummode) {
+        note_log2_freq += log2f(440.0f) - 69.0f / 12.0f;
+        return true;
+    }
+    return microtonal->updatenotefreq_log2(note_log2_freq,
+        (int)Pkeyshift - 64 + masterkeyshift);
 }
 
 float Part::getVelocity(uint8_t velocity, uint8_t velocity_sense,
@@ -791,6 +951,36 @@ void Part::setkeylimit(unsigned char Pkeylimit_)
         notePool.enforceKeyLimit(keylimit);
 }
 
+/*
+ * Enforce voice limit
+ */
+void Part::limit_voices(int new_note)
+{
+    int voicelimit = Pvoicelimit;
+    if(voicelimit == 0) /* voice limit disabled */
+        return;
+
+    /* If we're called because a new note is imminent, we need to enforce
+     * one less note than the limit, so we don't go above the limit when the
+     * new note is triggered.
+     */
+    if (new_note >= 0)
+        voicelimit--;
+
+    int running_voices = notePool.getRunningVoices();
+    if(running_voices >= voicelimit)
+        notePool.enforceVoiceLimit(voicelimit, new_note);
+}
+
+/*
+ * Set Part's voice limit
+ */
+void Part::setvoicelimit(unsigned char Pvoicelimit_)
+{
+    Pvoicelimit = Pvoicelimit_;
+
+    limit_voices(-1);
+}
 
 /*
  * Prepare all notes to be turned off
@@ -805,6 +995,24 @@ void Part::AllNotesOff()
  */
 void Part::ComputePartSmps()
 {
+    /* When we are in the process of being disabled (Penabled set to false),
+     * AllNotesOff will be called, setting killallnotes, which causes all
+     * playing voices to terminate and the signal level being graciously
+     * muted during the course of the current buffer. After that, all
+     * subsequent output buffers will be set to 0 until we are enabled again.
+     */
+    if (!Penabled && !killallnotes) {
+        /* We only need to clear the output buffer once when disabled; since
+         * it's static within the part it's wasteful to do it every time. */
+        if (!silent) {
+            memset(partoutl, 0, synth.bufferbytes);
+            memset(partoutr, 0, synth.bufferbytes);
+            silent = true;
+        }
+        return;
+    }
+    silent = false;
+
     assert(partefx[0]);
     for(unsigned nefx = 0; nefx < NUM_PART_EFX + 1; ++nefx) {
         memset(partfxinputl[nefx], 0, synth.bufferbytes);
@@ -827,6 +1035,8 @@ void Part::ComputePartSmps()
             if(note.finished())
                 notePool.kill(s);
         }
+    if (d.portamentoRealtime)
+        d.portamentoRealtime->portamento.update();
     }
 
     //Apply part's effects and mix them
@@ -862,22 +1072,36 @@ void Part::ComputePartSmps()
         for(int nefx = 0; nefx < NUM_PART_EFX; ++nefx)
             partefx[nefx]->cleanup();
     }
-    ctl.updateportamento();
 }
 
 /*
  * Parameter control
  */
-float Part::volume127ToFloat(unsigned char volume_)
+
+float Part::volume127TodB(unsigned char volume_)
 {
+    assert( volume_  <= 127 );
     return (volume_ - 96.0f) / 96.0f * 40.0;
 }
 
-void Part::setVolume(float Volume_)
+void Part::setVolumedB(float Volume_)
 {
+    //Fixes bug with invalid loading
+    if(fabs(Volume_ - 50.0f) < 0.001)
+        Volume_ = 0.0f;
+
+    Volume_ = limit(Volume_, -40.0f, 13.333f);
+
+    assert(Volume_ < 14.0);
     Volume = Volume_;
-    volume  =
-        dB2rap(Volume) * ctl.expression.relvolume;
+
+    float volume = dB2rap( Volume_ );
+
+    /* printf( "Volume: %f, Expression %f\n", volume, ctl.expression.relvolume ); */
+
+    assert( volume <= dB2rap(14.0f) );
+
+    gain = volume * ctl.expression.relvolume;
 }
 
 void Part::setPpanning(char Ppanning_)
@@ -939,7 +1163,7 @@ void Part::add2XMLinstrument(XMLwrapper& xml)
     for(int i = 0; i < NUM_KIT_ITEMS; ++i) {
         xml.beginbranch("INSTRUMENT_KIT_ITEM", i);
         xml.addparbool("enabled", kit[i].Penabled);
-        if(kit[i].Penabled != 0) {
+        if(kit[i].Penabled != 0 || xml.SaveFullXml) {
             xml.addparstr("name", (char *)kit[i].Pname);
 
             xml.addparbool("muted", kit[i].Pmuted);
@@ -949,21 +1173,21 @@ void Part::add2XMLinstrument(XMLwrapper& xml)
             xml.addpar("send_to_instrument_effect", kit[i].Psendtoparteffect);
 
             xml.addparbool("add_enabled", kit[i].Padenabled);
-            if(kit[i].Padenabled && kit[i].adpars) {
-                xml.beginbranch("ADD_SYNTH_PARAMETERS");
+            if((kit[i].Padenabled  || xml.SaveFullXml) && kit[i].adpars) {
+               xml.beginbranch("ADD_SYNTH_PARAMETERS");
                 kit[i].adpars->add2XML(xml);
                 xml.endbranch();
             }
 
             xml.addparbool("sub_enabled", kit[i].Psubenabled);
-            if(kit[i].Psubenabled && kit[i].subpars) {
+            if((kit[i].Psubenabled || xml.SaveFullXml) && kit[i].subpars) {
                 xml.beginbranch("SUB_SYNTH_PARAMETERS");
                 kit[i].subpars->add2XML(xml);
                 xml.endbranch();
             }
 
             xml.addparbool("pad_enabled", kit[i].Ppadenabled);
-            if(kit[i].Ppadenabled && kit[i].padpars) {
+            if((kit[i].Ppadenabled || xml.SaveFullXml) && kit[i].padpars) {
                 xml.beginbranch("PAD_SYNTH_PARAMETERS");
                 kit[i].padpars->add2XML(xml);
                 xml.endbranch();
@@ -1010,6 +1234,7 @@ void Part::add2XML(XMLwrapper& xml)
     xml.addparbool("poly_mode", Ppolymode);
     xml.addpar("legato_mode", Plegatomode);
     xml.addpar("key_limit", Pkeylimit);
+    xml.addpar("voice_limit", Pvoicelimit);
 
     xml.beginbranch("INSTRUMENT");
     add2XMLinstrument(xml);
@@ -1041,6 +1266,13 @@ int Part::loadXMLinstrument(const char *filename)
 
     if(xml.enterbranch("INSTRUMENT") == 0)
         return -10;
+
+    // store filename in member variable
+    int length = sizeof(loaded_file)-1;
+    strncpy(loaded_file, filename, length);
+    // set last element to \0 in case filename is too long or not terminated
+    loaded_file[length]='\0';
+
     getfromXMLinstrument(xml);
     xml.exitbranch();
 
@@ -1132,10 +1364,10 @@ void Part::getfromXMLinstrument(XMLwrapper& xml)
             if(xml.enterbranch("INSTRUMENT_KIT_ITEM", i) == 0)
                 continue;
             setkititemstatus(i, xml.getparbool("enabled", kit[i].Penabled));
-            if(kit[i].Penabled == 0) {
-                xml.exitbranch();
-                continue;
-            }
+//           if(kit[i].Penabled == 0) { // Remove this as it loses data that may be in the xml file but disabled!
+//                xml.exitbranch();
+//                continue;
+//            }
 
             xml.getparstr("name", (char *)kit[i].Pname, PART_MAX_NAME_LEN);
 
@@ -1206,9 +1438,9 @@ void Part::getfromXML(XMLwrapper& xml)
 {
     Penabled = xml.getparbool("enabled", Penabled);
     if (xml.hasparreal("volume")) {
-        setVolume(xml.getparreal("volume", Volume));
+        setVolumedB(xml.getparreal("volume", Volume));
     } else {
-        setVolume(volume127ToFloat(xml.getpar127("volume", -40.0f)));
+        setVolumedB(volume127TodB( xml.getpar127("volume", 96)));
     }
     setPpanning(xml.getpar127("panning", Ppanning));
 
@@ -1226,6 +1458,7 @@ void Part::getfromXML(XMLwrapper& xml)
     if(!Plegatomode)
         Plegatomode = xml.getpar127("legato_mode", Plegatomode);
     Pkeylimit = xml.getpar127("key_limit", Pkeylimit);
+    Pvoicelimit = xml.getpar127("voice_limit", Pvoicelimit);
 
 
     if(xml.enterbranch("INSTRUMENT")) {
