@@ -24,12 +24,14 @@ class AbsTime
         void operator++(){++frames;};
         void operator++(int){frames++;};
         int64_t time() const {return frames;};
+        unsigned int tempo;
         float dt() const { return s.dt(); }
         float framesPerSec() const { return 1/s.dt();}
         int   samplesPerFrame() const {return s.buffersize;}
     private:
         int64_t frames;
         const SYNTH_T &s;
+        
 };
 
 //Marker for an event relative to some position of the absolute timer
@@ -40,10 +42,10 @@ class RelTime
             :t(t_)
         {
             //Calculate time of event
-            double deltaFrames = sec*t.framesPerSec();
+            float deltaFrames = sec*t.framesPerSec();
             int64_t tmp = (int64_t)deltaFrames;
             frame = t.time() + tmp;
-            sample = t.samplesPerFrame()*(deltaFrames-tmp);
+            sample = (int32_t)(t.samplesPerFrame()*(deltaFrames-tmp));
         }
         bool inThisFrame() {return t.time() == frame;};
         bool inPast() {return t.time() > frame;}
