@@ -24,6 +24,8 @@
 #include "OutMgr.h"
 #include "../Misc/Stereo.h"
 
+namespace zyn {
+
 class AlsaEngine:public AudioOut, MidiIn
 {
     public:
@@ -74,4 +76,6 @@ class AlsaEngine:public AudioOut, MidiIn
         void *processAudio();
 };
 
+}
+
 #endif
