@@ -31,7 +31,7 @@ class AudioOut:public virtual Engine
         void setSamplerate(int _samplerate);
 
         /**Sets the Samples required per Out of this driver
-         * not a realtime opperation */
+         * not a realtime operation */
         int getSampleRate();
         void setBufferSize(int _bufferSize);
 
@@ -41,6 +41,8 @@ class AudioOut:public virtual Engine
 
         virtual void setAudioEn(bool nval) = 0;
         virtual bool getAudioEn() const    = 0;
+        
+        bool isOutputCompressionEnabled = 0;
 
     protected:
         /**Get the next sample for output.
@@ -50,6 +52,7 @@ class AudioOut:public virtual Engine
         const SYNTH_T &synth;
         int samplerate;
         int bufferSize;
+        
 };
 
 }
