@@ -125,6 +125,16 @@ string OutMgr::getSink() const
     return "ERROR";
 }
 
+void OutMgr::setAudioCompressor(bool isEnabled)
+{
+    currentOut->isOutputCompressionEnabled=isEnabled;
+}
+
+bool OutMgr::getAudioCompressor(void)
+{
+    return currentOut->isOutputCompressionEnabled;
+}
+
 void OutMgr::setMaster(Master *master_)
 {
     master=master_;
@@ -136,7 +146,7 @@ void OutMgr::applyOscEventRt(const char *msg)
 }
 
 //perform a cheap linear interpolation for resampling
-//This will result in some distortion at frame boundries
+//This will result in some distortion at frame boundaries
 //returns number of samples produced
 static size_t resample(float *dest,
                        const float *src,
