@@ -59,17 +59,17 @@ static const rtosc::Ports subports = {
 
 #define rObject FilterParams
 #undef  rChangeCb
-#define rChangeCb obj->changed = true; if ( obj->time) {      \
-    obj->last_update_timestamp = obj->time->time(); }
+#define rChangeCb do { obj->changed = true; if ( obj->time) {      \
+    obj->last_update_timestamp = obj->time->time(); } } while(false)
 const rtosc::Ports FilterParams::ports = {
     rSelf(FilterParams),
-    rPaste,
-    rArrayPaste,
+    rPasteRt,
+    rArrayPasteRt,
     rOption(loc, rProp(internal),
             rOptions(ad_global_filter, ad_voice_filter, sub_filter, in_effect),
             "location of the filter"),
     rOption(Pcategory,          rShort("class"),
-            rOptions(analog, formant, st.var.), rDefault(analog),
+            rOptions(analog, formant, st.var., moog, comb), rDefault(analog),
             "Class of filter"),
     rOption(Ptype,              rShort("type"),
             rOptions(LP1, HP1, LP2, HP2, BP, notch, peak, l.shelf, h.shelf),
@@ -131,7 +131,12 @@ const rtosc::Ports FilterParams::ports = {
     {"type-svf::i", rProp(parameter) rShort("type")
         rOptions(low, high, band, notch)
             rDoc("Filter Type"), 0, rOptionCb(Ptype)},
-
+    {"type-moog::i", rProp(parameter) rShort("type")
+        rOptions(HP, BP, LP)
+            rDoc("Filter Type"), 0, rOptionCb(Ptype)},
+    {"type-comb::i", rProp(parameter) rShort("type")
+        rOptions(BWD, FWD, both)
+            rDoc("Comb Filter Type"), 0, rOptionCb(Ptype)},
     //UI reader
     {"Pvowels:", rDoc("Get Formant Vowels"), NULL,
         [](const char *, RtData &d) {
@@ -144,8 +149,8 @@ const rtosc::Ports FilterParams::ports = {
         rEnabledByCondition(is_formant_filter),
         &subports,
         [](const char *msg, RtData &d) {
-            const char *mm = msg; \
-            while(*mm && !isdigit(*mm)) ++mm; \
+            const char *mm = msg;
+            while(*mm && !isdigit(*mm)) ++mm;
             unsigned idx = atoi(mm);
 
             SNIP;
@@ -207,6 +212,22 @@ const rtosc::Ports FilterParams::ports = {
                         (float)obj->Pstages,
                         cf.b[0], cf.b[1], cf.b[2],
                         0.0,     -cf.a[1], -cf.a[2]);
+            } else if(obj->Pcategory == 3) {
+                int order = 0;
+                float gain = dB2rap(obj->getgain());
+                if(obj->Ptype != 6 && obj->Ptype != 7 && obj->Ptype != 8)
+                    gain = 1.0;
+                int tmp = 4-obj->Ptype;
+                if(tmp < 0 || tmp > 8)
+                    return;
+                auto cf = AnalogFilter::computeCoeff(4-obj->Ptype,
+                        Filter::getrealfreq(obj->getfreq()),
+                        obj->getq(), obj->Pstages,
+                        gain, 48000, order);
+                d.reply(d.loc, "fffffff",
+                        (float)obj->Pstages,
+                        cf.c[0], cf.c[1], cf.c[2],
+                        0.0,     cf.d[1], cf.d[2]);
             }
         }},
     //    "", NULL, [](){}},"/freq"
@@ -226,7 +247,7 @@ const rtosc::Ports FilterParams::ports = {
             FilterParams *obj = (FilterParams *) d.obj;
 
             rtosc_arg_t args[2+3*FF_MAX_FORMANTS*FF_MAX_VOWELS];
-            char type[2+3*FF_MAX_FORMANTS*FF_MAX_VOWELS + 1] = {0};
+            char type[2+3*FF_MAX_FORMANTS*FF_MAX_VOWELS + 1] = {};
 
             type[0] = 'i';
             type[1] = 'i';
@@ -469,7 +490,7 @@ float FilterParams::getq() const
 }
 float FilterParams::getfreqtracking(float notefreq) const
 {
-    return log2f(notefreq / 440.0f) * (freqtracking / 100.0);
+    return log2f(notefreq / 440.0f) * (freqtracking / 100.0f);
 }
 
 float FilterParams::getgain() const
