@@ -40,15 +40,18 @@ class LFOParams:public Presets
         LFOParams(float freq_,
                   char Pintensity_,
                   char Pstartphase_,
+                  char Pcutoff_,
                   char PLFOtype_,
                   char Prandomness_,
                   float delay_,
+                  float fadein_,
+                  float fadeout_,
                   char Pcontinous,
                   consumer_location_t loc,
                   const AbsTime* time_ = nullptr);
-        ~LFOParams();
+        ~LFOParams() override;
 
-        void add2XML(XMLwrapper& xml);
+        void add2XML(XMLwrapper& xml) override;
         void defaults();
         /**Loads the LFO from the xml*/
         void getfromXML(XMLwrapper& xml);
@@ -58,11 +61,16 @@ class LFOParams:public Presets
         float         freq;      /**<frequency*/
         unsigned char Pintensity; /**<intensity*/
         unsigned char Pstartphase; /**<start phase (0=random)*/
+        unsigned char Pcutoff; /**<cutoff */
         unsigned char PLFOtype; /**<LFO type (sin,triangle,square,ramp,...)*/
         unsigned char Prandomness; /**<randomness (0=off)*/
         unsigned char Pfreqrand; /**<frequency randomness (0=off)*/
         float         delay; /**<delay (0=off)*/
+        float         fadein; /**<fadein, relative to delay*/
+        float         fadeout; /**<fadeout on key release (10.0=off)*/
         unsigned char Pcontinous; /**<1 if LFO is continous*/
+        int           numerator;  /**<numerator for integer ratio between system tempo and LFO freq (0=off)*/
+        int           denominator;/**<denominator for integer ratio between system tempo and LFO freq (0=off)*/
         unsigned char Pstretch; /**<how the LFO is "stretched" according the note frequency (64=no stretch)*/
 
         //! what kind is the LFO (0 - frequency, 1 - amplitude, 2 - filter)
@@ -81,9 +89,12 @@ class LFOParams:public Presets
         float         Dfreq;
         unsigned char Dintensity;
         unsigned char Dstartphase;
+        unsigned char Dcutoff;
         unsigned char DLFOtype;
         unsigned char Drandomness;
         float         Ddelay;
+        float         Dfadein;
+        float         Dfadeout;
         unsigned char Dcontinous;
 };
 
