@@ -36,12 +36,25 @@ namespace zyn {
 #define rChangeCb if (obj->time) { obj->last_update_timestamp = obj->time->time(); }
 static const rtosc::Ports SUBnotePorts = {
     rSelf(SUBnoteParameters),
-    rPaste,
+    rPasteRt,
     rToggle(Pstereo,    rShort("stereo"), rDefault(true), "Stereo Enable"),
     rParamF(Volume,  rShort("volume"), rDefault(0), rUnit(dB), rLinear(-60.0f,20.0f), "Volume"),
     rParamZyn(PPanning, rShort("panning"), rDefault(64), "Left Right Panning"),
     rParamF(AmpVelocityScaleFunction, rShort("sense"), rDefault(70.86),
         rLinear(0.0, 100.0), "Amplitude Velocity Sensing function"),
+    {"PAmpVelocityScaleFunction::i", rShort("sense") rProp(parameter) rLinear(0,127)
+        rDefault(90) rDoc("Amplitude Velocity Sensing function"), 0,
+        [](const char *m, rtosc::RtData &d) {
+            SUBnoteParameters *obj = (SUBnoteParameters*)d.obj;
+            if (rtosc_narguments(m)==0) {
+                d.reply(d.loc, "i", (int) roundf(obj->AmpVelocityScaleFunction * 127.0f / 100.0f));
+            } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
+                const uint8_t value = limit<char>(rtosc_argument(m, 0).i, 0, 127);
+                obj->AmpVelocityScaleFunction = value * 100.0f / 127.0f;
+                d.broadcast(d.loc, "i", value);
+                rChangeCb
+            }
+    }},
     rParamI(PDetune,       rShort("detune"), rLinear(0, 16383), rDefault(8192),
         "Detune in detune type units"),
     rParamI(PCoarseDetune, rShort("cdetune"), rDefault(0), "Coarse Detune"),
@@ -133,28 +146,36 @@ static const rtosc::Ports SUBnotePorts = {
     {"octave::c:i", rProp(parameter) rShort("octave") rLinear(-8,7)
         rDoc("Note octave shift"), NULL,
         rBegin;
-        if(!rtosc_narguments(msg)) {
-            int k=obj->PCoarseDetune/1024;
-            if (k>=8) k-=16;
-            d.reply(d.loc, "i", k);
-        } else {
-            int k=(int) rtosc_argument(msg, 0).i;
-            if (k<0) k+=16;
-            obj->PCoarseDetune = k*1024 + obj->PCoarseDetune%1024;
-        }
+            auto get_octave = [&obj](){
+                int k=obj->PCoarseDetune/1024;
+                if (k>=8) k-=16;
+                return k;
+            };
+            if(!rtosc_narguments(msg)) {
+                d.reply(d.loc, "i", get_octave());
+            } else {
+                int k=(int) rtosc_argument(msg, 0).i;
+                if (k<0) k+=16;
+                obj->PCoarseDetune = k*1024 + obj->PCoarseDetune%1024;
+                d.broadcast(d.loc, "i", get_octave());
+            }
         rEnd},
     {"coarsedetune::c:i", rProp(parameter) rShort("coarse") rLinear(-64, 63)
         rDoc("Note coarse detune"), NULL,
         rBegin;
-        if(!rtosc_narguments(msg)) {
-            int k=obj->PCoarseDetune%1024;
-            if (k>=512) k-=1024;
-            d.reply(d.loc, "i", k);
-        } else {
-            int k=(int) rtosc_argument(msg, 0).i;
-            if (k<0) k+=1024;
-            obj->PCoarseDetune = k + (obj->PCoarseDetune/1024)*1024;
-        }
+            auto get_coarse = [&obj](){
+                int k=obj->PCoarseDetune%1024;
+                if (k>=512) k-=1024;
+                return k;
+            };
+            if(!rtosc_narguments(msg)) {
+                d.reply(d.loc, "i", get_coarse());
+            } else {
+                int k=(int) rtosc_argument(msg, 0).i;
+                if (k<0) k+=1024;
+                obj->PCoarseDetune = k + (obj->PCoarseDetune/1024)*1024;
+                d.broadcast(d.loc, "i", get_coarse());
+            }
         rEnd},
     {"response:", rDoc("Filter response at 440Hz. with 48kHz sample rate\n\n"
             "Format: stages, filter*active_filters\n"
