@@ -17,8 +17,10 @@
 #include "SynthNote.h"
 #include "Envelope.h"
 #include "LFO.h"
+#include "Portamento.h"
 #include "../Params/ADnoteParameters.h"
 #include "../Params/Controller.h"
+#include "WatchPoint.h"
 
 //Globals
 
@@ -36,13 +38,13 @@ class ADnote:public SynthNote
         /**Constructor.
          * @param pars Note Parameters
          * @param spars Synth Engine Agnostic Parameters*/
-        ADnote(ADnoteParameters *pars, SynthParams &spars,
+        ADnote(ADnoteParameters *pars, const SynthParams &spars,
                 WatchManager *wm=0, const char *prefix=0);
         /**Destructor*/
         ~ADnote();
 
         /**Alters the playing note for legato effect*/
-        void legatonote(LegatoParams pars);
+        void legatonote(const LegatoParams &pars);
 
         int noteout(float *outl, float *outr);
         void releasekey();
@@ -57,7 +59,7 @@ class ADnote:public SynthNote
         int  setupVoiceUnison(int nvoice);
         void setupVoiceDetune(int nvoice);
         void setupVoiceMod(int nvoice, bool first_run = true);
-
+        VecWatchPoint watch_be4_add,watch_after_add, watch_punch, watch_legato;
         /**Changes the frequency of an oscillator.
          * @param nvoice voice to run computations on
          * @param in_freq new frequency*/
@@ -76,12 +78,16 @@ class ADnote:public SynthNote
         /**Deallocate Note resources and voice resources*/
         void KillNote();
         /**Get the Voice's base frequency*/
-        inline float getvoicebasefreq(int nvoice) const;
+        inline float getvoicebasefreq(int nvoice, float adjust_log2 = 0.0f) const;
         /**Get modulator's base frequency*/
         inline float getFMvoicebasefreq(int nvoice) const;
         /**Compute the Oscillator's samples.
          * Affects tmpwave_unison and updates oscposhi/oscposlo*/
         inline void ComputeVoiceOscillator_LinearInterpolation(int nvoice);
+        /**Compute the Oscillator's samples.
+         * Affects tmpwave_unison and updates oscposhi/oscposlo
+         * @todo remove this declaration if it is commented out*/
+        inline void ComputeVoiceOscillator_SincInterpolation(int nvoice);
         /**Compute the Oscillator's samples.
          * Affects tmpwave_unison and updates oscposhi/oscposlo
          * @todo remove this declaration if it is commented out*/
@@ -94,7 +100,7 @@ class ADnote:public SynthNote
         /**Computes the Frequency Modulated Oscillator.
          * @param FMmode modulation type 0=Phase 1=Frequency*/
         inline void ComputeVoiceOscillatorFrequencyModulation(int nvoice,
-                                                              int FMmode);
+                                                              FMTYPE FMmode);
         //  inline void ComputeVoiceOscillatorFrequencyModulation(int nvoice);
         /**TODO*/
         inline void ComputeVoiceOscillatorPitchModulation(int nvoice);
@@ -111,7 +117,7 @@ class ADnote:public SynthNote
         ADnoteParameters &pars;
         unsigned char     stereo; //if the note is stereo (allows note Panning)
         float note_log2_freq;
-        float velocity, basefreq;
+        float velocity;
 
         ONOFFTYPE   NoteEnabled;
 
@@ -172,6 +178,9 @@ class ADnote:public SynthNote
             /* If the voice is enabled */
             ONOFFTYPE Enabled;
 
+            /* if AntiAliasing is enabled */
+            bool AAEnabled;
+
             /* Voice Type (sound/noise)*/
             int noisetype;
 
@@ -243,65 +252,65 @@ class ADnote:public SynthNote
             /* Wave of the Voice */
             float *FMSmp;
 
-            float FMVolume;
+            smooth_float FMVolume;
             float FMDetune;  //in cents
 
             Envelope *FMFreqEnvelope;
             Envelope *FMAmpEnvelope;
+
+            /********************************************************/
+            /*    INTERNAL VALUES OF THE NOTE AND OF THE VOICES     */
+            /********************************************************/
+
+            //pinking filter (Paul Kellet)
+            float pinking[14];
+
+            //the size of unison for a single voice
+            int unison_size;
+
+            //the stereo spread of the unison subvoices (0.0f=mono,1.0f=max)
+            float unison_stereo_spread;
+
+            //fractional part (skip)
+            float *oscposlo, *oscfreqlo;
+
+            //integer part (skip)
+            int *oscposhi, *oscfreqhi;
+
+            //fractional part (skip) of the Modullator
+            float *oscposloFM, *oscfreqloFM;
+
+            //the unison base_value
+            float *unison_base_freq_rap;
+
+            //how the unison subvoice's frequency is changed (1.0f for no change)
+            float *unison_freq_rap;
+
+            //which subvoice has phase inverted
+            bool *unison_invert_phase;
+
+            //unison vibratto
+            struct {
+                float  amplitude; //amplitude which be added to unison_freq_rap
+                float *step; //value which increments the position
+                float *position; //between -1.0f and 1.0f
+            } unison_vibratto;
+
+            //integer part (skip) of the Modullator
+            unsigned int *oscposhiFM, *oscfreqhiFM;
+
+            //used to compute and interpolate the amplitudes of voices and modullators
+            float oldamplitude, newamplitude,
+                  FMoldamplitude, FMnewamplitude;
+
+            //used by Frequency Modulation (for integration)
+            float *FMoldsmp;
+
+            //1 - if it is the fitst tick (used to fade in the sound)
+            char firsttick;
+
         } NoteVoicePar[NUM_VOICES];
 
-
-        /********************************************************/
-        /*    INTERNAL VALUES OF THE NOTE AND OF THE VOICES     */
-        /********************************************************/
-
-        //pinking filter (Paul Kellet)
-        float pinking[NUM_VOICES][14];
-
-        //the size of unison for a single voice
-        int unison_size[NUM_VOICES];
-
-        //the stereo spread of the unison subvoices (0.0f=mono,1.0f=max)
-        float unison_stereo_spread[NUM_VOICES];
-
-        //fractional part (skip)
-        float *oscposlo[NUM_VOICES], *oscfreqlo[NUM_VOICES];
-
-        //integer part (skip)
-        int *oscposhi[NUM_VOICES], *oscfreqhi[NUM_VOICES];
-
-        //fractional part (skip) of the Modullator
-        float *oscposloFM[NUM_VOICES], *oscfreqloFM[NUM_VOICES];
-
-        //the unison base_value
-        float *unison_base_freq_rap[NUM_VOICES];
-
-        //how the unison subvoice's frequency is changed (1.0f for no change)
-        float *unison_freq_rap[NUM_VOICES];
-
-        //which subvoice has phase inverted
-        bool *unison_invert_phase[NUM_VOICES];
-
-        //unison vibratto
-        struct {
-            float  amplitude; //amplitude which be added to unison_freq_rap
-            float *step; //value which increments the position
-            float *position; //between -1.0f and 1.0f
-        } unison_vibratto[NUM_VOICES];
-
-
-        //integer part (skip) of the Modullator
-        unsigned int *oscposhiFM[NUM_VOICES], *oscfreqhiFM[NUM_VOICES];
-
-        //used to compute and interpolate the amplitudes of voices and modullators
-        float oldamplitude[NUM_VOICES],
-              newamplitude[NUM_VOICES],
-              FMoldamplitude[NUM_VOICES],
-              FMnewamplitude[NUM_VOICES];
-
-        //used by Frequency Modulation (for integration)
-        float *FMoldsmp[NUM_VOICES];
-
         //temporary buffer
         float  *tmpwavel;
         float  *tmpwaver;
@@ -314,11 +323,8 @@ class ADnote:public SynthNote
         //interpolate the amplitudes
         float globaloldamplitude, globalnewamplitude;
 
-        //1 - if it is the fitst tick (used to fade in the sound)
-        char firsttick[NUM_VOICES];
-
-        //1 if the note has portamento
-        int portamento;
+        //Pointer to portamento if note has portamento
+        Portamento *portamento;
 
         //how the fine detunes are made bigger or smaller
         float bandwidthDetuneMultiplier;
