@@ -41,11 +41,12 @@ class Envelope
     private:
         int   envpoints;
         int   envsustain;    //"-1" means disabled
-        float envdt[MAX_ENVELOPE_POINTS]; //millisecons
+        float envdt[MAX_ENVELOPE_POINTS]; //seconds
         float envval[MAX_ENVELOPE_POINTS]; // [0.0f .. 1.0f]
         float envstretch;
         int   linearenvelope;
         int   mode;
+        bool  repeating;
 
         int   currentpoint;    //current envelope point (starts from 1)
         bool  forcedrelease;
