@@ -19,6 +19,8 @@
 #include "../DSP/SVFilter.h"
 #include "../DSP/AnalogFilter.h"
 #include "../DSP/FormantFilter.h"
+#include "../DSP/MoogFilter.h"
+#include "../DSP/CombFilter.h"
 #include <cassert>
 
 namespace zyn {
@@ -30,12 +32,11 @@ ModFilter::ModFilter(const FilterParams &pars_,
                            bool         stereo,
                            float        notefreq)
     :pars(pars_), synth(synth_), time(time_), alloc(alloc_),
-    baseQ(pars.getq()), baseFreq(pars.getfreq()),
     noteFreq(notefreq),
-    left(nullptr), 
+    left(nullptr),
     right(nullptr),
     env(nullptr),
-    lfo(nullptr)  
+    lfo(nullptr)
 {
     tracking = pars.getfreqtracking(notefreq);
     baseQ    = pars.getq();
@@ -54,7 +55,7 @@ ModFilter::~ModFilter(void)
     alloc.dealloc(left);
     alloc.dealloc(right);
 }
-        
+
 void ModFilter::addMod(LFO &lfo_)
 {
     lfo = &lfo_;
@@ -99,7 +100,7 @@ void ModFilter::update(float relfreq, float relq)
 void ModFilter::updateNoteFreq(float noteFreq_)
 {
     noteFreq = noteFreq_;
-    tracking = pars.getfreqtracking(noteFreq);
+    tracking = pars.getfreqtracking(noteFreq_);
 }
 
 void ModFilter::updateSense(float velocity, uint8_t scale,
@@ -108,7 +109,7 @@ void ModFilter::updateSense(float velocity, uint8_t scale,
     const float velScale = scale / 127.0f;
     sense = velScale * 6.0f * (VelF(velocity, func) - 1);
 }
-        
+
 void ModFilter::filter(float *l, float *r)
 {
     if(left && l)
@@ -125,6 +126,10 @@ static int current_category(Filter *f)
         return 1;
     else if(dynamic_cast<SVFilter*>(f))
         return 2;
+    else if(dynamic_cast<MoogFilter*>(f))
+        return 3;
+    else if(dynamic_cast<CombFilter*>(f))
+        return 4;
 
     assert(false);
     return -1;
@@ -135,7 +140,7 @@ void ModFilter::paramUpdate(Filter *&f)
     //Common parameters
     baseQ    = pars.getq();
     baseFreq = pars.getfreq();
-    
+
     if(current_category(f) != pars.Pcategory) {
         alloc.dealloc(f);
         f = Filter::generate(alloc, &pars,
@@ -147,6 +152,10 @@ void ModFilter::paramUpdate(Filter *&f)
         svParamUpdate(*sv);
     else if(auto *an = dynamic_cast<AnalogFilter*>(f))
         anParamUpdate(*an);
+    else if(auto *mg = dynamic_cast<MoogFilter*>(f))
+        mgParamUpdate(*mg);
+    else if(auto *cb = dynamic_cast<CombFilter*>(f))
+        cbParamUpdate(*cb);
 }
 
 void ModFilter::svParamUpdate(SVFilter &sv)
@@ -162,4 +171,15 @@ void ModFilter::anParamUpdate(AnalogFilter &an)
     an.setgain(pars.getgain());
 }
 
+void ModFilter::mgParamUpdate(MoogFilter &mg)
+{
+    mg.settype(pars.Ptype);
+    mg.setgain(pars.getgain());
+}
+
+void ModFilter::cbParamUpdate(CombFilter &cb)
+{
+    cb.settype(pars.Ptype);
+    cb.setgain(pars.getgain());
+}
 }
