@@ -45,20 +45,21 @@ class ModFilter
         void paramUpdate(Filter *&f);
         void svParamUpdate(SVFilter &sv);
         void anParamUpdate(AnalogFilter &an);
+        void mgParamUpdate(MoogFilter &mg);
+        void cbParamUpdate(CombFilter &cb);
+
 
-        
         const FilterParams &pars;  //Parameters to Pull Updates From
         const SYNTH_T      &synth; //Synthesizer Buffer Parameters
         const AbsTime      &time;  //Time for RT Updates
         Allocator          &alloc; //RT Memory Pool
 
 
-
-        float baseQ;    //filter sharpness
-        float baseFreq; //base filter frequency
+        smooth_float baseQ;    //filter sharpness
+        smooth_float baseFreq; //base filter frequency
         float noteFreq; //frequency note was initialized to
-        float tracking; //shift due to note frequency
-        float sense;    //shift due to note velocity
+        smooth_float tracking; //shift due to note frequency
+        smooth_float sense;    //shift due to note velocity
 
 
         Filter       *left; //left  channel filter
