@@ -28,7 +28,7 @@ namespace zyn {
 
 const rtosc::Ports Resonance::ports = {
     rSelf(Resonance),
-    rPaste,
+    rPasteRt,
     rToggle(Penabled,      rShort("enable"), rDefault(false),
             "resonance enable"),
     rToggle(Pprotectthefundamental, rShort("p.fund."), rDefault(false),
@@ -63,7 +63,7 @@ const rtosc::Ports Resonance::ports = {
             }
         } else {
             rtosc_arg_t args[N_RES_POINTS];
-            char        types[N_RES_POINTS+1] = {0};
+            char        types[N_RES_POINTS+1] = {};
             for(int i=0; i<N_RES_POINTS; ++i) {
                 args[i].f = o.Prespoints[i]/127.0;
                 types[i]  = 'f';
@@ -209,7 +209,7 @@ void Resonance::randomize(int type)
 
 void Resonance::zero(void)
 {
-    for(int i=0; i<N_RES_POINTS; ++i) 
+    for(int i=0; i<N_RES_POINTS; ++i)
         setpoint(i,64);
 }
 
