@@ -192,7 +192,7 @@ ui_handle_t GUI::createUi(Fl_Osc_Interface *osc, void *exit)
     {
         if (long long winId = atoll(embedId))
         {
-            // running as plugin
+            // embedId passed means running as plugin
             isPlugin = true;
             MasterUI::menu_mastermenu[11].hide(); // file -> nio settings
             MasterUI::menu_mastermenu[26].deactivate(); // misc -> switch interface mode
@@ -202,6 +202,8 @@ ui_handle_t GUI::createUi(Fl_Osc_Interface *osc, void *exit)
                 MasterUI::menu_mastermenu[13].hide(); // file -> exit
                 fl_embed(ui->masterwindow, winId);
             }
+#else
+            (void) winId; // Silences compiler warning.
 #endif
             ui->masterwindow->show();
         }
@@ -216,7 +218,7 @@ void GUI::destroyUi(ui_handle_t ui)
 
 #define BEGIN(x) {x,":non-realtime\0",NULL,[](const char *m, rtosc::RtData d){ \
     MasterUI *ui   = static_cast<MasterUI*>(d.obj); \
-    rtosc_arg_t a0 = {0}, a1 = {0}; \
+    rtosc_arg_t a0 = {}, a1 = {}; \
     if(rtosc_narguments(m) > 0) \
         a0 = rtosc_argument(m,0); \
     if(rtosc_narguments(m) > 1) \
@@ -574,9 +576,10 @@ void watch_lo(void)
 
 const char *help_message =
 "zynaddsubfx-ext-gui [options] uri - Connect to remote ZynAddSubFX\n"
-"    --help   print this help message\n"
-"    --no-uri run without a remote ZynAddSubFX\n"
-"    --embed  window ID [Internal Flag For Embedding Windows]\n"
+"    --help               print this help message\n"
+"    --no-uri             run without a remote ZynAddSubFX\n"
+"    --embed <window ID>  internal flag to let UI run in plugin mode\n"
+"                         NTK only: Embed master window into passed window\n"
 "\n"
 "    example: zynaddsubfx-ext-gui osc.udp://localhost:1234/\n"
 "      This will connect to a running zynaddsubfx instance on the same\n"
@@ -634,4 +637,4 @@ int main(int argc, char *argv[])
     lo_watch.join();
     return 0;
 }
-#endif
+#endif
\ No newline at end of file
